#pragma rtGlobals=1		// Use modern global access method.
#pragma hide=1

//////////////////////////////////////////////////////////////////////////////////
//Licence
//////////////////////////////////////////////////////////////////////////////////
//
//	If you use the 'Fit Assistant' to analyze data, please read and cite the following paper:
//	Surace and Interface Analysis, (2014), 46, 505-511
//	This publication contains more details on the fit function used by XPST.
//
//	XPST is a front-end to the IgorPro fitting engine. Although every possible effort was made 
//	to prevent serious bugs and wrong results, the author can not give any warranty for XPST as 
//	large parts actually depend on the underlying IgorPro code. 
//	Author: Dr. Martin Schmid
//
//////////////////////////////////////////////////////////////////////////////////




//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// 	Panel for the  CREATION OF A COEFFICIENT WAVE 
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// The coefficient wave W_coef is the wave, where Igor reads out starting values for fits and saves the 
// results of a fit. This wave is REALLY important.
//
//This code implements the following functionality
//
// 1. Select a spectrum (only waveform or x-y data)
// 2. Select a Peak Type
// 3. Set a cursor to the spectrum, each time a "add" button is pressed, its position is appended
//    to a new created coefficient wave,   ALWAYS named W_coef
// 
// This tool offers a way to set the starting values for a peak fit with the functions Gauss Singlet and Voigt
// functions defined in the files "PEFitFunctions.ipf"

//Add a menu entry
menu "XPST"
	"Fit Assistant", LaunchCursorPanel()
end

function LaunchCursorPanel()

	DoWindow CursorPanel   //this will write "1" into V_Flag if the window is open already
	if (V_Flag )
		DoWindow /F CursorPanel  // if it is already open, bring it to front
		return -1
	endif
	 //this should be in the initializing function, but then it gives problems with the setVariable elements
	   //remember this ... in order leave the project folder later on
	Execute "CursorPanel()"
	SVAR StartingFolder = root:STFitAssVar:ST_StartDirectory
	StartingFolder = GetDataFolder(1)
end


Window CursorPanel() : Panel

	//string subfolder = root:CP_workingFolder
	//Button showBut,pos={340,83},size={159,31},proc=displayWavesNewProjectPanel,title="Display Data   >>"
	initCursorPanel()
	PauseUpdate; Silent 1		// building window...
	NewPanel /K=2 /W=(125,57,1099,781) as "Fit Assistant ------  XPST 1.0"
	ModifyPanel cbRGB=(65534,65534,65534), fixedSize=1
	SetDrawLayer UserBack
	DrawRect 10,662,646,716
	DrawText 725,85,"# 0   =  no link"
	DrawText 710,659,"Convergence detection"
	DrawText 712,691,"coarse"
	DrawText 802,690,"fine"
	SetDrawEnv fstyle= 1
	DrawText 732,114,"Parameters to link:"
	SetDrawEnv linefgc= (39168,39168,39168)
	DrawLine 748,283,748,266
	SetDrawEnv linefgc= (39168,39168,39168)
	DrawLine 748,282,758,282
	
	TitleBox notify,pos={356,670},size={246,39},title="If you use this program, please cite:"
	TitleBox notify,labelBack=(65000,65000,65000),frame=0
	TitleBox notifyPart, pos={356,690}, title="Surf. Interface Anal. (2014) 46, 505-511"
	TitleBox notifyPart,labelBack=(65000,65000,65000),frame=0
	TitleBox notify1,pos={19,667},size={213,26},title="Parent Data Folder: \rCurrent Project: "
	TitleBox notify1,labelBack=(65000,65000,65000),frame=0
	
	TitleBox notify2,pos={19,697},size={213,26},title="Filename:"
	TitleBox notify2,labelBack=(65000,65000,65000),frame=0
	
	GroupBox PopUpGroup,pos={5,10},size={209,221},title="Load / Open Data and Fit"
	GroupBox PopUpGroup,fStyle=1
	GroupBox Frame2,pos={254,10},size={392,638},title="Select Peaks and Observe the Fit"
	GroupBox Frame2,fStyle=1
	GroupBox WorkGroup,pos={684,429},size={276,288},title="Fit Options",fStyle=1
	GroupBox ExitGroup,pos={6,449},size={210,197},title="Save and Export",fStyle=1
	GroupBox WorkGroup2,pos={703,30},size={223,267},title="Add",fStyle=1
	GroupBox WorkGroup3,pos={699,300},size={229,109},title="Remove",fStyle=1
	GroupBox WorkGroup4,pos={683,8},size={274,414},title="Peak Operations",fStyle=1
	GroupBox WorkGroup5,pos={7,274},size={208,122},title="Analyze",fStyle=1
	GroupBox WorkGroup6,pos={700,630},size={153,72},fStyle=1
	GroupBox WorkGroup8,pos={699,520},size={251,101},fStyle=1
	CheckBox RecordCheck,pos={713,560},size={102,39},proc=IterationRecordCheck,title="Record iterations \rin M_Iterates\r(debug a failed fit)"
	CheckBox RecordCheck,value= 0
	CheckBox RobustCheck,pos={714,532},size={93,14},proc=RobustCheck,title="Robust curve fit"
	CheckBox RobustCheck,value= 0
	CheckBox SuppressCheck,pos={843,529},size={95,26},proc=SuppressCheck,title="Suppress \rcurve-fit window"
	CheckBox SuppressCheck,value= 0
	CheckBox check0,pos={731,121},size={40,14},proc=STAreaLinkCheck,title="Area"
	CheckBox check0,value= 0
	CheckBox check1,pos={801,121},size={55,14},proc=STPositionLinkCheck,title="Position"
	CheckBox check1,value= 0
	CheckBox check2,pos={731,144},size={46,14},proc=STWidthLinkCheck,title="Width"
	CheckBox check2,value= 1
	CheckBox check4,pos={802,144},size={55,14},proc=STGLLinkCheck,title="GL ratio"
	CheckBox check4,value= 1
	CheckBox check3,pos={731,167},size={68,14},proc=STAsymLinkCheck,title="Asymmetry"
	CheckBox check3,value= 1
	CheckBox check5,pos={731,191},size={97,14},proc=STSOSLinkCheck,title="Multiplet-Splitting"
	CheckBox check5,value= 1
	CheckBox check6,pos={731,212},size={132,14},proc=STDoubletRatioLinkCheck,title="Multiplet Intensity Ratios"
	CheckBox check6,value= 1
	
	CheckBox InitCheck,pos={756,276},size={149,14},proc=STsetKeepInitCheck,title=" plus:  Results ---> Initial"
	CheckBox InitCheck,value= 1
	
	Button LoadNewBtn,pos={22,46},size={170,35},proc=CallSTNewProjectPanel,title="  New Fit Project  >>"
	Button ImportFitBut,pos={22,120},size={170,35},proc=STImportFit,title="Import Fit Project    >>"
	Button OpenProjectBtn,pos={22,83},size={170,35},proc=CallSTProjectPanel,title="Open Fit Project  >>"
	Button closeBut,pos={22,179},size={170,35},proc=destroyCursorPanel,title="Close"
		
	Button ReportViewerBtn,pos={24,305},size={170,35},proc=CallPeakViewer,title="View Numeric Results"
	Button FinePlotBtn,pos={23,344},size={170,35},proc=makeFinePlot,title="Permanent Plot (saves last)"
	
	Button FitBut,pos={391,607},size={120,32},proc=launchCurveFit,title="Start Fit"
	Button FitBut,help={"Starts the curve fit"},fStyle=1,fColor=(44000,52000,65500)
	Button addBut,pos={727,237},size={170,31},proc=ReadOutPosition,title=" <<    Add Peak        "
	Button addBut,fColor=(44000,52000,65500)
	Button DeleteBut,pos={728,361},size={170,31},proc=RemoveEntry,title=">>     Delete Peak     "
	Button DeleteBut,fColor=(44000,52000,65500)
	Button CleanBut,pos={26,480},size={170,35},proc=saveFit,title="Save"
	Button SaveAsBut,pos={25,518},size={171,35},proc=saveFitAs,title="Save As"

	Button ExportFitBut,pos={26,556},size={170,35},proc=STExportFit,title="Export Fit Project"
	Button resetBut,pos={25,594},size={170,35},proc=STsaveTemplate,title="Save as Template "
	Button UpDateBut,pos={267,573},size={120,32},proc=upDateFitDisplay,title="Show Initial Values"
	Button ConstraintBut,pos={391,573},size={120,32},proc=editDisplayWave,title="Peak Editor"
	Button ConstraintBut,fColor=(44000,52000,65500)
	Button UpDateButFin,pos={267,607},size={120,32},proc=upDateFitDisplayFinalVal,title="Show Final Values"
	Button ReUseBut,pos={515,573},size={120,32},proc=ReUseWCoef,title="Result ---> Initial"
	Button restorBut,pos={515,607},size={120,32},proc=RestoreSavedFit,title="Recover last Saved"
	
	Slider ToleranceControl,pos={721,661},size={91,19},proc=DisplayTol
	Slider ToleranceControl,limits={-1,-5,0},value= -3,live= 0,vert= 0,ticks= 0
	SetVariable InputFitMin,pos={702,452},size={139,16},title="Range:    x min"
	SetVariable InputFitMin,limits={0,2000,0},value= root:STFitAssVar:STFitMin
	SetVariable InputFitMax,pos={849,452},size={91,16},title="x max"
	SetVariable InputFitMax,limits={0,2000,0},value= root:STFitAssVar:STFitMax
	SetVariable InputSetLink,pos={725,51},size={146,16},proc=SetPeakToLink,title="Link next peak to  #"
	SetVariable InputSetLink,limits={0,3,1},value= root:STFitAssVar:STPeakToLink
	SetVariable InputRemoveLink,pos={732,329},size={146,16},proc=SetPeakToRemove,title="Remove peak # "
	SetVariable InputRemoveLink,limits={1,3,1},value= root:STFitAssVar:STPeakToRemove
	SetVariable InputMaxFitIterations,pos={704,480},size={129,16},title="Max. Iterations"
	SetVariable InputMaxFitIterations,limits={1,100,1},value=V_FitMaxIters


	Display/W=(264,32,634,302)/HOST=#
	RenameWindow #,guiCursorDisplay
	SetActiveSubwindow ##

	Display/W=(263,305,633,557)/HOST=#
	RenameWindow #,guiCursorDisplayFit
	SetActiveSubwindow ##
	
	if (!Exists("KeepCheckIndicate"))
		Variable /G root:STFitAssVar:KeepCheckIndicate = 0
	//	CheckBox KeepCheck,value= 0
	else 
	//	CheckBox LoadCheck value = 1
		NVAR keepIndicate = root:STFitAssVar:KeepCheckIndicate
		keepIndicate = 1
		//CheckBox KeepCheck, value = 1 
	endif

	DeActivateButtons(1)
	

	
EndMacro

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//         initialize variables
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function initCursorPanel()
	//SVAR name = root:CP_workingFolder
	//SetDataFolder name
	
	NewDataFolder /o root:STFitAssVar
	
	Variable /G root:STFitAssVar:CleanCheckIndicate = 0
	Variable /G root:STFitAssVar:CP_IndepCheck = 0  //this variable will go later on, use the ones below instead
	Variable /G root:STFitAssVar:AreaLink=0
	Variable /G root:STFitAssVar:PositionLink=0
	Variable /G root:STFitAssVar:WidthLink=1
	Variable /G root:STFitAssVar:GLLink =1
	Variable /G root:STFitAssVar:AsymLink=1
	Variable /G root:STFitAssVar:SOSLink=1
	Variable /G root:STFitAssVar:DoubletRatioLink=1
	Variable /G root:STFitAssVar:ST_NumPeaks = 0
	Variable /G root:STFitAssVar:saved = 0
	Variable /G root:STFitAssVAr:savedLast = 0
	Variable /G root:STFitAssVar:projectStarted = 0
	Variable /G root:STFitAssVar:useTemplate = 0
	Variable /G root:STFitAssVar:STFitMin= 0
	Variable /G root:STFitAssVar:STFitMax= 0
	Variable /G root:STFitAssVar:STPeakToLink= 0
	Variable /G root:STFitAssVar:STPeakToRemove = 0
	Variable /G root:STFitAssVar:IterationsStart = 50  
	Variable /G root:STFitAssVar:AreaLinkFactorLow = 0.4
	Variable /G root:STFitAssVar:AreaLinkFactorHigh = 0.6
	Variable /G root:STFitAssVar:PositionLinkOffsetMax = 1.5
	Variable /G root:STFitAssVar:PositionLinkOffsetMin = 1.5
	Variable /G root:STFitAssVar:V_FitTol = 0.001
	Variable /G root:STFitAssVar:savedNumPeaks //for saving and recovery
	Variable /G root:STFitAssVar:keepInitConfiguration = 0
	Variable /G root:STFitAssVar:EpsilonFactor = 1
		
	string /G root:STFitAssVar:PR_nameWorkWave
	string /G root:STFitAssVar:PR_PeakType = "Singlet"
	string /G root:STFitAssVar:PR_Background = "SK"
	string /G root:STFitAssVar:PR_PeakTypeTemp  ="Singlet"
	string /G root:STFitAssVar:PR_XRawDataCursorPanel 
	string /G root:STFitAssVar:PR_FitRawData 
	string /G root:STFitAssVar:PR_CoefWave = "W_coef"
	string /G root:STFitAssVar:PR_FitWave
	string /G root:STFitAssVar:CP_workingFolder 
	string /G root:STFitAssVar:ST_StartDirectory 
	string /G root:STFitAssVar:ProjectName
	string /G root:STFitAssVar:TemplateName = ""
	string /G  root:STFitAssVar:nextProjectToOpen=""
	make /d /n=(5,3) /o myColors
	myColors[0][q] = 0
	myColors[1][0] = 65535
	myColors[1][1] = 0
	myColors[1][2] = 0
	myColors[2][0] = 55000
	myColors[2][1] = 60000
	myColors[2][2] = 64000
	myColors[3][0] = 51000
	myColors[3][1] = 56000
	myColors[3][2] = 60000
	myColors[4][0] = 65000
	myColors[4][1] = 55000
	myColors[4][2] = 45000
	
	duplicate myColors root:STFitAssVar:myColors
	killwaves /z myColors
	
	
end

Function SetPeakToLink (ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum	// value of variable as number
	String varStr		// value of variable as string
	String varName	// name of variable
	
	NVAR numPeaks = root:STFitAssVar:ST_NumPeaks
	NVAR peakToLink = root:STFitAssVar:STPeakToLink
	
	SetVariable InputSetLink, win=CursorPanel, limits={0,numPeaks,1}
End


Function SetPeakToRemove (ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum	// value of variable as number
	String varStr		// value of variable as string
	String varName	// name of variable
	
	NVAR numPeaks = root:STFitAssVar:ST_NumPeaks
	NVAR peakToLink = root:STFitAssVar:STPeakToRemove
	
	SetVariable InputRemoveLink, win=CursorPanel, limits={0,numPeaks,1}
End

Function DisplayTol(name, value, event) : SliderControl
	String name	// name of this slider control
	Variable value	// value of slider
	Variable event	// bit field: bit 0: value set; 1: mouse down, 
	
	NVAR tolerance = root:STFitAssVar:V_FitTol				
	tolerance =1*10^(value)
end


Function SetEpsilonFactor(name, value, event) : SliderControl
	String name	// name of this slider control
	Variable value	// value of slider
	Variable event	// bit field: bit 0: value set; 1: mouse down, 
	
	NVAR factor = root:STFitAssVar:EpsilonFactor				
	factor =1*10^(value+5)
end


function saveFitAs(ctrlName):ButtonControl
	string ctrlName
	
	SVAR parentDir = root:STFitAssVar:ST_StartDirectory
	SVAR projectName = root:STFitAssVar:projectName
	NVAR saved = root:STFitAssVar:saved             //check if it is saved at all (at any point)
	NVAR savedLast = root:STFitAssVar:savedLast   //check if the very last action was saved too
	
	NVAR fitMin = root:STFitAssVar:STFitMin
	NVAR fitMax = root:STFitAssVar:STFitMax
	NVAR numPeaks = root:STFitAssVar:ST_NumPeaks

	NVAR ProjectStarted = root:STFitAssVar:projectStarted
	NVAR ProjectSaved = root:STFitAssVar:savedLast
	
	SVAR projectPath = root:STFitAssVar:CP_workingFolder
	
	NVAR tolerance = V_FitTol
	NVAR iterations = V_FitMaxiters
	NVAR fitOptions = V_FitOptions
	SVAR nameWorkWave = root:STFitAssVar:PR_nameWorkWave
	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR background = root:STFitAssVar:PR_Background
	SVAR peakTypeTemp = root:STFitAssVar:PR_PeakTypeTemp
	SVAR XRawWave = root:STFitAssVar:PR_XRawDataCursorPanel
	SVAR FitRawData = root:STFitAssVar:PR_FitRawData
	SVAR CoefWave = root:STFitAssVar:PR_CoefWave
	SVAR fitWave = root:STFitAssVar:PR_FitWave
	
	wave /t setup = STsetup
	if (DimSize(setup,0) == 0)
		DoAlert 0, "You can not just save an empty project, please add some peaks and/or do a fit."
		return -1
	endif 
	string CurrentName = GetDataFolder(0)
	string CurrentPath = GetDataFolder(1)
	projectName=CurrentName
	string newName
	Prompt newName, "Enter a new project name "		// Set prompt for x param
	DoPrompt "Save project as", newName
	if (V_Flag)
		return -1								// User canceled
	endif
	
	variable legal = !GrepString(newName,"[^[:alnum:]_+-]") 
	variable letter = GrepString(newName,"[[:alpha:]]") 
	variable number =  GrepString(newName,"[[:digit:]]")
	variable nameOkay = legal && ( (letter && number) || (letter && !number))
	variable invalid = ! nameOkay // !nameOkay //look for not alpha-numeric characters
	
	if (invalid)
		DoAlert 0, "Process aborted because of an invalid name.\rThe project name may only contain '+', ' _', '-', letters, and digits.\r \rPlease provide a new project name."
		return -1
	endif
	
	SetDataFolder parentDir
	if (DataFolderExists(newName))
		DoAlert 0, "Process aborted because of an invalid name.\rThis name already exists, pick another one."
		SetDataFolder CurrentPath
		return -1
	endif
	//switch the names
	DuplicateDataFolder $CurrentPath, oldTemp
	RenameDataFolder $CurrentPath, $newName
	string substitute = ReplaceString("'", projectName, "")
	RenameDataFolder oldTemp, $substitute

	
	SetDataFolder CurrentPath//projectName   //this is actually the original name, now carried by the duplicate
	if (saved == 1)
		
		////RestoreSavedFit("CalledBy:saveFitAs")
		DuplicateDataFolder :FitTemp, Setup
		//NewDataFolder /o :Setup
		//duplicate /o :FitTemp:STsetup :Setup:STsetup 
		//duplicate /o :FitTemp:selSTsetup :Setup:selSTsetup
		//duplicate /o :FitTemp:Numerics :Setup:Numerics 
		//duplicate /o :FitTemp:selNumerics :Setup:selNumerics
		//DuplicateDataFolder :FitTemp:Peaks, :Peaks
		killwaves /z STsetup, selSTsetup, Numerics, selNumerics
		killwaves /z W_Coef, Min_Limit, Max_Limit, epsilon, hold, W_Sigma, T_Constraints, InitializeCoef
		killvariables /z V_numNaNs, V_numINFs, V_npnts, V_nterms, V_nheld, V_startRow, V_endRow, V_startCol, V_endCol
		killvariables /z V_startLayer, V_endLayer, V_startChunk, V_endChunk, V_FitTol, V_FitMaxIters, V_FitOptions
		killdataFolder :FitTemp
		//now make a hard-copy of all relevant variables, use this, if the project is re-opened to initialize everything properly
		
		make /t /n=(100,2) /o FitControls    //more points than needed right now - maybe useful in the future
		FitControls[p][q] = "0"  
		FitControls[0][0] = num2str(fitMin)
		FitControls[1][0] = num2str(fitMax)
		FitControls[2][0] = num2str(tolerance)
		FitControls[3][0] = num2str(iterations)
		FitControls[4][0]= num2str(fitOptions)
		FitControls[5][0] = ProjectName
		FitControls[6][0] = nameWorkWave
		FitControls[7][0] = XRawWave
		FitControls[8][0] = peakType
		FitControls[9][0] = peakTypeTemp
		FitControls[10][0] = background
		FitControls[11][0] = num2str(numPeaks)
		duplicate /o FitControls :Setup:FitControls
		killwaves /z FitControls
		SetDataFolder parentDir
	else
		SetDataFolder parentDir
		killdataFolder :$projectName
		//leave the subfolder and delete it ... nothing has been saved anyway
	endif

	
	SetDataFolder newName //now we are still in the old datafolder /// however with a new name
	projectName = newName
	ProjectPath = GetDataFolder(1)
	saveFit("fromTheSaveAsButton")
	
	string displayString = "" 
	sprintf displaystring,"Parent Data Folder:\t%s \rCurrent Project:\t\t\t%s ", parentDir, ProjectName
	TitleBox notify1, win=CursorPanel, title=displayString
	sprintf displaystring,"Filename:\t\t\t\t\t\t%s ", nameWorkWave
	TitleBox notify2, win=CursorPanel, title=displayString
	// 1. duplicate the project Folder with a new name
	// 2. set the current working directory there
	// the "old" project is either dismissed, if it was not saved previously
	//                        or will be set to the last save-point
	// all auxiliary waves etc. have to be purged from the old directory
	
End



end

// read the peak type
function ReadType_CursorPanel(ctrlName,popNum,popStr): PopupMenuControl
	string ctrlName
	variable popNum
	String popStr
	//if ( CheckLocation() )
	//	PopupMenu TypePop , mode = 1, popvalue= " "
	//	return 1
	//endif
	SVAR value = root:STFitAssVar:PR_PeakTypeTemp

	strswitch(popStr)
		case "Singlet":
			value = "Singlet"
		break		
		case "Multiplet":
			value = "Multiplet"
		break
	default:
		value = popStr
	endswitch

      combinePeakBackground()
	ReDoPeaks()                              
end

function ReDoPeaks()
	print "This function conserves the background after a change of the peaks"
	//if there is already a valid fit setup present, inform the user that a change in peak type will destroy his peaks, the background will stay as it is
end

function ReDoBackground()
       print "This function conserves the peaks after a change of the background"
       //if there is already a valid fit setup, conserve the peak information
end

function combinePeakBackground()
//this is basically a matrix that associates a certain peak function with a combination of peak and background
SVAR peak = root:STFitAssVar:PR_PeakTypeTemp
SVAR background = root:STFitAssVar:PR_Background
SVAR PeakFunc = root:STFitAssVar:PR_PeakType

	strswitch(background)     //later on, there could be more backgrounds
		case "SK":
			strswitch(peak)
				case "Singlet":
					PeakFunc = "SingletSK"
				break
				case "Multiplet":
					PeakFunc = "MultiSK"
				break
				default:
					PeakFunc = ""
					Print "Sorry, not implemented yet"
					return 1
			endswitch	
		break		
		default:
			PeakFunc = ""
			print "This combination is not implemented yet"
			return 1
	endswitch



end


// read the x-axis of the raw data
function initXWavePopUp(ctrlName,popNum,popStr): PopupMenuControl
	string ctrlName
	variable popNum
	String popStr
	//if ( CheckLocation() )
	//	PopupMenu WavePop2 , mode = 1, popvalue= "_calculated_ "
	//	return 1
	//endif
	SVAR value = root:STFitAssVar:PR_XRawDataCursorPanel
	value = popStr                          
end


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Check box for multiple runs

function IterationRecordCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	if (!Exists("V_FitOptions"))
		Variable /G V_FitOptions = 0
	endif
	NVAR value = V_FitOptions
	
	if (checked ==1)
		value = 8
		CheckBox RobustCheck value = 0
		CheckBox SuppressCheck value =0
	else
		value = 0
	endif	
end

function STAreaLinkCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	NVAR value = root:STFitAssVar:AreaLink
	NVAR peakToLink = root:STFitAssVar:STPeakToLink
	
	if (checked ==1)
		value = 1
		//call the procedure to input the variables
		if (peakToLink != 0)
			Execute "GetAreaFactors()"
		else
			DoAlert 0, "If you link to peak '0', the resulting peak will be independent!\r\rSelect peak #1 or #2 etc. to actually do a link"
			CheckBox check0,value= 0
			value = 0
		endif
	else
		value = 0
	endif	
end

function STPositionLinkCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	NVAR value = root:STFitAssVar:PositionLink
	NVAR peakToLink = root:STFitAssVar:STPeakToLink
	
	if (checked ==1)
		value = 1
		if (peakToLink != 0)
			Execute "GetOffsetInterval()"
		else
			DoAlert 0, "If you link to peak '0', the resulting peak will be independent!\r Select peak #1 or #2  or .... etc. to actually do a link"
			CheckBox check1,value= 0
			value = 0
		endif
		
		
	else
		value = 0
	endif	
end

function STWidthLinkCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	NVAR value = root:STFitAssVar:WidthLink
	
	if (checked ==1)
		value = 1
	else
		value = 0
	endif	
end

function STGLLinkCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	NVAR value = root:STFitAssVar:GLLink
	
	if (checked ==1)
		value = 1
	else
		value = 0
	endif	
end

function STAsymLinkCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	NVAR value = root:STFitAssVar:AsymLink
	
	if (checked ==1)
		value = 1
	else
		value = 0
	endif	
end

function STSOSLinkCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	NVAR value = root:STFitAssVar:SOSLink
	
	if (checked ==1)
		value = 1
	else
		value = 0
	endif	
end

function STDoubletRatioLinkCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	NVAR value = root:STFitAssVar:DoubletRatioLink
	
	if (checked ==1)
		value = 1
	else
		value = 0
	endif	
end

function STsetKeepInitCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	NVAR value = root:STFitAssVar:keepInitConfiguration
	
	if (checked ==1)
		value = 0   //this assignment is a bit screwed
	else
		value = 1 
	endif	

end

function RobustCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	if (!Exists("V_FitOptions"))
		Variable /G V_FitOptions = 0
	endif
	NVAR value = V_FitOptions
	if (checked ==1)
		value = 1
		CheckBox RecordCheck value = 0
		CheckBox SuppressCheck value = 0
	else
		value = 0
		
	endif	
end

function SuppressCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	if (!Exists("V_FitOptions"))
		Variable /G V_FitOptions = 0
	endif
	NVAR value = V_FitOptions
	if (checked ==1)
		value = 4
		CheckBox RecordCheck value = 0
		CheckBox RobustCheck value = 0
	else
		value = 0
		
	endif	
end

function KeepCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	NVAR value = root:STFitAssVar:KeepCheckIndicate
	value = checked
end

function CleanCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	NVAR value = root:STFitAssVar:CleanCheckIndicate
	value = checked
end

function IndepCheck(ctrlName, checked):CheckBoxControl
	string ctrlName
	variable checked
	CheckLocation()
	NVAR value = CP_IndepCheck
	value = checked
end

Function SetIterations(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum	// value of variable as number
	String varStr		// value of variable as string
	String varName	// name of variable
	NVAR value = V_FitOptions
	value = varNum
End

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Button functions
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function MakeFinePlot(ctrlName):ButtonControl
	string ctrlName
	SVAR rawY = root:STFitAssVar:PR_nameWorkWave
	SVAR rawX = root:STFitAssVar:PR_XrawDataCursorPanel
	SVAR peakType = root:STFitAssVar:PR_PeakType
	
	wave /t setup = STsetup
	variable numPeaks = 0
	variable i,j
	variable numWaves
	string WavesInFolder, item
	//Do the following things only, if a fit wave is there
	string wavesWithFit = WaveList("fit_*",";","")
	variable FitThere = ItemsInList(wavesWithFit)
	if ( FitThere == 0)
		DoAlert 0, "There seems to be no fit wave. Did you already do a fit? Press 'Start Fit' and retry then."
		return -1
	endif
	
	
	saveFit("void")
	updateFitDisplayFinalVal("fromMakeFinePlot")
	variable numSubFolders
	string dynamicName
	string parent = GetDataFolder(1)
	
	if ( DataFolderExists("PermanentPlot") )
		NewDataFolder /o /s OldPermanentPlots   //that makes a new one or attaches to the old one, if it is already there
		numSubFolders = CountObjects("",4)
		dynamicName = "PlotSet"+ num2str(numSubFolders)
		SetDataFolder parent
		RenameDataFolder :PermanentPlot, $dynamicName
		MoveDataFolder :$dynamicName, :OldPermanentPlots
			
	endif
	//NewDataFolder /o PermanentPlot
	
	SetDataFolder :Peaks
	WavesInFolder = WaveList("*",";","")  
	numWaves = itemsInList(WavesInFolder)

	SetDataFolder parent
	
	DuplicateDataFolder :Peaks, PermanentPlot
	//for (i = 0; i < numWaves; i += 1)
	//	item = StringFromList(i,WavesInFolder)	
	//	duplicate /o :Peaks:$item :PermanentPlot:$item
	//endfor

	string fitName = "fit_" + rawY
	duplicate /o $rawY :PermanentPlot:$rawY
	duplicate /o $fitName :PermanentPlot:$fitName
	
	strswitch(rawX)
		case "":
		break
		case "_calculated_":
		break
		default:
			duplicate /o $rawX :PermanentPlot:$rawX
			numPeaks -= 1
		break
	endswitch
	
	SetDataFolder :PermanentPlot
	//now do all the plotting
	WavesInFolder = WaveList("*",";","")  
	variable show = ItemsInList(WavesInFolder)
	strswitch(peakType)
			case "MultiSK":
				numPeaks += (ItemsInList(WavesInFolder)-3)/5
			break
			default:
				numPeaks += ItemsInList(WavesInFolder) - 3  //raw,bg,and fit
			break
	endswitch
	
	strswitch(rawX)
		case "":
			Display /K=1 $rawY
		break
		case "_calculated_":
			Display /K=1 $rawY
		break
		default:
			Display $rawY vs $rawX			
		break
	endswitch
	
	ModifyGraph mode=3,marker=19,msize=1.5
	ModifyGraph rgb=(47872,47872,47872)
	//ModifyGraph
	string TagName, PeakTag
	string TempString
	variable TagPosition
	variable index
	for ( i = 0; i < numPeaks; i +=1)
		
		strswitch(peakType)
			case "MultiSK":
				WavesInFolder = WaveList("m"+num2str(i+1)+"p*",";","")
				
				for (j = 0; j < 4; j += 1 )
					item = StringFromList(j,WavesInFolder)
					index = 15*i+5
					if ( (j == 0 && str2num(setup[index][2]) != 0) || (str2num(setup[index+6+3*(j-1)][2]) != 0 && j != 0) )  //append only those waves which actually contain something
						AppendToGraph $item
						ModifyGraph rgb($item)=(21760,21760,21760), lsize($item)=1, lstyle($item)=2
					endif
				endfor
				WavesInFolder = WaveList("m"+num2str(i+1)+"_*",";","")   //this is necessary for tagging (later on)
			break
			default:
				WavesInFolder = WaveList("p"+num2str(i+1)+"_*",";","")
			break
		endswitch
		item = StringFromList(0,WavesInFolder)
		AppendToGraph $item
		ModifyGraph rgb($item)=(26112,26112,26112), lsize($item)=1
			
		duplicate /o $item ItemTemp
		wave itemTemp = ItemTemp
		TempString = "bg_" + rawY
		wave bgWave = $TempString
		itemTemp -= bgWave  //this is the corrected wave without background
		//this is necessary, because a steep background might screw up the tag positions
			
		WaveStats /Q itemTemp                       // get the location of the maximum
		TagName = "tag"+num2istr(i)           //each tag has to have a name
		PeakTag = num2istr(i+1)                 // The tag displays the peak index
		TagPosition = V_maxloc                 // and is located at the maximum
		Tag  /C /N= $TagName  /F=0 /L=1  /Y =2.0  $item, TagPosition ,PeakTag    // Now put the tag there
	endfor
	killwaves /z ItemTemp
	WavesInFolder = WaveList("fit_*",";","")  
	item = StringFromList(0,WavesInFolder)
	AppendToGraph $item
	ModifyGraph rgb($item)=(0,0,0)  
	SetAxis /A /R bottom	
	WavesInFolder = WaveList("bg_*",";","")
	item = StringFromList(0,WavesInFolder)
	AppendToGraph $item
	ModifyGraph rgb($item)=(0,0,0),lstyle($item)=2
	ModifyGraph mirror=2,minor(bottom)=1
	Label left "\\f01 intensity"
	Label bottom "\\f01  binding energy (eV)"
	//
	SetDataFolder parent
end



function saveFit(ctrlName):ButtonControl
string ctrlName
//this function sets the global variable saved to 1
//    -  moves the fit setup to FitTemp  (from there it can be restored)
NVAR saved = root:STFitAssVar:saved
NVAR savedLast = root:STFitAssVar:savedLast
NVAR savedNumPeaks = root:STFitAssVar:savedNumPeaks
SVAR rawY = root:STFitAssVar:PR_nameWorkWave
NVAR numPeaks = root:STFitAssVar:ST_NumPeaks

wave /t setup = STsetup
if (DimSize(setup,0) == 0)
	DoAlert 0, "You can not just save an empty project, please add some peaks and/or do a fit."
	return -1
endif 



saved = 1 //okay, now the rest of the program knows
savedLast = 1
makeFitControls()
duplicate /o $"STsetup" :FitTemp:$"STsetup"
duplicate /o $"selSTsetup" :FitTemp:$"selSTsetup"
duplicate /o $"Numerics" :FitTemp:$"Numerics"
duplicate /o $"selNumerics" :FitTemp:$"selNumerics"
duplicate /o $"FitControls" :FitTemp:$"FitControls"
duplicate /o $rawY :FitTemp:$rawY

//variable /G :FitTemp:SavedNumPeaks
savedNumPeaks = numPeaks //remember the number of peaks
end


function RestoreSavedFit(ctrlName):ButtonControl
	string ctrlName
	NVAR numPeaks = root:STFitAssVar:ST_NumPeaks
	NVAR peakToExtract = root:STFitAssVar:STPeakToRemove
	NVAR peakToLink = root:STFitAssVar:STPeakToLink
	NVAR savedNumPeaks =  root:STFitAssVar:SavedNumPeaks
	NVAR savedLast = root:STFitAssVar:savedLast
	NVAR saved = root:STFitAssVar:saved
	SVAR rawY = root:STFitAssVar:PR_nameWorkWave
	if (saved ==0)
		DoAlert 0, "You have not saved the fit so far ... nothing to recover"
		return 1
	endif

	savedLast = 1 //this change has not beed saved yet
	duplicate /o :FitTemp:$"STsetup" $"STsetup" 
	duplicate /o :FitTemp:$"selSTsetup" $"selSTsetup"
	duplicate /o :FitTemp:$"Numerics" $"Numerics"
	duplicate /o :FitTemp:$"selNumerics" $"selNumerics"
	
	string currentFolder = GetDataFolder(1)
	setDataFolder :FitTemp
	if (WaveExists(rawY))
		setDataFolder currentFolder
		duplicate /o :FitTemp:$rawY $rawY
	else
		setDataFolder currentFolder
	endif
	
	numPeaks = getPeakNumber()// savedNumPeaks
	string ListOfCurves = TraceNameList("CursorPanel#guiCursorDisplayFit",";",1)
	variable numberCurves = ItemsInList(ListOfCurves)
	
	KillDataFolder /z :Peaks  //if it exists from a previous run, kill it
	//now recreate it, so everything is updated             
	NewDataFolder /O :Peaks
	variable i
	//update the graph, remove everything	
	for (i =1; i<numberCurves; i +=1)
		RemoveFromGraph /W=CursorPanel#guiCursorDisplayFit /Z $"#"+num2istr(numberCurves-i)
	endfor
	
	
	setup2waves()
	updateFitDisplayFinalVal("void")
	
	peakToExtract = numPeaks
	peakToLink = 0
	SetVariable InputSetLink, limits={0,numPeaks,1}, value = peakToLink
	SetVariable InputRemoveLink,limits={1,numPeaks,1}, value = peakToExtract
end
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Re-usage of W_coef

function ReUseWCoef(ctrlName):ButtonControl
	string ctrlName
	wave /t source = STsetup
	NVAR savedLast = root:STFitAssVar:savedLast
	savedLast = 0 //this change has not beed saved yet
	variable length = DimSize(source,0)
	variable i
	
	for ( i = 0; i < length; i += 1 )
		if (numtype(str2num(source[i][2])) == 0)
			source[i][3] = source[i][2]
		endif
	endfor
	//just copy W_coef as InitializeCoef
	duplicate /o W_coef InitializeCoef

end


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Clean up button
function PlotAndSort(ctrlName):ButtonControl
	string ctrlName
	CheckLocation()
	//disable this button after it has been used once
	//CheckBox KeepCheck disable = 0
	//Button CleanBut disable = 2
	//Button closeBut, title="Close"
	//Titlebox notify, title="Results saved; use Igor's Data Browser to view\rthose waves"
	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR RawYWave = root:STFitAssVar:PR_nameWorkWave
	SVAR coefWave = root:STFitAssVar:PR_CoefWave
	SVAR RawXWave = root:STFitAssVar:PR_XRawDataCursorPanel
	//NVAR saved = root:STFitAssVar:saved                                         //the code should remember, if the last action was to save, so the respective waves do not get erased
	//saved = 1                                                         // by the function selectiveDel()
	string fitWave = "fit_"+RawYWave
	string oldFitWave = fitWave + "old"
	NVAR clean = root:STFitAssVar:CleanCheckIndicate
	variable newFolder = 1
	
	string wavesWithFit = WaveList("fit_*",";","")
	variable FitThere = ItemsInList(wavesWithFit)
	if ( FitThere == 0)
		DoAlert 0, "There seems to be no fit wave. Did you already do a fit? Press 'Start Fit' and retry then."
		return -1
	endif
	
	
	RemoveFromGraph /W=CursorPanel#guiCursorDisplay /Z $"#1"
	PlotPeaks(coefWave,RawYWave,fitWave,peakType,RawXWave,newFolder)
	
	if (Exists(oldFitWave) )
		killwaves $oldFitWave 
	endif
	//once, this function was called, it deletes the old fit wave
	
end


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Close Button:  kill global variables after usage

function destroyCursorPanel(ctrlName) : ButtonControl
	string ctrlName
	string LastObjectName = WinList("*", "", "WIN:64")  //get the name of the target windows
	String cmd
	
	NVAR fitMin = root:STFitAssVar:STFitMin
	NVAR fitMax = root:STFitAssVar:STFitMax
	
	NVAR saved = root:STFitAssVar:saved
	NVAR ProjectStarted = root:STFitAssVar:projectStarted
	
	NVAR SavedLast = root:STFitAssVar:savedLast
	
	NVAR tolerance = V_FitTol
	NVAR iterations = V_FitMaxiters
	NVAR fitOptions = V_FitOptions
	NVAR numPeaks = root:STFitAssVar:ST_NumPeaks
	NVAR savedNumPeaks = root:STFitAssVar:savedNumPeaks
	
	SVAR projectName = root:STFitAssVar:projectName
	SVAR StartingDirectory = root:STFitAssVar:ST_StartDirectory

	SVAR projectPath = root:STFitAssVar:CP_workingFolder
	SVAR nameWorkWave = root:STFitAssVar:PR_nameWorkWave
	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR background = root:STFitAssVar:PR_Background
	SVAR peakTypeTemp = root:STFitAssVar:PR_PeakTypeTemp
	SVAR XRawWave = root:STFitAssVar:PR_XRawDataCursorPanel
	SVAR FitRawData = root:STFitAssVar:PR_FitRawData
	SVAR CoefWave = root:STFitAssVar:PR_CoefWave
	SVAR fitWave = root:STFitAssVar:PR_FitWave
	
	string pathToSaved = projectName + "FitTemp:"
	string pathSTsetup = pathToSaved + "STsetup"
	string pathSelSTsetup = pathToSaved + "selSTsetup"
	
	variable restart = 0
	
	strswitch(ctrlName)
	case "restart":
		restart = 1
		break
	endswitch
		
	wave /t STsetup = STsetup
	variable WorkedInProject = DimSize(STsetup,0)	
		
	if (WorkedInProject != 0 && SavedLast == 0 && saved == 0 && restart == 0) //(ProjectStarted == 1 && SavedLast == 0 && saved == 0 && restart == 0)
		DoAlert 1, "There are unsaved changes in the current project. Do you really want to close?"
		if (V_Flag ==2) //user pressed no
			return -1
		endif
	elseif (WorkedInProject != 0 && SavedLast==0 && saved==1 && restart == 0)
		DoAlert 1, "You did not save the most recent changes in this project - however a previous version was saved. Do you want to close and keep the previous version?"
		if (V_Flag ==2)
			return -1
		endif
	endif

	DoWindow /K PeakEditor   //in case it is still open
	DoWindow /K PeakViewer
	DoWindow  /k CursorPanel
	DoWindow /K STOpenProjectPanel
	DoWindow /K STNewProjectPanel
		
	if (saved == 0)  // no one saved the project
		SetDataFolder StartingDirectory
		strswitch (projectName)   //well, if there is an "" as project name, igor deletes everything
			case "" :
				break
			default:
				killdatafolder $projectName
			break
		endswitch
	else
		
		numPeaks = savedNumPeaks
		makeFitControls()
		//NewDataFolder /o :Setup
		
		DuplicateDataFolder :FitTemp, Setup
		//duplicate /o :FitTemp:STsetup :Setup:STsetup 
		//duplicate /o :FitTemp:selSTsetup :Setup:selSTsetup
		//duplicate /o :FitTemp:Numerics :Setup:Numerics 
		//duplicate /o :FitTemp:selNumerics :Setup:selNumerics
		
		//DuplicateDataFolder :FitTemp:Peaks, :Peaks
		killwaves /z STsetup, selSTsetup, Numerics, selNumerics
		killwaves /z W_Coef, Min_Limit, Max_Limit, epsilon, hold, W_Sigma, T_Constraints, InitializeCoef
		killvariables /z V_numNaNs, V_numINFs, V_npnts, V_nterms, V_nheld, V_startRow, V_endRow, V_startCol, V_endCol
		killvariables /z V_startLayer, V_endLayer, V_startChunk, V_endChunk, V_FitTol, V_FitMaxIters, V_FitOptions
		
		//now make a hard-copy of all relevant variables, use this, if the project is re-opened to initialize everything properly
		
		//make /t /n=(100,2) /o FitControls    //more points than needed right now - maybe useful in the future
		//FitControls[p][q] = "0"  
		//FitControls[0][0] = num2str(fitMin)
		//FitControls[1][0] = num2str(fitMax)
		//FitControls[2][0] = num2str(tolerance)
		//FitControls[3][0] = num2str(iterations)
		//FitControls[4][0]= num2str(fitOptions)
		//FitControls[5][0] = projectName
		//FitControls[6][0] = nameWorkWave
		//FitControls[7][0] = XRawWave
		//FitControls[8][0] = peakType
		//FitControls[9][0] = peakTypeTemp
		//FitControls[10][0] = background
		//FitControls[11][0] = num2str(numPeaks)
		
		duplicate /o FitControls :Setup:FitControls
		killwaves /z FitControls
		killdataFolder :FitTemp
		SetDataFolder StartingDirectory
		
	endif
	
	killdatafolder /z root:STFitAssVar
end

function makeFitControls()

	NVAR fitMin = root:STFitAssVar:STFitMin
	NVAR fitMax = root:STFitAssVar:STFitMax
	
	NVAR saved = root:STFitAssVar:saved
	NVAR ProjectStarted = root:STFitAssVar:projectStarted
	
	NVAR SavedLast = root:STFitAssVar:savedLast
	
	NVAR tolerance = V_FitTol
	NVAR iterations = V_FitMaxiters
	NVAR fitOptions = V_FitOptions
	NVAR numPeaks = root:STFitAssVar:ST_NumPeaks
	
	SVAR projectName = root:STFitAssVar:projectName
	SVAR StartingDirectory = root:STFitAssVar:ST_StartDirectory

	SVAR projectPath = root:STFitAssVar:CP_workingFolder
	SVAR nameWorkWave = root:STFitAssVar:PR_nameWorkWave
	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR background = root:STFitAssVar:PR_Background
	SVAR peakTypeTemp = root:STFitAssVar:PR_PeakTypeTemp
	SVAR XRawWave = root:STFitAssVar:PR_XRawDataCursorPanel
	SVAR FitRawData = root:STFitAssVar:PR_FitRawData
	SVAR CoefWave = root:STFitAssVar:PR_CoefWave
	SVAR fitWave = root:STFitAssVar:PR_FitWave

	
	make /t /n=(100,2) /o FitControls    //more points than needed right now - maybe useful in the future
	wave /t FitControls = FitControls
	FitControls[p][q] = "0"  
	FitControls[0][0] = num2str(FitMin)
	FitControls[1][0] = num2str(FitMax)
	FitControls[2][0] = num2str(tolerance)
	FitControls[3][0] = num2str(iterations)
	FitControls[4][0]= num2str(fitOptions)
	FitControls[5][0] = projectName
	FitControls[6][0] = nameWorkWave
	FitControls[7][0] = XRawWave
	FitControls[8][0] = peakType
	FitControls[9][0] = peakTypeTemp
	FitControls[10][0] = background
	FitControls[11][0] = num2str(getPeakNumber())
end




function STsaveTemplate(ctrlName) : ButtonControl
	string ctrlName
	
	wave /t STsetup = STsetup
	if (DimSize(STsetup,0) == 0)
		DoAlert 0, "You can not save an empty project as template, please add some peaks and/or do a fit."
		return -1
	endif 
	
	makeFitControls()
	variable index = 0
	string templateName = ""
	Prompt templateName, "Provide a name for the template "
	DoPrompt "Save as template",templateName
	if (V_Flag)
		return -1								// User canceled
	endif
	
	string parent = GetDataFolder(1)
	SetDataFolder root:
	
	if (!DataFolderExists("Fit_templates"))
		NewDataFolder /O Fit_templates
		//provide a name for the template
	endif
	SetDataFolder Fit_templates
	NewDataFolder /O templateName
	SetDataFolder parent
	////wave /t setup = STsetup
	
	if (DataFolderExists("Setup"))
	SetDataFolder :Setup
	Duplicate /T /o STsetup, root:Fit_templates:templateName:STsetup
	Duplicate /T /o selSTsetup root:Fit_templates:templateName:selSTsetup
	Duplicate /T /o Numerics root:Fit_templates:templateName:Numerics
	Duplicate /T /o selNumerics root:Fit_templates:templateName:selNumerics
	Duplicate /T /o FitControls root:Fit_templates:templateName:FitControls
	SetDataFolder parent
	
	else
	Duplicate /T /o STsetup, root:Fit_templates:templateName:STsetup
	Duplicate /T /o selSTsetup root:Fit_templates:templateName:selSTsetup
	Duplicate /T /o Numerics root:Fit_templates:templateName:Numerics
	Duplicate /T /o selNumerics root:Fit_templates:templateName:selNumerics
	Duplicate /T /o FitControls root:Fit_templates:templateName:FitControls
	endif
	
	SetDataFolder root:
	SetDataFolder :Fit_templates
	string tempName = templateName
	if (dataFolderExists(templateName))
	do
		index += 1
		tempName = templateName +"_" + num2str(index)
	while (dataFolderExists(tempName))
	templateName = tempName
	endif
	RenameDataFolder templateName, $templateName
	SetDataFolder parent
end





//////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
//    Display the constraints wave and the respective legend
proc editDisplayWave(ctrlName):ButtonControl
	string ctrlName
	CheckLocation()
	DoWindow PeakEditor//ConstraintTable
	if ( V_Flag )
		DoWindow /F PeakEditor
		return -1
	endif
	PeakEditor()
	DoWindow /T PeakEditor, "Constraints and initial values"
end

//////////////////////////////////////////
/////////
//	Show the various waves
Window ConstraintTable(): Table
	initializeConstraintTable()  //the window command goes back to the root directory by default, so one has to force it to go back to the correct data folder
	Button UpDateBut disable =0
	if ( !exists("Min_Limit") )
		CreateBoundaryWaves()    // to be backwards compatible
	endif
	if (exists("epsilon"))
		edit /k=1 /W=(10,50,710,390)  CoefLegend,W_Coef,InitializeCoef,hold, Min_Limit, Max_Limit, epsilon
		modifytable title(epsilon)="Iteration Step", rgb(epsilon)=(30000,0,0) , alignment(epsilon) =1
	else
		edit /k=1 /W=(10,50,710,390)  CoefLegend,W_Coef,InitializeCoef,hold, Min_Limit, Max_Limit
	endif
	modifytable alignment(InitializeCoef) =1, title(InitializeCoef)="Initial Values", width(hold) =35, alignment(hold)=1
	modifytable width(CoefLegend) =150, size(CoefLegend)=10, style(CoefLegend)=1, style(hold)=1, alignment(CoefLegend)=0
	modifytable alignment(W_Coef)=2, rgb(W_Coef)=(0,23000,12000), alignment(Min_Limit)=0, alignment(Max_Limit)=0, width(Min_Limit)=100, width(Max_Limit)=100
	modifytable rgb(CoefLegend)=(0,23000,65535), alignment(W_coef)=1,title(CoefLegend) ="Parameters", title(Min_Limit)="Lower Limit",title(Max_Limit) = "Upper Limit" 
	modifytable title(W_coef)="Results", style(W_Coef)=1
end

/////////////////////////////////////////////////////////////////////
/////////
//	Set the correct location in the data browser
function initializeConstraintTable()
	SVAR name = root:STFitAssVar:CP_workingFolder
	SetDataFolder name
end

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////
//	Read  T_Constraints and make Min_Limit and Max_Limit --> backward compatibility function
function CreateBoundaryWaves()
	wave /t T_Constraints = T_Constraints    //make a local reference to T_Constraints
	variable length = numpnts(T_Constraints)
	variable i = 0
	length *= 0.5              						//Min_Limit and Max_Limit do only have half the length of T_Constraints
	make /t /o /n=(length) Min_Limit
	make /t /o /n=(length) Max_Limit
	for ( i=0 ;  i < length ;  i += 1 )
		Min_Limit[i] = T_Constraints[2*i]
		Max_Limit[i] = T_Constraints[2*i+1]
	endfor
end



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// function behind the "Delete Peak" button  
//
// removes the very last graph from the small display and cuts off the last peak parameters form
// all relevant waves


function RemoveEntry(ctrlName):ButtonControl
	string ctrlName
	
	SVAR peakType = root:STFitAssVar:PR_PeakType
	strswitch(peakType)
	case "SingletSK":
		RemoveSinglet()
		break
	case "MultiSK":
		RemoveMultiplet()
		break
	default:
		DoAlert 0, "Peak type not recognized in RemoveEntry()"
		break
	endswitch

end

function RemoveSinglet()
	CheckLocation()
	//these are needed to be able to call SinglePeakDisplay, in case of the background functions
	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR RawXWave = root:STFitAssVar:PR_XRawDataCursorPanel
	SVAR RawYWave = root:STFitAssVar:PR_nameWorkWave
	SVAR coefWave = root:STFitAssVar:PR_CoefWave
	NVAR numPeaks = root:STFitAssVar:ST_NumPeaks
	NVAR toLink = root:STFitAssVar:STPeakToLink
	NVAR peakToExtract = root:STFitAssVar:STPeakToRemove
	NVAR savedLast = root:STFitAssVar:savedLast
	savedLast = 0 //this change has not beed saved yet
	UpdateFitDisplayFinalVal("fromAddPeak")
	updateCoefs()
	numPeaks -= 1
	numPeaks=max(0,numPeaks)
	toLink = min(toLink,numPeaks)
	//peakToExtract = max(0,numPeaks)
	/////////////////////////////////////////////////////////////


	wave /t source = STsetup
	//NVAR peakToExtract = PeakToDelete
	wave sw = selSTsetup

	wave /t  numerics = Numerics
	wave selNumerics = selNumerics

	variable i,j,k
	variable length = DimSize(source,0)
	variable NumLength = DimSize(numerics,0)

	//this needs to be rewritten for doublet functions as well
	//numPeaks = (length-5)/6

	//this is the simple version of delete which only removes the last entry
	//if (length>=6)
	//Redimension /n=(length-6,-1) source
	//Redimension /n=(length-6,-1) sw
	//endif

	if (length == 5) ///only the background is left
		return 1 /// do nothing, the background may stay there foreve	r
	endif

	string ListOfCurves
	variable numberCurves
	variable startCutIndex, endCutIndex
	variable numCoefs = 6

	//FancyUP("foe")

	//now do a sophisticated form of delete which removes a certain peak from within the waves
	//for example peak 2
	//peakToExtract = 2 //this needs to be soft-coded later on

	//duplciate the sections that need to go
	//to do so: calculate the indizes that have to be removed
	//this needs to be extended for doublet functions as well

	startCutIndex = 5 + (peakToExtract-1)*numCoefs
	endCutIndex = startCutIndex + 6

	variable startCutIndexNumerics = (peakToExtract-1)*4
	variable endCutIndexNumerics = startCutIndexNumerics + 4

	//now, check if there are any constraints linked to this peak, if yes, refuse to do the deleting and notify the user
	// that means the ax, px, wx, etc of this peak e.g. a2, w2, etc show up anywhere else in the constraints wave, if so, abort
	variable abortDel = 0

	string planeName = "backColors"
	variable plane = FindDimLabel(sw,2,planeName)

	Variable nplanes = max(1,Dimsize(sw,2))
	if (plane <0)
		Redimension /N=(-1,-1,nplanes+1) sw
		plane = nplanes
		SetDimLabel 2,nplanes,$planeName sw
	endif

	variable Errors = 0
	string tempString
	string CoefficientList = "a;p;w;g;s;t"
	string matchString 
	string badSpotList = ""

	for (j = 0; j<itemsInList(CoefficientList); j += 1)
		matchString = "*" + StringFromList(j,CoefficientList) + num2str(peakToExtract) + "*"
		for (i=0; i < startCutIndex; i += 1)
			tempString =source[i][5]
			abortDel = StringMatch(tempString,matchString)
			if (abortDel != 0)
				Errors += 1
				sw[i][5][plane] =1
				badSpotList += num2str(i) + ";"	
			endif
			tempString =source[i][6]
			abortDel = StringMatch(tempString,matchString)
			if (abortDel != 0)
				Errors += 1
				sw[i][6][plane] =1
				badSpotList += num2str(i) + ";"	
			endif
		endfor
		for (i=endCutIndex; i < length; i += 1)
			tempString =source[i][5]
			abortDel = StringMatch(tempString,matchString)
			if (abortDel != 0)
				Errors += 1
				sw[i][5][plane] =1	
				badSpotList += num2str(i) + ";"
			endif
			tempString =source[i][6]
			abortDel = StringMatch(tempString,matchString)
			if (abortDel != 0)
				Errors += 1
				sw[i][6][plane] =1	
				badSpotList += num2str(i) + ";"
			endif
		endfor
	endfor

	variable badSpots = ItemsInList(badSpotList)
	variable badSpot

	if (Errors != 0)
		tempString = "Other peaks are linked to the one you want to remove. \r\rDelete all references to the peak you want to remove from 'Lower Limit' and 'Upper Limit'."
		Doalert 0, tempString
		tempString = "editDisplayWave(\"foe\")"
		Execute tempString
		// and now do the highlighting
		for ( i = 0; i < badSpots; i += 1 )
			badSpot = str2num(StringFromList(i,badSpotList))
			sw[badSpot][5][plane] =1		
			sw[badSpot][6][plane] =1		
		endfor
	
		//end highlighting
	
		numpeaks += 1
		return -1
	endif

	//everything seems to be fine, now continue
	duplicate /o /r=(0,startCutIndex-1) source $"lowerSectionSetup" 
	wave /t lowerSetup = $"lowerSectionSetup"
	duplicate /o /r=(0,startCutIndex-1) sw $"lowerSectionSw" 
	wave  lowerSW = $"lowerSectionSw"

	duplicate /o /r=(endCutIndex,length-1) source $"upperSectionSetup" 
	wave /t upperSetup = $"upperSectionSetup"
	duplicate /o /r =(endCutIndex, length -1) sw $"upperSectionSw" 
	wave upperSW = $"upperSectionSw"


	duplicate /o /r=(0,startCutIndexNumerics-1) numerics $"lowerSectionNumerics" 
	wave /t lowerNumerics = $"lowerSectionNumerics"
	duplicate /o /r=(0,startCutIndexNumerics-1) selNumerics $"lowerSectionSelNumerics" 
	wave  lowerSelNumerics = $"lowerSectionSelNumerics"

	duplicate /o /r=(endCutIndexNumerics,NumLength-1) numerics $"upperSectionNumerics" 
	wave /t upperNumerics = $"upperSectionNumerics"
	duplicate /o /r =(endCutIndexNumerics, NumLength -1) selNumerics $"upperSectionSelNumerics" 
	wave upperSelNumerics = $"upperSectionSelNumerics"


	//remove also the entries for the numerics wave

	//remove the space for one peak
	Redimension /n=(length-6,-1) source
	Redimension /n=(length-6,-1) sw

	Redimension /n=(NumLength-4,-1) numerics    //four lines per peak if the peak type is singlet
	Redimension /n=(NumLength-4,-1) selNumerics

	//and now, copy the stuff back, start with the lowerSection
	for (i = 0; i < startCutIndex; i += 1)
		for ( j =2; j < 8; j +=1) // do not overwrite the legend waves, this would be redundant
			if (j  != 4)
				source[i][j]=lowerSetup[i][j]
			endif
			sw[i][j]=lowerSW[i][j]
		endfor	
	endfor
	//and continue with the upper section
	for (i = startCutIndex; i < length-6; i += 1)
		for ( j =2; j < 8; j +=1)
			if (j  != 4)
				source[i][j]=upperSetup[i-startCutIndex][j]
			endif
			sw[i][j]=upperSW[i-startCutIndex][j]
		endfor
	endfor

	//now repeat everything for the Numerics wave
	for (i = 0; i < startCutIndexNumerics; i += 1)
		for ( j =2; j < 15; j +=1) // do not overwrite the legend waves, this would be redundant
			numerics[i][j]=lowerNumerics[i][j]
			selNumerics[i][j]=lowerSelNumerics[i][j]
		endfor	
	endfor
	//and continue with the upper section
	for (i = startCutIndex; i < length-4; i += 1)
		for ( j =2; j < 15; j +=1)
			numerics[i][j]=upperNumerics[i-startCutIndex][j]
			selNumerics[i][j]=upperSelNumerics[i-startCutIndex][j]
		endfor
	endfor

	killwaves /z upperSetup, upperSW, lowerSetup, lowerSW, lowerSelNumerics, upperSelNumerics, lowerNumerics, upperNumerics

	//now make sure that all the parameter names, such as a2, a3, etc are updated
	//if the second peak was removed:   old > new 
	//								a1 > a1
	//								a2 > removed
	//								a3 > a2 //k = 0
	//								a4 > a3  //k = 1
	string lowerIndexIn, higherIndexOut

	for ( k = 0; k< numpeaks; k += 1)
		for ( j = 0; j < itemsInList(CoefficientList); j += 1 )
			lowerIndexIn = StringFromList(j,CoefficientList) + num2str(peakToExtract+k )  
			higherIndexOut = StringFromList(j,CoefficientList) +num2str(peakToExtract + k +1)
			//print lowerIndexIn, higherIndexOut
			for ( i = 0; i < length-6; i += 1 )
				tempString = source[i][5]
				source[i][5]=ReplaceString(higherIndexOut, tempString, lowerIndexIn)
				tempString = source[i][6]
				source[i][6]=ReplaceString(higherIndexOut, tempString, lowerIndexIn)
			endfor
		endfor
	endfor
	
	///////////////////////////////////////////////////////////
	setup2waves()	
	ListOfCurves = TraceNameList("CursorPanel#guiCursorDisplayFit",";",1)
	numberCurves = ItemsInList(ListOfCurves)
	//remove only the very last curve, if there are e.g. 3 curves on the graph it has the index #2
	//this needs to be adapted to the background functions
	RemoveFromGraph /W=CursorPanel#guiCursorDisplayFit /Z $"#"+num2istr(numberCurves-1)
	//if (BackgroundType != 0 )
	for (i =2; i<numberCurves; i +=1)
		RemoveFromGraph /W=CursorPanel#guiCursorDisplayFit /Z $"#"+num2istr(numberCurves-i)
	endfor
	//and now redisplay, if there are any peaks left
	SinglePeakDisplay(peakType,RawYWave,RawXWave, "InitializeCoef")//coefWave)
	FancyUp("foe")
	peakToExtract = max(1,numPeaks)
	SetVariable InputSetLink, limits={0,numPeaks,1}
	SetVariable InputRemoveLink,limits={1,peakToExtract,1}
	
end

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
function PeakDisplay(ctrlName):ButtonControl
	string ctrlName
	CheckLocation()	
	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR RawYWave = root:STFitAssVar:PR_nameWorkWave
	SVAR coefWave = root:STFitAssVar:PR_CoefWave
	SVAR RawXWave = root:STFitAssVar:PR_XRawDataCursorPanel
	//extensions: this function has to be modified as well	
	SinglePeakDisplay(peakType,RawYWave,RawXWave,coefWave)
	
end
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//   Display the single peaks
// to do so, analyze the coefficient wave and the peak type


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Display-Button 
// display the wave to be fitted

function displayWavesCursorPanel(ctrlName): ButtonControl
	string ctrlName
	string HostWindowName = "CursorPanel"
	CheckLocation()
	SVAR coefWave = root:STFitAssVar:PR_CoefWave
	SVAR RawXWave = root:STFitAssVar:PR_XRawDataCursorPanel
	wave rawX = $RawXWave
	SVAR RawYWave = root:STFitAssVar:PR_nameWorkWave
	wave rawY = $RawYWave
	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR templateName = root:STFitAssVar:templateName
	
	NVAR useTemplate = root:STFitAssVar:useTemplate
	NVAR keepIndicate = root:STFitAssVar:KeepCheckIndicate
	NVAR FitMin = root:STFitAssVar:STFitMin
	NVAR FitMax = root:STFitAssVar:STFitMax	

	NVAR numPeaks = root:STFitAssVar:ST_NumPeaks
	
	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR background = root:STFitAssVar:PR_Background
	SVAR peakTypeTemp = root:STFitAssVar:PR_PeakTypeTemp
	
	NVAR peakToExtract = root:STFitAssVar:STPeakToRemove
	
	
	if ( strlen(RawYWave) >= 26)		  //26 charackters would be okay, but there are names generated form this string, and those get possibly too long
		doalert 0, "The name of the fit-wave is too long! Please shorten the names."
		return 1
	endif
	combinePeakBackground()  //call the function that gets the peak type from chosen profile and background
	
	strswitch(RawYWave)
		case "":
			doalert 0, "No data wave given!"
			return -1
			break
		default:
			break
	endswitch
	
	//figure out if there is an xwave to get the default values for the fit min and max
	strSwitch(RawXWave)
		case "":
			FitMin = min( pnt2x(rawY,0),  pnt2x( rawY,numpnts(rawY)-1 ) )
			FitMax = max( pnt2x(rawY,0),  pnt2x( rawY,numpnts(rawY)-1 ) )
			break
		case "_calculated_":
			FitMin = min( pnt2x(rawY,0),  pnt2x( rawY,numpnts(rawY)-1 ) )
			FitMax = max( pnt2x(rawY,0),  pnt2x( rawY,numpnts(rawY)-1 ) )
			break
		default:
			FitMin=WaveMin(rawX)
			FitMax=WaveMax(rawX)
			break
	endswitch

	strswitch(peakType)
		case "":
			doalert 0, "No peak type given, or selected combination of peak and background not implemented yet!"
			return -1
			break
		default:
			break
	endswitch

	if (useTemplate == 0)
	make /t /n=(0,8) /o STsetup
	make /n=(0,8) /o selSTsetup
	make /t /n=(0,15) /o Numerics
	make /n=(0,15) /o selNumerics
	
	makeFitControls()
	
	else
	//duplicate the wave from the template folder to this location
	// this is not working:     Duplicate /o root:Fit_templates:$templateName:STsetup STsetup        and it is annoying
	
	DuplicateDataFolder root:Fit_templates:$templateName, Setup
	string parentFolder = GetDataFolder(1)
	SetDataFolder Setup 
		duplicate /o STsetup, $(parentFolder + "STsetup")
		duplicate /o selSTsetup, $(parentFolder + "selSTsetup")
		duplicate /o Numerics, $(parentFolder + "Numerics")
		duplicate /o selNumerics, $(parentFolder + "selNumerics")
		duplicate /o FitControls, $(parentFolder + "FitControls")
	SetDataFolder parentFolder
	//duplicate /o :Setup:STsetup :STsetup 
	//duplicate /o :Setup:selSTsetup :selSTsetup 
	//duplicate /o :Setup:Numerics :Numerics 
	//duplicate /o :Setup:selNumerics :selNumerics 
	//duplicate /o :Setup:FitControls :FitControls 
	wave /t FitControls = FitControls
	KillDataFolder Setup

	
	/////	reverse this
	FitMin = str2num(FitControls[0][0])
	FitMax = str2num(FitControls[1][0]) 
	peakType = FitControls[8][0] 
	peakTypeTemp = FitControls[9][0] 
	background = FitControls[10][0]
	numPeaks = str2num(FitControls[11][0])
	peakToExtract = numPeaks
	setup2waves()
	updateFitDisplayFinalVal("fromTemplateLoad")
	//now reset everything
	useTemplate = 0 
	templateName = ""
	
	endif
	
	//Titlebox notify win=$HostWindowName, title="Use the mouse to select a peak ..."
	Button addBut win=$HostWindowName, fColor=(44000,52000,65500), disable=0        //(32768,54528,65280)
	Button DeleteBut win=$HostWindowName, fColor=(44000,52000,65500), disable=0           //(32768,54528,65280)
	Button FitBut win=$HostWindowName, fColor=(44000,52000,65500)    
	Button ConstraintBut win=$HostWindowName, fColor=(44000,52000,65500)   

end


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Add Button
// read the cursor position and write into the coefficient wave, which is used for fitting later on
function ReadOutPosition(ctrlName): ButtonControl
	string ctrlName
	CheckLocation()
	//hard-coded to the root directroy, maybe change later
	SVAR RawXWave = root:STFitAssVar:PR_XRawDataCursorPanel
	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR RawYWave = root:STFitAssVar:PR_nameWorkWave
	//SVAR coefWave = root:STFitAssVar:PR_CoefWave
	NVAR keepIndicate = root:STFitAssVar:KeepCheckIndicate
	NVAR indepCheck = root:STFitAssVar:CP_IndepCheck
	NVAR numPeaks = root:STFitAssVar:ST_NumPeaks
	NVAR savedLast = root:STFitAssVar:savedLast
	NVAR keepInitConfiguration = root:STFitAssVar:keepInitConfiguration
	savedLast = 0 //this change has not beed saved yet
	NVAR ProjectStarted = root:STFitAssVar:projectStarted
	
	NVAR toLink = root:STFitAssVar:STPeakToLink
	NVAR toRemove = root:STFitAssVar:STPeakToRemove
	NVAR areaLink = root:STFitAssVar:AreaLink
	NVAR positionLink = root:STFitAssVar:PositionLink
	NVAR WidthLink = root:STFitAssVar:WidthLink
	NVAR GLLink = root:STFitAssVar:GLLink
	NVAR AsymLink =root:STFitAssVar:AsymLink
	
	//add a peak and, if there is an old fit result, add it to this fit results
	if ( keepInitConfiguration == 0 && WaveExists(W_Coef))
		ReUseWCoef("fromAddPeak")
	endif
	
	string initCoef = "InitializeCoef"
	numPeaks += 1
	numPeaks =max(0,numPeaks)
	toLink = min(toLink,numPeaks)
	toRemove = max(0,numPeaks)
	
	Button DeleteBut disable = 0
	Button ConstraintBut disable =0
	Button UpDateBut disable = 0
	SetVariable InputSetLink,limits={0,numPeaks,1}
	SetVariable InputRemoveLink,limits={1,numPeaks,1}

	RemoveFromGraph /W=CursorPanel#guiCursorDisplay /Z $"#1"
	updateCoefs()
	//now use this, to append the cursor position, this is necessary since the following function works only on W_coef
	//this function has to be edited as well
	ReadOutPositionFunc(peakType,RawYWave, RawXWave,initCoef,indepCheck)
	
	//SinglePeakDisplay(peakType,RawYWave,RawXWave,coefWave)
	//duplicate /o $"W_coef" $initCoef
	areaLink = 0
	positionLink = 0
	WidthLink =1
	GLLink = 1
	AsymLink = 1
	CheckBox check0,value= 0
	CheckBox check1,value= 0
	CheckBox check2,value= 1
	CheckBox check3,value= 1
	CheckBox check4,value= 1
	ProjectStarted = 1

end

function CheckLocation()
	SVAR location = root:STFitAssVar:CP_workingFolder
	variable match
	string actualLocation = GetDataFolder(1)
	match=stringmatch(location,actualLocation)    //returns 1 if they match
	if (match != 1)
		doalert 0,"Do not change the data folder while a session is running. Resetting to start folder. If you want to work elsewhere: (1) Close, (2) select the folder and (3) start the script againg."
		SetDataFolder location
		return 1
	endif
	return 0
end


// auxiliary function to normalize the constraints wave
function /S NormAndDeNormConstraints(line,factor,NormSwitch)
	string line
	variable factor
	
	variable NormSwitch           //0: normalize  //1: de-normalize
	string answer = ""  //also used as temporay string for cleaning of whitespaces
	
	string filterResult
	string ListWithSeparators = ">;<;+;- ;"
	variable i,j 
	string Separator = "", positionList = ""
	string S_lowIndex, S_highIndex
	variable lowIndex, highIndex
	string partString
	variable partNumber
	variable LenPositionList
	
	if ( factor == 0)
		print "error in NormAndDeNormConstraints, division by zero"
		return "error"
	endif
	
	//clean string from whitespaces answer is used as a volatile container
	line = UpperStr(line)    //deal only with capital characters
	variable sampleLen = strlen(line)
	for ( i = 0; i < sampleLen ; i += 1 )
		if (stringmatch(line[i]," ") == 0)
			answer += line[i]
		endif
	endfor
	line = answer
	answer = ""
	// .... done
	
	//get the location of the separators and the last location in the string
	for ( i = 0; i < 4; i += 1 )
		Separator = StringFromList(i,ListWithSeparators)
		for ( j = 0 ; j < sampleLen;  j += 1 )
			if (stringmatch(line[j],Separator) == 1 && stringmatch(line[j-1],"e") == 0)
				positionList += num2str(j)+";"
			endif
		endfor
	endfor
	//attach sart and end-index, and sort it
	positionList += num2str(sampleLen)
	positionList = "0;" + positionList
	positionList = SortList(positionList,";",2)
	// .... done

	LenPositionList = ItemsInList(positionList)
	for ( i = 1; i < LenPositionList; i += 1 )	
		S_lowIndex = StringFromList(i-1, positionList)
		S_highIndex = StringFromList(i, positionList)
		lowIndex = str2num(S_lowIndex)
		highIndex = str2num(S_highIndex)
		if (lowIndex != 0)
			partString = line[lowIndex+1,highIndex-1]
		else
			partString = line[lowIndex, highIndex-1]
		endif
		if ( MyGrepString(partString,"K") == 0)
			partNumber = str2num(partString)
			if (NormSwitch == 0)
				partNumber /= factor
			else 
				partNumber *= factor
			endif
			if (partNumber != Nan)    //this can happen sometimes
				partString = num2str(partNumber)
				answer +=  " " + line[lowIndex] + " " + partString + " "
			endif
		else
			if ( i == 1)  
				answer +=  line[lowIndex, highIndex-1] + " "    //make correct indents
			else
				answer +=  " " + line[lowIndex] + " " + line[lowIndex+1, highIndex-1] + " "
			endif
		
		endif
	endfor	
	return answer
end

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Add Button
// read the cursor position and write into the coefficient wave, which is used for fitting later on
function ReadOutPositionFunc(peakType,RawYWave, RawXWave,coefWave,indepCheck)
	
	string RawXWave 
	string peakType
	string RawYWave
	string coefWave
	variable indepCheck                //decide whether to link the new peak to peak 1 or not
	
	Button FitBut disable=0           //now the user can press the fit button
	//Button showBut disable=2       //once the graph is displayed, there is no way back
	 
	string name = "CursorPanel#guiCursorDisplay"    //hard-coded to avoid more global variables
	
	//Variables for the cursor position
	variable width, relPos, relHeight, indexA, posA, heightA
	variable PeakArea
	variable WCoef_length,i, nPeaks, numPara
	variable left, right
	variable EstimatedPeakArea
	// the creation of the local refernece is necessary to
	// copy the values of W_coef in the form
	// of newWave = oldWave
	// with this method, no new wave ( a global object ) is created
	wave locRefW_coef = W_Coef
	wave /t Min_Limit = Min_Limit   //just create a local refernce, do not get confused by the identical names
	wave /t Max_Limit = Max_Limit 
	wave /t locRefConstraints = T_Constraints
	wave /t locRefLegend = LegendWave
	wave /t CoefLegendRef = CoefLegend
	wave holdRef = hold                      //The global wave is named "hold"
	wave epsilon = epsilon
	//check if all necessary informations are given, if not, refuse to make further actions
	strswitch(RawYWave)
		case "":
			doalert 0, "No data wave given!"
			return -1
			break
		default:
			break
	endswitch

	strswitch(peakType)
		case "":
			doalert 0, "No peak type given!"
			return -1
			break
		default:
			break
	endswitch
	
	//get the cursor position
	 indexA = pcsr(A,name)   
	 posA = hcsr(A, name)
	 heightA= vcsr(A,name)
	 
	 
	//decide, if the spectrum is in waveform or x-y format 
	//and do the scaling of the cursor positions accordingly
	strswitch(RawXWave)
	 	case "":
	 		 left = max(leftx($RawYWave),rightx($RawYWave))
	 		 right = min(leftx($RawYWave),rightx($RawYWave))
	 		 width = leftx($RawYWave)-rightx($RawYWave)  
	 	break
	 	case "_calculated_":
	 		 left = max(leftx($RawYWave),rightx($RawYWave))
	 		 right = min(leftx($RawYWave),rightx($RawYWave))
	 		 width = leftx($RawYWave)-rightx($RawYWave)  
	 	break
	 	default:
	 		waveStats /Q $RawXWave
	 		left = V_max
	 		right = V_min
	 		width = left - right
	 	break
	 endswitch
	 
	WCoef_length =DimSize(W_coef,0)    
	variable ConstraintLength =DimSize(locRefConstraints,0)

	strswitch(peakType)
	///////////////////////////////////////////////////////////////////////////////////////////////// 
	//			values like 'interval', 'Width_Start' and so on are pre-defined in the file Constants.ipf
	// 			they are no ordinary variables and should not be changed during execution
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// Gauss Singlet
		
		case "SingletSK":
			AddVoigtSKPeak(coefWave,RawYWave,indepCheck,heightA,posA,left,right,Wcoef_length)
		break
		case "MultiSK":
			AddVoigtSKMultiplet(coefWave,RawYWave,indepCheck,heightA,posA,left,right,Wcoef_length)
			
		break
		default:
			DoAlert 0, "Peak type not recognized, something is wrong"
		break
	endswitch
	SinglePeakDisplay(peakType,RawYWave,RawXWave,coefWave)
end

// the following function is called, after the Add-peak button was pressed and the waves have been updated
// it displays the individual peaks in the graphical user interface

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// single peak display 

function SinglePeakDisplay(peakType,RawYWave, RawXWave,coefWave)

	//SVAR peakType = root:S_PeakType
	//SVAR RawYWave = root:S_nameWorkWave
	//SVAR coefWave = root:S_CoefWave
	//SVAR RawXWave = root:S_XRawDataCursorPanel
	
	string peakType
	string RawYWave
	string RawXWave
	string coefWave
	string TagName    // the Tag in the result window
	string PeakTag     // text in this tag
	string PkName, parentDataFolder //, cleanUpString=""		
	string BGName //background
	string PeakSumName
	
	
 	strswitch(peakType)
		case "SingletSK":     //we are here for now
			PlotPseudoVoigtSKDisplay(peakType,RawYWave, RawXWave,coefWave)
		break
		case "MultiSK":
			PlotMultipletSKDisplay(peakType,RawYWave, RawXWave,coefWave)
		break
		default:
			DoAlert 0, "Peak type not recognized, something is wrong"
			print "Peak type was not recognized, something is wrong."
			break
	endswitch
end

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// display the waves in the GUI





////////////////////////////////
////  Export and Import

function STExportFit(ctrlName):ButtonControl
	string ctrlName
	//get the full fit setup, this includes the setup waves, the fit_control wave and the spectrum (+xWave if necessary)
	SVAR projectName = root:STFitAssVar:projectName	
	SVAR rawY = root:STFitAssVar:PR_nameWorkWave
	wave spectrum = $rawY
	SVAR rawX = root:STFitAssVar:PR_XRawDataCursorPanel
	
	string outputName = "Fit_" + projectname +".itx"
	wave /t setup = STsetup
	wave selSetup = selSTsetup
	wave /t num = Numerics
	wave selNum = selNumerics
	wave /t fitC = FitControls
	string fitResult = WaveList("fit_*",";","")
	variable fitThere = itemsInList(fitResult)
	if (fitThere != 0)
		wave fitWave = $StringFromList(0,fitResult)
	endif
	
	if (DimSize(setup,0) == 0)
		DoAlert 0, "You can not export an empty project, please add some peaks."
		return -1
	endif 
	
	
	makeFitControls()
	
	strswitch(rawX)
		case "":
			if (fitThere != 0)
				save /I /T setup,selSetup,num,selNum,fitC,fitWave,spectrum, as outputName
			else
				save /I /T setup,selSetup,num,selNum,fitC,spectrum as outputName
			endif
		break
		case "_calculated_":
			if (fitThere != 0)
				save /I /T setup,selSetup,num,selNum,fitC,fitWave,spectrum as outputName
			else
				save /I /T setup,selSetup,num,selNum,fitC,spectrum as outputName
			endif
		break
		default:
			wave xSpectrum = $rawX
			if (fitThere != 0)
				save /I /T setup,selSetup,num,selNum,fitC,fitWave,spectrum,xSpectrum as outputName
			else
				save /I /T setup,selSetup,num,selNum,fitC,spectrum,xSpectrum as outputName
			endif
		break
	endswitch
	
end

function STImportFit(ctrlName):ButtonControl
	string CtrlName
	//go to the main dataFolder
	SVAR parent = root:STFitAssVar:ST_StartDirectory
	SVAR oldWorkingDirectory = root:STFitAssVar:CP_workingFolder
	SVAR toOpen = root:STFitAssVar:nextProjectToOpen
	
	NVAR saved = root:STFitAssVar:saved
	NVAR savedLast = root:STFitAssVar:savedLast
	NVAR projectStarted = root:STFitAssVar:projectStarted
	
	DoWindow /K PeakEditor
	DoWindow /K PeakViewer
	DoWindow /K GetAreaFactors
	DoWindow /K GetOffsetIntervall
	
	variable index = 1
	string oldPlace = oldWorkingDirectory
	
	if (projectStarted == 1 && saved ==0)
		DoAlert 1, "You did not save the current fitting project. Proceed anyway?"
		if (V_Flag == 2)
			return 1
		endif
	elseif (saved == 1 && savedLast == 0)
		DoAlert 1, "You did not save the most recent changes to the current fitting project. Proceed and keep the previous saved version?"
		if (V_Flag == 2)
			return 1
		endif
	endif
	SetDataFolder parent
	NewDataFolder /O /S imported
	LoadWave /T 
	if (V_Flag == 0)
		SetDataFolder parent
		KillDataFolder /z imported
		SetDataFolder oldWorkingDirectory
		return 1
	endif
	SetDataFolder parent
	S_FileName = ReplaceString("Fit_",S_Filename,"")
	S_FileName = ReplaceString(".itx",S_Filename,"")
	
	string temp = S_FileName
	string output = ""
	if (DataFolderExists(S_FileName))
		do 	
			temp = S_FileName + num2str(index)
			index += 1
		while (DataFolderExists(temp))
		
		sprintf output, "This project name already existed.\rThe name of the imported project was changed to:    %s ", temp
		DoAlert 0, output
	endif
	S_FileName = temp
	
	RenameDataFolder :imported, $S_FileName
	SetDataFolder $S_FileName
	ClearGraphs() //empty the graphs so that the main window is ready for a change
	toOpen = S_FileName
	NewDataFolder /o Peaks
	NewDataFolder /o Setup
	duplicate /o STsetup :Setup:STsetup
	duplicate /o selSTsetup :Setup:selSTsetup
	duplicate /o Numerics :Setup:Numerics
	duplicate /o selNumerics :Setup:selNumerics
	wave /t FitControls = FitControls
	FitControls[5][0] = S_FileName
	duplicate /o FitControls :Setup:FitControls
	
	
	killwaves /z FitControls, Numerics, selNumerics,STsetup, selSTsetup
	
	//now go back to the folder where you started from and simply call OpenFitProject
//	SetDataFolder oldWorkingDirectory
	SetDataFolder parent
	OpenFitProject("FormImport")
	SetDataFolder parent
	if (saved == 0)
		killDataFolder /z oldPlace
	endif
	SetDataFolder toOpen
end




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////
//////////////////////////////
///////////////////////////////                                                                                   MORE PANELS
//////////////////////////////
//////////////////////////////
//////////////////////////////
//////////////////////////////
//////////////////////////////
//////////////////////////////
//////////////////////////////
//////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



function CallSTProjectPanel(ctrlName):ButtonControl
	string ctrlName
	NVAR saved = root:STFitAssVar:saved
	NVAR ProjectStarted = root:STFitAssVar:projectStarted
	NVAR savedLast = root:STFitAssVar:savedLast
	SVAR parentFolder = root:STFitAssVAr:ST_StartDirectory
	SVAR workingFolder = root:STFitAssVar:CP_workingFolder
	
	DoWindow /K PeakEditor
	DoWindow /K PeakViewer
	DoWindow /K GetAreaFactors
	DoWindow /K GetOffsetIntervall
	
	wave /t STsetup = STsetup
	variable WorkedInProject = DimSize(STsetup,0)
	
	SetDataFolder parentFolder
	DoWindow STOpenProjectPanel   //this will write "1" into V_Flag if the window is open already
	if (V_Flag )
		DoWindow /F STOpenProjectPanel  // if it is already open, bring it to front
		return -1
	endif
	
	
	if (ProjectStarted == 1 &&  saved == 0)
		DoAlert 1, "There are unsaved changes in the current project. Do you really want to proceed?"
		if (V_Flag ==2) //user pressed no
			SetDataFolder workingFolder
			return -1
		elseif (V_Flag == 1)
			//restart the panel
			destroyCursorPanel("restart") //now,we are using the string as a switch for a restart
			LaunchCursorPanel()	
		endif
	elseif (ProjectStarted ==1 && SavedLast==0 && saved==1)
		DoAlert 1, "You did not save the most recent changes in this project - however a previous version was saved. Do you want to proceed and keep only the previous version?"
		if (V_Flag ==2)
			SetDataFolder workingFolder
			return -1
		elseif (V_Flag == 1)
		//yes, that means restore the last valid fit
			SetDataFolder workingFolder
			restoreSavedFit("fromOpenProject")
			SetDataFolder parentFolder
		endif
	endif
	
	execute "STOpenProjectPanel()"
	DeActivateButtons(0)
	Button OpenProjectBtn,  win=CursorPanel, disable=0
end

function KillSTOpenProjectPanel(ctrlName):ButtonControl
	string ctrlName
	SVAR originalLocation = root:STFitAssVar:OpenDialogOriginalLocation //root:STFitAssVar:CP_WorkingFolder
	SVAR originalProject = root:STFitAssVar:OpenDialogOriginalProject
	NVAR projectStarted = root:STFitAssVar:projectStarted
	SVAR startingDirectory = root:STFitAssVar:ST_StartDirectory
	SVAR toOpen = root:STFitAssVar:nextProjectToOpen
	
	DoWindow /K STOpenProjectPanel
	string parent = GetDataFolder(1)
	
	strswitch(ctrlName)
		case "afterOpening":
			//the user chose a project and wants to work with it now
			projectStarted = 1
		break
		default:
			if (projectStarted ==1 )
				OpenFitProject("reset")
				SetDataFolder originalLocation
			else
				ClearGraphs()
				SetDataFolder :$toOpen
				CleanUpAfterPreview()
				SetDataFolder startingDirectory
			endif
			
		break
	endswitch
	//killstrings /z root:STFitAssVar:nextProjectToOpen,root:STFitAssVar:OpenDialogOriginalLocation,root:STFitAssVar:OpenDialogOriginalProject 
	if (projectStarted == 0)
		ActivateButtons(1)
	else
		ActivateButtons(0)
	endif
end

function CleanUpAfterPreview()
	if (DataFolderExists("FitTemp"))
		DuplicateDataFolder :FitTemp, Setup	
			//NewDataFolder /o :Setup
			//duplicate /o :FitTemp:STsetup :Setup:STsetup 
			//duplicate /o :FitTemp:selSTsetup :Setup:selSTsetup
			//duplicate /o :FitTemp:Numerics :Setup:Numerics 
			//duplicate /o :FitTemp:selNumerics :Setup:selNumerics
			//duplicate /o :FitTemp:FitControls :Setup:FitControls
	endif
		//DuplicateDataFolder :FitTemp:Peaks, :Peaks
		killwaves /z STsetup, selSTsetup, Numerics, selNumerics, FitControls
		killwaves /z W_Coef, Min_Limit, Max_Limit, epsilon, hold, W_Sigma, T_Constraints, InitializeCoef
		killvariables /z V_numNaNs, V_numINFs, V_npnts, V_nterms, V_nheld, V_startRow, V_endRow, V_startCol, V_endCol
		killvariables /z V_startLayer, V_endLayer, V_startChunk, V_endChunk, V_FitTol, V_FitMaxIters, V_FitOptions
		killdatafolder /z :FitTemp
end

Window STOpenProjectPanel() : Panel
	PauseUpdate; Silent 1		// building window...
	//NewPanel /K=2 /W=(453,176,749,428) as "open/preview fit projects"
	//SetDrawLayer UserBack
	//DrawRect 17,144,281,243
	//DrawText 30,166,"Look for fit projects in the current"
	//DrawText 30,182,"parent data folder."
	//SetDrawEnv fstyle= 1
	//DrawText 30,204,"Use  \"Data/Data Browser\" to navigate"
	//SetDrawEnv fstyle= 1
	//DrawText 30,220,"to a different data folder."
	//SetDrawEnv fstyle= 1
	//DrawText 30,235,"(use red arrow and + mouse)"
	//Button CancelBtn,pos={13,103},size={80,33},proc=KillSTOpenProjectPanel,title="Cancel"
	//Button OpenBtn,pos={201,104},size={80,33},title="Open", proc=OpenFitProject
	//PopupMenu ProjectPop,pos={14,65},size={266,21},bodyWidth=173,title="Open a fit project   "
	//PopupMenu ProjectPop,mode=2,popvalue="select a project",value=getValidDataFolders(), proc=SelectFitProject
	//TitleBox notifyParent,pos={56,12},size={222,34},title="    Your current parent data folder is:               \r    root : "
	
	
	PauseUpdate; Silent 1		// building window...
	NewPanel /K=2 /W=(77,158,406,535) as "open/preview fit projects"
	ModifyPanel cbRGB=(65534,65534,65534)
	
	SetDrawLayer UserBack
	//DrawRect 14,231,316,340
	//DrawText 34,254,"Only fit projects in the current"
	//DrawText 35,269,"parent data folder will be displayed."
	//SetDrawEnv fstyle= 1
	//DrawText 34,302,"Use  \"Data/Data Browser\" to navigate"
	//SetDrawEnv fstyle= 1
	//DrawText 34,318,"to a different data folder."
	//SetDrawEnv fstyle= 1
	//DrawText 34,333,"(mark it with the red arrow)"
	SetDrawEnv fsize= 14,fstyle= 1
	DrawText 17,37,"Open / Browse Projects"
	
	DrawRect 12,201,316,360
	DrawText 28,224,"Only fit projects in the current parent folder"
	DrawText 28,239,"will be displayed"
	SetDrawEnv fstyle= 1
	DrawText 26,276,"To change the parent folder:"
	SetDrawEnv fstyle= 1
	DrawText 28,299,"Close the 'Fit Assistant' and use"
	SetDrawEnv fstyle= 1
	DrawText 28,315,"\"Data/Data Browser\" to select"
	SetDrawEnv fstyle= 1
	DrawText 28,330,"a different data folder."
	SetDrawEnv fstyle= 1
	DrawText 27,345,"(mark it with the red arrow)"
	SetDrawEnv fstyle= 1
	DrawText 26,283,"--------------------------------------------------------"
	Button CancelBtn,pos={26,127},size={120,35},proc=KillSTOpenProjectPanel,title="Cancel"
	Button OpenBtn,pos={183,127},size={120,35},proc=OpenFitProject,title="Open"
	PopupMenu ProjectPop,pos={26,77},size={278,21},bodyWidth=173,proc=SelectFitProject,title="Available Projects:     "
	PopupMenu ProjectPop,mode=2,popvalue="select a project",value=getValidDataFolders()
	GroupBox group0,pos={14,63},size={301,111}

	initOpenProjectPanel()

EndMacro

function initOpenProjectPanel()
	SVAR parentFolder = root:STFitAssVar:ST_StartDirectory
	SVAR workingFolder = root:STFitAssVar:CP_workingFolder
	SVAR projectName = root:STFitAssVar:projectName
	string /G root:STFitAssVar:nextProjectToOpen = ""
	SetDataFolder parentFolder
	string /G root:STFitAssVar:OpenDialogOriginalLocation = workingFolder
	string /G root:STFitAssVar:OpenDialogOriginalProject = projectName
	
end

function OpenFitProject(ctrlName):ButtonControl
	string ctrlName
	
	
	NVAR fitMin = root:STFitAssVar:STFitMin
	NVAR fitMax = root:STFitAssVar:STFitMax
	NVAR peakToExtract = root:STFitAssVar:STPeakToRemove
	NVAR peakToLink = root:STFitAssVar:STPeakToLink
	NVAR numPeaks = root:STFitAssVar:ST_NumPeaks
	
	NVAR saved = root:STFitAssVar:saved
	NVAR ProjectStarted = root:STFitAssVar:projectStarted
	
	NVAR SavedLast = root:STFitAssVar:savedLast
	NVAR savedNumPeaks = root:STFitAssVar:savedNumPeaks	
	
	
	SVAR projectName = root:STFitAssVar:projectName
	SVAR originalProject = root:STFitAssVar:OpenDialogOriginalProject
	SVAR StartingDirectory = root:STFitAssVar:ST_StartDirectory
	SVAR projectPath = root:STFitAssVar:CP_workingFolder
	
	SVAR nameWorkWave = root:STFitAssVar:PR_nameWorkWave
	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR background = root:STFitAssVar:PR_Background
	SVAR peakTypeTemp = root:STFitAssVar:PR_PeakTypeTemp
	SVAR XRawWave = root:STFitAssVar:PR_XRawDataCursorPanel
	SVAR FitRawData = root:STFitAssVar:PR_FitRawData
	SVAR CoefWave = root:STFitAssVar:PR_CoefWave
	SVAR fitWave = root:STFitAssVar:PR_FitWave
	wave workWave = $nameWorkWave
		
	SVAR toOpen = root:STFitAssVar:nextProjectToOpen
	
	strswitch(ctrlName)
		case "reset":
			toOpen = originalProject
		break
		default:
		break
	endswitch
	
	strswitch (toOpen)
		case "":
			DoAlert 0, "Please select a project."
			return 1
			break
		default:
		break
	endswitch
	
	//saved = 1 //it has been saved at least once, otherwise it could not be re-opened
	SetDataFolder $projectPath
	if (WaveExists(STsetup) && DataFolderExists("FitTemp") ) //(ProjectStarted == 1)     //this happens only, if there is already a project open
		variable isOkay = 0
		NVAR tolerance = V_FitTol
		NVAR iterations = V_FitMaxiters
		NVAR fitOptions = V_FitOptions
		//before leaving to the new folder, clean up in the old one
		make /t /n=(100,2) /o FitControls    //more points than needed right now - maybe useful in the future
		wave /t FitControls = FitControls
		FitControls[p][q] = "0"  
		FitControls[0][0] = num2str(fitMin)
		FitControls[1][0] = num2str(fitMax)
		FitControls[2][0] = num2str(tolerance)
		FitControls[3][0] = num2str(iterations)
		FitControls[4][0]= num2str(fitOptions)
		FitControls[5][0] = projectName
		FitControls[6][0] = nameWorkWave
		FitControls[7][0] = XRawWave
		FitControls[8][0] = peakType
		FitControls[9][0] = peakTypeTemp
		FitControls[10][0] = background
		FitControls[11][0] = num2str(getPeakNumber())
		//this works, if a fit was done and later on a new project is opened, if a new project is opened from scratch this is not necessary
		if (DataFolderExists("FitTemp"))
			//NewDataFolder /o :Setup
			SetDataFolder FitTemp
			if (WaveExists(STsetup))
				isOkay = 1
			endif
			SetDataFolder $projectPath
			if (isOkay ==1)
				DuplicateDataFolder :FitTemp, Setup
				//duplicate /o :FitTemp:STsetup :Setup:STsetup 
				//duplicate /o :FitTemp:selSTsetup :Setup:selSTsetup
				//duplicate /o :FitTemp:Numerics :Setup:Numerics 
				//duplicate /o :FitTemp:selNumerics :Setup:selNumerics
			endif
		endif
		if (DataFolderExists("Setup"))
			SavedNumPeaks = numPeaks
		endif
		//DuplicateDataFolder :FitTemp:Peaks, :Peaks
		killwaves /z STsetup, selSTsetup, Numerics, selNumerics
		killwaves /z W_Coef, Min_Limit, Max_Limit, epsilon, hold, W_Sigma, T_Constraints, InitializeCoef
		killvariables /z V_numNaNs, V_numINFs, V_npnts, V_nterms, V_nheld, V_startRow, V_endRow, V_startCol, V_endCol
		killvariables /z V_startLayer, V_endLayer, V_startChunk, V_endChunk, V_FitTol, V_FitMaxIters, V_FitOptions
		
		//now make a hard-copy of all relevant variables, use this, if the project is re-opened to initialize everything properly
	
		if (DataFolderExists("Setup"))
			duplicate /o FitControls :Setup:FitControls
		endif
	
		killwaves /z FitControls
		if (DataFolderExists("FitTemp"))
			killdataFolder :FitTemp
		endif
	endif
	
	SetDataFolder $StartingDirectory
	//saved = 1 //the one which was already opened was saved at least once
	//projectStarted = 1
	//now step into "toOpen"
	SetDataFolder :$toOpen
	
	variable /G V_FitMaxIters
	variable /G V_FitOptions 
	variable /G V_FitTol
	NVAR toleranceLocal = V_FitTol
	NVAR tolerance = root:STFitAssVar:V_FitTol
	toleranceLocal = tolerance
	NVAR iterations = V_FitMaxIters
	NVAR fitOptions = V_FitOptions

	
	ProjectPath = GetDataFolder(1)
	//now populate the global variables with meaningful values
	
	if (DataFolderExists("Setup"))
		SetDataFolder :Setup
		duplicate /o STsetup, $(ProjectPath+"STsetup")
		//duplicate /o :Setup:STsetup :STsetup
		
		duplicate /o selSTsetup, $(ProjectPath+"selSTsetup")
		duplicate /o Numerics, $(ProjectPath+"Numerics")
		duplicate /o selNumerics, $(ProjectPath+"selNumerics")
		duplicate /o FitControls, $(ProjectPath+"FitControls")
		//duplicate /o :Setup:selSTsetup :selSTsetup
		//duplicate /o :Setup:Numerics :Numerics
		//duplicate /o :Setup:selNumerics :selNumerics
		//duplicate /o :Setup:FitControls :FitControls
		SetDataFolder $ProjectPath
		wave /t FitControls = FitControls
		wave /t setup = STsetup
		RenameDataFolder :Setup, FitTemp
		SavedNumPeaks = numPeaks
	endif
//	if( DataFolderExists("FitTemp") && !DataFolderExists("Setup"))
//		Variable /G :FitTemp:SavedNumPeaks = numPeaks
//	endif
	fitMin = str2num(FitControls[0][0])
	fitMax = str2num(FitControls[1][0])
	tolerance = str2num(FitControls[2][0])
	iterations = str2num(FitControls[3][0])
	fitOptions = str2num(FitControls[4][0])
	projectName = FitControls[5][0]
	nameWorkWave = FitControls[6][0]
	XRawWave = FitControls[7][0]
	peakType = FitControls[8][0]
	peakTypeTemp = FitControls[9][0]
	background = FitControls[10][0]
	numPeaks = str2num(FitControls[11][0])
	peakToExtract = numPeaks
	savedNumPeaks = numPeaks
	
	if (DataFolderExists("Setup"))
		RenameDataFolder :Setup, FitTemp
//		Variable /G :FitTemp:SavedNumPeaks = numPeaks
	endif
	
	if (WaveExists(STsetup))
		setup2waves()
	endif
	
	SetVariable InputMaxFitIterations, win=$"CursorPanel", limits={1,100,1},value=iterations
	SetVariable InputRemoveLink,win=$"CursorPanel", limits={1,numPeaks,1}, value=peakToExtract
	peakToLink=0
	SetVariable InputSetLink, win=$"CursorPanel", limits={0,numPeaks,1},value= peakToLink
	if (fitOptions ==1)
		CheckBox RobustCheck, win=$"CursorPanel", value= 1
	else
		CheckBox RobustCheck, win=$"CursorPanel", value= 0
	endif
	
	//NewDataFolder /o FitTemp
	
	WaveStats /Q $nameWorkWave
	
	
	
	clearGraphs()	
	//now do the display

	strswitch(XRawWave)                                //have a look at the string with the name of the x-axis wave
		case "":                                             //if empty
			AppendToGraph /w=CursorPanel#guiCursorDisplayFit $nameWorkWave 
			ModifyGraph  /w = CursorPanel#guiCursorDisplayFit rgb($"#0")=(39168,39168,39168) 
			AppendToGraph /w=CursorPanel#guiCursorDisplay $nameWorkWave                     
			break
		case "_calculated_":
			AppendToGraph /w=CursorPanel#guiCursorDisplayFit $nameWorkWave
			ModifyGraph  /w = CursorPanel#guiCursorDisplayFit rgb($"#0")=(39168,39168,39168)  
			AppendToGraph /w=CursorPanel#guiCursorDisplay $nameWorkWave 
			break
		default:                                                 // if not empty
		       wave xraw = $XRawWave
			AppendToGraph /w=CursorPanel#guiCursorDisplayFit $nameWorkWave vs $XRawWave
			ModifyGraph  /w = CursorPanel#guiCursorDisplayFit rgb($"#0")=(39168,39168,39168) 
			AppendToGraph /w=CursorPanel#guiCursorDisplay  $nameWorkWave vs $XRawWave
			break
	endswitch
	SetAxis /A /R /w = CursorPanel#guiCursorDisplay bottom
	SetAxis /A /R /w = CursorPanel#guiCursorDisplayFit bottom
	SetAxis /w = CursorPanel#guiCursorDisplay left *, 1.02*V_max
	//SetAxis /w = CursorPanel#guiCursorDisplayFit left 0.9*V_min, 1.04*V_max
	
	//append not the local reference of the wave,since it does not exist globally. Use the $WaveNameString notation instead
	Cursor /H=1 /F /P /S=0 /L=1 /W=CursorPanel#guiCursorDisplay A $nameWorkWave, 0.55, 0.2
	ModifyGraph /w= CursorPanel#guiCursorDisplay zero(left)=2 
	ModifyGraph  /w = CursorPanel#guiCursorDisplay mirror(left)=2
	ModifyGraph  /w = CursorPanel#guiCursorDisplay mirror(bottom)=2
	ModifyGraph  /w = CursorPanel#guiCursorDisplay rgb($"#0")=(0,26112,39168) 
	ModifyGraph  /w = CursorPanel#guiCursorDisplay mode($"#0")=7, hbfill($"#0")=5
	ModifyGraph /w= CursorPanel#guiCursorDisplayFit zero(left)=2 
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit mirror(left)=2
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit mirror(bottom)=2, nticks(left)=0
	ModifyGraph /w = CursorPanel#guiCursorDisplayFit axRGB(left)=(34816,34816,34816),tlblRGB(left)=(34816,34816,34816), alblRGB(left)=(34816,34816,34816)
	ModifyGraph /w = CursorPanel#guiCursorDisplayFit axRGB(bottom)=(34816,34816,34816),tlblRGB(bottom)=(34816,34816,34816), alblRGB(bottom)=(34816,34816,34816) 
	
	Label  /w = CursorPanel#guiCursorDisplayFit Bottom "\\f01 binding energy (eV)"
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit minor(bottom)=1,sep(bottom)=2
	Label  /w = CursorPanel#guiCursorDisplay Bottom " "
	ModifyGraph  /w = CursorPanel#guiCursorDisplay minor(bottom)=1,sep(bottom)=2
	ModifyGraph  /w = CursorPanel#guiCursorDisplay minor(bottom)=1,sep(bottom)=2
	TextBox /C /w = CursorPanel#guiCursorDisplay /N=text0/F=0/A=LT "Drag the crosshair to a peak\rPress 'Add Peak'\rPress 'Start Fit'"
	
	strswitch(ctrlName)
		case "FromImport":
			
		break
		default:
			saved = 1
		break
	endswitch
	
	if (WaveExists(STsetup))
		strswitch(setup[0][2])
		case "":
			updateFitDisplay("void")  //only the starting values are there
			break
		default:
			updateFitDisplayFinalVal("void")
			break
		endswitch
	endif
	
	strswitch(ctrlName)
		case "preview":
			SetDataFolder $StartingDirectory
		break
		default:
			KillSTOpenProjectPanel("afterOpening")
			break
	endswitch
	
	string displayString = "" 
	//sprintf displaystring, "Parent Data Folder: %s \rCurrent Project: %s ", StartingDirectory, projectName
	sprintf displaystring, "Parent Data Folder:\t%s \rCurrent Project:\t\t\t%s", StartingDirectory, ProjectName
	TitleBox notify1, win=CursorPanel, title=displayString
	sprintf displaystring,"Filename:\t\t\t\t\t\t%s ", nameWorkWave
	TitleBox notify2, win=CursorPanel, title=displayString
	savedLast = 1
end

function getPeakNumber()
	SVAR peakType = root:STFitAssVar:PR_PeakType  //later on the number of coefficients changes with the peak function,
	wave /t setup = STsetup
	variable length = DimSize(setup,0)
	variable numCoef 
	variable numPeaks
	
	strswitch(peakType)
		case "MultiSK":
			numCoef = 15
			
		break
		default:
			numCoef = 6
		break
		endswitch
	numPeaks = max(0, (length-5)/numCoef)
	return numPeaks
	

end



function ClearGraphs()
	variable i
	//SetActiveSubWindow CursorPanel#guiCursorDisplay
	string traces = TraceNameList("CursorPanel#guiCursorDisplay", ";", 1)
	variable items = itemsInList(traces,";")

	for ( i =0; i< items; i += 1)   
		RemoveFromGraph /W=CursorPanel#guiCursorDisplay /Z $"#0"  //always remove the bottom curve
	endfor
		
	traces = TraceNameList("CursorPanel#guiCursorDisplayFit", ";", 1)
	items = itemsInList(traces,";")
	for ( i = 0; i < items; i += 1)
		RemoveFromGraph /W=CursorPanel#guiCursorDisplayFit /Z $"#0"
	endfor
end

Function SelectFitProject(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	SVAR nextProjectToOpen = root:STFitAssVar:nextProjectToOpen
	//this function updates the selection of the project which should be opened
	nextProjectToOpen = popStr
	OpenFitProject("preview")
end


function /s getValidDataFolders()
	string datafolders = ""
	variable i
	Variable numDataFolders = CountObjects(":",4)
	string parent = GetDataFolder(1)
	for(i=0; i<numDataFolders; i+=1)
		String nextPath = GetIndexedObjName(":",4,i)
		//now check if this datafolder contains a subfolder "Setup" and a wave STsetup, if so it is a valid fit project
		SetDataFolder nextPath
		if (DataFolderExists("Setup"))
			SetDataFolder :$"Setup"
			if (exists("STsetup"))
				datafolders += nextPath + ";"
			endif
		else 
			if (exists("STsetup"))
				datafolders += nextPath + ";"
			endif
		endif
		SetDataFolder parent
	endfor

	return datafolders
end

function CallSTNewProjectPanel(ctrlName):ButtonControl
	string ctrlName
	wave /t STsetup = STsetup
	NVAR ProjectStarted = root:STFitAssVar:projectStarted
	NVAR ProjectSaved = root:STFitAssVar:savedLast
	variable WorkedInProject = DimSize(STsetup,0)
	
	if  (ProjectStarted == 1 && ProjectSaved == 0)
		DoAlert 1, "There are unsaved changes in the current project. Proceed anyway?"
		if (V_Flag ==2) //user pressed no
			return -1
		elseif (V_Flag == 1)  //user pressed yes
			//restart the entire thing
			destroyCursorPanel("restart") //now,we are using the string as a switch for a restart
			LaunchCursorPanel()
		endif  

		
	elseif  (ProjectStarted == 1 && ProjectSaved == 1)
	
		destroyCursorPanel("void")
		LaunchCursorPanel()
	endif
	

	
	DoWindow STNewProjectPanel   //this will write "1" into V_Flag if the window is open already
	if (V_Flag )
		DoWindow /F STNewProjectPanel  // if it is already open, bring it to front
		return -1
	endif
	initSTNewProjectPanel()
	execute "STNewProjectPanel()"
	DeActivateButtons(0)  // 0 means all of them
	Button LoadNewBtn,  win=CursorPanel, disable= 0	
	    //make it right for the datafolder
end



Window STNewProjectPanel() : Panel

//	//initSTNewProjectPanel()
//	PauseUpdate; Silent 1		// building window...
//	NewPanel /K=2 /W=(107,73,384,548) as "Start a new project"
//	ModifyPanel cbRGB=(65534,65534,65534)
//	PopupMenu WavePop,pos={24,69},win=STnewProjectPanel,size={240,21},bodyWidth=180,proc=initCursorPopUp,title="Spectrum    "
//	PopupMenu WavePop,mode=1,popvalue="please select a wave",value= #"WaveList(\"!*Disp*\",\";\",\"\")"
//	PopupMenu WavePop2,win=STnewProjectPanel,pos={34,98},size={230,21},bodyWidth=180,proc=initXWavePopUp,title="Binding En.   "
//	PopupMenu WavePop2,mode=1,popvalue="_calculated_ ",value= #"\"_calculated_;\"+ WaveList(\"*\", \";\", \"\")"
//	PopupMenu TemplatePop,win=STnewProjectPanel,pos={48,261},size={216,21},bodyWidth=157, proc=SetTemplate,title="Template    "
//	PopupMenu TemplatePop,mode=1,popvalue="None",value=STtemplateList()
//	PopupMenu TypePop,pos={43,171},size={221,21},bodyWidth=157,proc=ReadType_CursorPanel,title="Peak Type   "
//	PopupMenu TypePop,mode=1,popvalue="Singlet",value= #"\"Singlet;Multiplet\"\t"
// //	PopupMenu BackgroundPop,pos={40,204},size={224,21},bodyWidth=157,proc=ReadBackground_CursorPanel,title="Background  "
// //	PopupMenu BackgroundPop,mode=2,popvalue="Shirley Kombi",value=#"\"Shirley Kombi\"\t"   //#"\"None;Line + Shirley;Parabola + Shirley\"\t"
	
//	Button showbut,pos={147,318},size={120,35},proc=StartNewProject,title="Do it"
//	Button CancelBtn,pos={12,318},size={120,35},proc=KillSTNewProjectPanel,title="Cancel"
	
//	SetVariable setvar0,pos={18,36},size={245,16},title="Project Name    "
//	SetVariable setvar0,value= root:STFitAssVar:ProjectName
//	GroupBox group1,pos={10,140},size={266,163},title="Choose a model",fStyle=1
//	GroupBox group2,pos={24,240},size={247,54},title="or select a template",fStyle=1
//	GroupBox group3,pos={9,9},size={265,126},title="Data",fStyle=1
	
//	SetDrawLayer UserBack
//	DrawRect 12,363,266,463
//	DrawText 28,386,"The project will be saved as subfolder"
//	DrawText 28,401,"in the current parent data folder."
//	SetDrawEnv fstyle= 1
//	DrawText 28,423,"Use  \"Data/Data Browser\" to navigate"
//	SetDrawEnv fstyle= 1
//	DrawText 28,439,"to a different data folder."
//	DrawText 73, 222, "Use Multiplet for 'hard' constraints."
//	SetDrawEnv fstyle= 1
//	DrawText 28,454,"(use red arrow + mouse)"
	
	
	
	PauseUpdate; Silent 1		// building window...
	NewPanel /K=2 /W=(107,73,384,582) as "Start a new project"
	ModifyPanel cbRGB=(65534,65534,65534)
	SetDrawLayer UserBack
//	DrawPict 12,363,0.567,0.567,ProcGlobal#NotificationNewXPSTProject
	
	DrawRect 12,363,266,500
	DrawText 28,384,"The project will be saved as subfolder"
	DrawText 28,399,"in the current parent data folder."
	SetDrawEnv fstyle= 1
	DrawText 26,426,"To change the parent folder:"
	SetDrawEnv fstyle= 1
	DrawText 28,449,"Close the 'Fit Assistant' and use"
	SetDrawEnv fstyle= 1
	DrawText 28,465,"\"Data/Data Browser\" to select"
	SetDrawEnv fstyle= 1
	DrawText 28,480,"a different data folder."
	SetDrawEnv fstyle= 1
	DrawText 27,495,"(mark it with the red arrow)"
	SetDrawEnv fstyle= 1
	DrawText 26,433,"--------------------------------------------------------"
	PopupMenu WavePop,pos={24,69},size={240,21},bodyWidth=180,proc=initCursorPopUp,title="Spectrum    "
	PopupMenu WavePop,mode=1,popvalue="please select a wave",value= #"WaveList(\"!*Disp*\",\";\",\"\")"
	PopupMenu WavePop2,pos={18,98},size={246,21},bodyWidth=180,proc=initXWavePopUp,title="Binding En.   "
	PopupMenu WavePop2,mode=1,popvalue="_calculated_ ",value= #"\"_calculated_;\"+ WaveList(\"*\", \";\", \"\")"
	PopupMenu TemplatePop,pos={48,261},size={216,21},bodyWidth=157,proc=SetTemplate,title="Template    "
	PopupMenu TemplatePop,mode=1,popvalue="None",value= #"STtemplateList()"
	PopupMenu TypePop,pos={43,171},size={221,21},bodyWidth=157,proc=ReadType_CursorPanel,title="Peak Type   "
	PopupMenu TypePop,mode=1,popvalue="Singlet",value= #"\"Singlet;Multiplet\"\t"
	Button showbut,pos={147,318},size={120,35},proc=StartNewProject,title="Do it"
	Button CancelBtn,pos={12,318},size={120,35},proc=KillSTNewProjectPanel,title="Cancel"
	SetVariable setvar0,pos={18,36},size={245,16},title="Project Name    "
	SetVariable setvar0,value= root:STFitAssVar:ProjectName
	GroupBox group1,pos={10,140},size={266,163},title="Choose a model",fStyle=1
	GroupBox group2,pos={24,240},size={247,54},title="or select a template",fStyle=1
	GroupBox group3,pos={9,9},size={265,126},title="Data",fStyle=1
	Button showbut1,pos={104,197},size={160,23},proc=CallPeakTypeHelp,title="Explain the peak types"
EndMacro


function initSTNewProjectPanel()
	SVAR StartingFolder = root:STFitAssVar:ST_StartDirectory
	
	SetDataFolder StartingFolder
end

function CallPeakTypeHelp(ctrlName):ButtonControl
	string ctrlName
	DisplayHelpTopic "Singlet and Multiplet peaks"
end


function KillSTNewProjectPanel(ctrlName):ButtonControl
	string ctrlName
	SVAR workingFolder = root:STFitAssVar:CP_workingFolder
	NVAR projectStarted = root:STFitAssVar:projectStarted
	SetDataFolder workingFolder
	DoWindow /K STNewProjectPanel
	if (projectStarted == 0)
		ActivateButtons(1)
	else
		ActivateButtons(0)
	endif
end


function StartNewProject(ctrlName):ButtonControl
	string ctrlName

	SVAR currentFolder = root:STFitAssVar:CP_workingFolder 
	SVAR RawXWave = root:STFitAssVar:PR_XRawDataCursorPanel
	SVAR RawYWave =root:STFitAssVar:PR_nameWorkWave
	SVAR StartingFolder = root:STFitAssVar:ST_StartDirectory
	SVAR ProjectName = root:STFitAssVar:ProjectName
	SVAR templateName = root:STFitAssVar:templateName
	
	NVAR useTemplate = root:STFitAssVar:useTemplate
	
	NVAR ProjectStarted = root:STFitAssVar:projectStarted
	

	StartingFolder = GetDataFolder(1)  //remember this ... in order leave the project folder later on
	
	wave yraw = $RawYWave
	variable invalid = 0
	
	strswitch(RawYWave)
		case "":
			DoAlert 0, "Please provide a Spectrum"
		return -1
		break
	endswitch
	
	strswitch(ProjectName)
		case "":
			DoAlert 0, "Please provide a project name"
			return -1
			break
	endswitch
	
	//the same construction is in saveFitAs .... maybe this can be outsourced into a separate function
	variable legal = !GrepString(Projectname,"[^[:alnum:]_+-]") 
	variable letter = GrepString(Projectname,"[[:alpha:]]") 
	variable number =  GrepString(Projectname,"[[:digit:]]")
	variable nameOkay = legal && ( (letter && number) || (letter && !number))
	invalid = ! nameOkay // !nameOkay //look for not alpha-numeric characters
	//this would allow + and - too   [^[:alnum:]+-]

	if (invalid)
		DoAlert 0, "The project name may only contain '+', ' _', '-', letters, and digits.\r \rPlease provide a new project name."
		return -1
	endif
	
	if (DataFolderExists(ProjectName))
		DoAlert 0, "This project name already exists! \rPlease choose a different name."
		return -1
	endif
	
	if (strlen(ProjectName)>27)
		DoAlert 0, "The project name can only be 27 characters long, please provide a new one."
		return -1
	endif
	//make a test, if everything is okay with the global variables
	strswitch(RawYWave)
	case "":
		doalert 0, "No data wave given!"
		Button addBut disable=2 
		return -1
		break
	default:
		break
	endswitch

	if (strlen(RawYWave) > 25)
		string newName
		Prompt newName, "The original wave will be duplicated into the project folder as:"		// Set prompt for x param
		DoPrompt "Wave name is too long (more than 25 characters)", newName
		if (V_Flag)
			return -1								// User canceled
		endif
		if (strlen(newName) == 0 )
			DoAlert 0, "This was too short ..."
			return -1
		endif
		RawYWave = newName
		//DoAlert 0,"The wave name is too long, please rename the wave (Menu: Data).\rThe new name must not exceed 22 characters."
		//return -1
	endif
	
	
	string shiftNameY = ":'" + ProjectName + "':" + "'" + RawYWave + "'"
	
	NewDataFolder $ProjectName
	strswitch(RawXWave)
		case "":
		break
		case "_calculated_":
		break
		default:
			wave xraw = $RawXWave
			string shiftNameX = ":" + ProjectName + ":" + "'" + RawXWave + "'"
			duplicate /o xraw $shiftNameX
		break
	endswitch

	duplicate /o yraw $shiftNameY
	SetDataFolder ProjectName
	
	currentFolder = getDataFolder(1)   //remember where we are ...
	
	variable /G V_FitTol    //those have to remain in the current working directory
	variable /G V_FitMaxIters = 50
	Variable /G V_FitOptions = 0
	NVAR iterations= V_FitMaxIters
	NewDataFolder FitTemp
	NewDataFolder Peaks
	string HostWindowName = "CursorPanel"
	SetVariable  InputMaxFitIterations, win=$HostWindowName, limits={1,100,1},value=iterations
	
	
	
	//strswitch(peakType)
	//case "":
	//	doalert 0, "No peak type given!"
	//	Button addBut disable=2   
	//	return -1
	//default:
	//	break
	//endswitch

	wave WorkWave = $RawYWave 
	Wavestats /Q WorkWave	
	//Remove old traces from the graph, four of them to be on the safe side
	RemoveFromGraph /W=CursorPanel#guiCursorDisplay /Z $"#0" $"#1" $"#2" $"#3" $"#4"
	//this is quite a lot of coding to accomplish such a simple task
	string path = "CursorPanel#guiCursorDisplayFit"
	string traces = TraceNameList(path,";",1)
	variable itemsOnGraph = itemsInList(traces)
	string tempString
	do
		sprintf tempString, "#%d", itemsOnGraph
		RemoveFromGraph /W=CursorPanel#guiCursorDisplayFit /Z $tempString
		itemsOnGraph -= 1
	while ( itemsOnGraph >= 0)
	/// done
	
	
	strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
		case "":                                             //if empty
			AppendToGraph /w=CursorPanel#guiCursorDisplayFit WorkWave 
			ModifyGraph  /w = CursorPanel#guiCursorDisplayFit rgb($"#0")=(39168,39168,39168) 
			AppendToGraph /w=CursorPanel#guiCursorDisplay WorkWave                     
			break
		case "_calculated_":
			AppendToGraph /w=CursorPanel#guiCursorDisplayFit WorkWave
			ModifyGraph  /w = CursorPanel#guiCursorDisplayFit rgb($"#0")=(39168,39168,39168)  
			AppendToGraph /w=CursorPanel#guiCursorDisplay WorkWave 
			break
		default:                                                 // if not empty
		       wave xraw = $RawXWave
			AppendToGraph /w=CursorPanel#guiCursorDisplayFit WorkWave vs xraw
			ModifyGraph  /w = CursorPanel#guiCursorDisplayFit rgb($"#0")=(39168,39168,39168) 
			AppendToGraph /w=CursorPanel#guiCursorDisplay WorkWave vs xraw
			break
	endswitch
	SetAxis /A /R /w = CursorPanel#guiCursorDisplay bottom
	SetAxis /A /R /w = CursorPanel#guiCursorDisplayFit bottom
	SetAxis /w = CursorPanel#guiCursorDisplay left *, 1.02*V_max
	//SetAxis /w = CursorPanel#guiCursorDisplayFit left 0.9*V_min, 1.04*V_max
	
	//append not the local reference of the wave,since it does not exist globally. Use the $WaveNameString notation instead
	Cursor /H=1 /F /P /S=0 /L=1 /W=CursorPanel#guiCursorDisplay A $RawYWave, 0.55, 0.2
	ModifyGraph /w= CursorPanel#guiCursorDisplay zero(left)=2 
	ModifyGraph  /w = CursorPanel#guiCursorDisplay mirror(left)=2
	ModifyGraph  /w = CursorPanel#guiCursorDisplay mirror(bottom)=2
	ModifyGraph  /w = CursorPanel#guiCursorDisplay rgb($"#0")=(0,26112,39168) 
	ModifyGraph  /w = CursorPanel#guiCursorDisplay mode($"#0")=7, hbfill($"#0")=5
	ModifyGraph /w= CursorPanel#guiCursorDisplayFit zero(left)=2 
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit mirror(left)=2
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit mirror(bottom)=2, nticks(left)=0
	ModifyGraph /w = CursorPanel#guiCursorDisplayFit axRGB(left)=(34816,34816,34816),tlblRGB(left)=(34816,34816,34816), alblRGB(left)=(34816,34816,34816)
	ModifyGraph /w = CursorPanel#guiCursorDisplayFit axRGB(bottom)=(34816,34816,34816),tlblRGB(bottom)=(34816,34816,34816), alblRGB(bottom)=(34816,34816,34816) 
	
	Label  /w = CursorPanel#guiCursorDisplayFit Bottom "\\f01 binding energy (eV)"
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit minor(bottom)=1,sep(bottom)=2
	Label  /w = CursorPanel#guiCursorDisplay Bottom " "
	ModifyGraph  /w = CursorPanel#guiCursorDisplay minor(bottom)=1,sep(bottom)=2
	ModifyGraph  /w = CursorPanel#guiCursorDisplay minor(bottom)=1,sep(bottom)=2
	TextBox /C /w = CursorPanel#guiCursorDisplay /N=text0/F=0/A=LT "Drag the crosshair to a peak\rPress 'Add Peak'\rPress 'Start Fit'"
	
	ProjectStarted = 1
	displayWavesCursorPanel("void")
	KillSTNewProjectPanel("foe")
	wave /t FitControls = FitControls
	FitControls[5][0] = ProjectName
	FitControls[6][0] = RawYWave
	FitControls[7][0] = RawXWave
	
	string displayString = "" 
	//sprintf displaystring, "Parent Data Folder: %s \rCurrent Project: %s ", StartingFolder, ProjectName
	sprintf displaystring, "Parent Data Folder:\t%s \rCurrent Project:\t\t\t%s ", StartingFolder, ProjectName
	TitleBox notify1, win=CursorPanel, title=displayString
	sprintf displaystring,"Filename:\t\t\t\t\t\t%s ", RawYWave
	TitleBox notify2, win=CursorPanel, title=displayString
end

function /S STtemplateList()

	string templateList = "None;"
	variable i
	string parent = GetDataFolder(1)
	
	if (DataFolderExists("root:Fit_templates"))
	SetDataFolder root:Fit_templates
	Variable numDataFolders = CountObjects(":",4)
	string nextPath
	for(i=0; i<numDataFolders; i+=1)
		nextPath = GetIndexedObjName(":",4,i)
		templateList += nextPath + ";"
	endfor
	SetDataFolder parent
	return templateList
	else
	return "None;"
	endif


end

function SetTemplate(ctrlName,popNum,popStr) : PopupMenuControl
	string ctrlName
	variable popNum
	string popStr
	NVAR useTemplate = root:STFitAssVar:useTemplate
	SVAR templateName = root:STFitAssVar:templateName
	strswitch(popStr)
	case "None":
		PopupMenu TypePop disable=0
		useTemplate = 0
		templateName = ""
		break
	default:
		PopupMenu TypePop disable=2
		useTemplate = 1
		templateName = popStr
		break
	endswitch
	
end

function displayWavesNewProjectPanel(ctrlName):ButtonControl
	string ctrlName
	string HostWindowName = "CursorPanel"
	//Button addBut win=$HostWindowName, fColor=(44000,52000,65500), disable=0        //(32768,54528,65280)
	//Button DeleteBut win=$HostWindowName, fColor=(44000,52000,65500), disable=0           //(32768,54528,65280)
	//Button FitBut win=$HostWindowName, fColor=(44000,52000,65500)    
	//Button ConstraintBut win=$HostWindowName, fColor=(44000,52000,65500)    
	//displayWavesCursorPanel("foe")
end

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Read the values in the pop-up menus
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//read the name of the raw-data wave
function initCursorPopUp(ctrlName, popNum, popStr) : PopupMenuControl
	string ctrlName //not used here
	variable popNum //not used here
	string popStr
	//if ( CheckLocation() )
	//	PopupMenu WavePop , mode = 1, popvalue= " "
	//	return 1
	//endif
	SVAR value = root:STFitAssVar:PR_nameWorkWave    
	value = popStr 
end

//read the background type
function ReadBackground_CursorPanel(ctrlName,popNum,popStr): PopupMenuControl
	string ctrlName
	variable popNum
	String popStr
	//if ( CheckLocation() )
	//	PopupMenu TypePop , mode = 1, popvalue= " "
	//	return 1
	//endif
	SVAR value = root:STFitAssVar:PR_Background
	//print value
	strswitch(popStr)
		case "Shirley Kombi":
			value = "SK"
		break		
	default:
		value = popStr
	endswitch              
	combinePeakBackground()
	ReDoBackground()
end

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// ListBoxPanels

Window PeakEditor() : Panel
	initializeConstraintTable()  //the window command goes back to the root directory by default, so one has to force it to go back to the correct data folder
	PauseUpdate; Silent 1		// building window...
	//NewPanel  /W=(44,83,1134,530)
	NewPanel /K=1 /W=(44,83,940,700) as "Constraints and initial values"
	//ModifyPanel fixedSize=1
	ListBox list0,pos={6,6},size={880,520},win=PeakEditor, proc=refresh,listWave=STsetup,userColumnResize=1
	ListBox list0,selWave=selSTsetup,colorWave=root:STFitAssVar:myColors
	ListBox list0,widths={200,30,100,100,40,100,100,60}
	Button RefreshButton,pos={638,563},size={122,33},proc=ApplyChanges,title="Apply"
	Button CheckButton,pos={764,563},size={122,33},proc=checkSetup,title="Check 'Hold-Logic'"
	Button HelpCalBut,pos={6,540},size={150,34},proc=callHelp,title="How to get a stable fit"
	Button CallHelpDoNots,pos={6,575},size={150,34},proc=callHelpDoNots,title="What to avoid ..."
	fancyUP("foe")
EndMacro

function ApplyChanges(ctrlName):ButtonControl
	string ctrlName
	updateCoefs()
	updateFitDisplay("void")

end

function callHelpDoNots(ctrlName):ButtonControl
string ctrlName
DisplayHelpTopic "Things to avoid during a XPST fit"

end


function Refresh(ctrlName,row,col,event): ListBoxControl
	string ctrlName
	variable row
	variable col
	variable event
	if (event == 2)
		fancyUp("foe")
	endif
	//updateCoefs()
	//if (event == 7 && col == 3)
	//	updateFitDisplay("void")
	//endif
end

function updateCoefs()
	wave /t source= setup
	variable length = dimsize(source,0)   //0 for rows, 1 for columns
	wave W_coef = W_Coef      //could also be initalizeCoef
	wave init = InitializeCoef
	variable i
	for (i =0; i < length; i += 1)
		init[i] = str2num(source[i][3])
		W_Coef[i] = str2num(source[i][3])
	endfor
	setup2waves()
end



function FancyUp(ctrlName):ButtonControl
	string ctrlName
	wave /t source = STsetup
	wave sw = selSTsetup
	wave myColors = root:STFitAssVar:myColors
	SVAR peakType = root:STFitAssVar:PR_PeakType
	
	SetDimLabel 1,0,$"Legend",source
	SetDimLabel 1,1,$"",source
	SetDimLabel 1,2,$"Fit Results",source
	SetDimLabel 1,3,$"Initial Guess", source
	SetDimLabel 1,4,$"hold", source
	SetDimLabel 1,5,$"Lower Limit", source
	SetDimLabel 1,6,$"Upper Limit", source
	SetDimLabel 1,7,$"epsilon", source

	string planeName = "backColors"
	variable plane = FindDimLabel(sw,2,planeName)

	Variable nplanes = max(1,Dimsize(sw,2))
	if (plane <0)
		Redimension /N=(-1,-1,nplanes+1) sw
		plane = nplanes
		SetDimLabel 2,nplanes,$planeName sw
	endif


	//now do the color coding
	//these are the basic colors
	sw[][][plane] = 4   //4th row in the color wave
	sw[][3][plane] =3
	sw[][5][plane] =3
	sw[][6][plane] =3
	sw[][7][plane] =3
	
	variable length, numpeaks, index, i,j,k
	strswitch(peakType)
	case "SingletSK":
		//now re-colorcode every other peak meaing peak 2, 4, 6, 8, ....
		//first extract the number of peaks
	 	length = DimSize(source,0)
	 	numpeaks = (length-5)/ 6 

		make /d /n=4 /o tempWaveAsArray
		wave tempWave = tempWaveAsArray
		tempWave={3,5,6,7}

		for (i=0; i<numpeaks; i += 1)
			index = 6*i + 5
			if (mod(i,2) != 0)
				continue
			endif
			sw[index][][plane] = 0
			sw[index+1][][plane] = 0
			sw[index+2][][plane] = 0
			sw[index+3][][plane] = 0
			sw[index+4][][plane] = 0
			sw[index+5][][plane] = 0

			for (j= 0; j<4;j+=1)
				k = tempWave[j]

				sw[index][k][plane] = 2
				sw[index+1][k][plane] = 2
				sw[index+2][k][plane] = 2
				sw[index+3][k][plane] = 2
				sw[index+4][k][plane] = 2
				sw[index+5][k][plane] = 2
			endfor
		endfor


		planeName = "foreColors"   //used to be backColors
		plane = FindDimLabel(sw,2,planeName)
		nplanes = max(1,Dimsize(sw,2))
		if (plane <0)
			Redimension /N=(-1,-1,nplanes+1) sw
			plane = nplanes
			SetDimLabel 2,nplanes,$planeName sw
		endif

		for ( i = 0; i < length; i += 1)
			if (sw[i][4] == 48)
				sw[i][5][0] = 0
				sw[i][6][0] = 0
				sw[i][5][plane] = 4
				sw[i][6][plane] = 4
			else
				sw[i][5][0] = (0x02)
				sw[i][6][0] = (0x02)
				sw[i][5][plane] = 0
				sw[i][6][plane] = 0
			endif
		endfor
	break
	case "MultiSK":                                                //begin section for multiplet
		length = DimSize(source,0)
		numpeaks = (length-5)/15 

		make /d /n=4 /o tempWaveAsArray
		wave tempWave = tempWaveAsArray
		//stopped for now ....
		tempWave={3,5,6,7}

		for (i=0; i<numpeaks; i += 1)
			index = 15*i + 5
			if (mod(i,2) != 0)
				continue
			endif
			sw[index][][plane] = 0
			sw[index+1][][plane] = 0
			sw[index+2][][plane] = 0
			sw[index+3][][plane] = 0
			sw[index+4][][plane] = 0
			sw[index+5][][plane] = 0
			sw[index+6][][plane] = 0
			sw[index+7][][plane] = 0
			sw[index+8][][plane] = 0
			sw[index+9][][plane] = 0
			sw[index+10][][plane] = 0
			sw[index+11][][plane] = 0
			sw[index+12][][plane] = 0
			sw[index+13][][plane] = 0
			sw[index+14][][plane] = 0

			for (j= 0; j<4;j+=1)
				k = tempWave[j]

				sw[index][k][plane] = 2
				sw[index+1][k][plane] = 2
				sw[index+2][k][plane] = 2
				sw[index+3][k][plane] = 2
				sw[index+4][k][plane] = 2
				sw[index+5][k][plane] = 2
				sw[index+6][k][plane] = 2
				sw[index+7][k][plane] = 2
				sw[index+8][k][plane] = 2
				sw[index+9][k][plane] = 2
				sw[index+10][k][plane] = 2
				sw[index+11][k][plane] = 2
				sw[index+12][k][plane] = 2
				sw[index+13][k][plane] = 2
				sw[index+14][k][plane] = 2
			endfor
		endfor


		planeName = "foreColors"   //used to be backColors
		plane = FindDimLabel(sw,2,planeName)
		nplanes = max(1,Dimsize(sw,2))
		if (plane <0)
			Redimension /N=(-1,-1,nplanes+1) sw
			plane = nplanes
			SetDimLabel 2,nplanes,$planeName sw
		endif

		for ( i = 0; i < length; i += 1)
			if (sw[i][4] == 48)
				sw[i][5][0] = 0
				sw[i][6][0] = 0
				sw[i][5][plane] = 4
				sw[i][6][plane] = 4
			else
				sw[i][5][0] = (0x02)
				sw[i][6][0] = (0x02)
				sw[i][5][plane] = 0
				sw[i][6][plane] = 0
			endif
		endfor
	break                                                            //end of section for multiplet
	default:
		DoAlert 0, "Peak type not recognized in function FancyUp() "
		break
	endswitch
	
	killwaves /z tempWaveAsArray
	//Listbox list0, colorWave=myColors

end




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////Peak Viewer and depending functions
proc CallPeakViewer(ctrlName):ButtonControl
	string ctrlName
	DoWindow PeakViewer//ConstraintTable
	if ( V_Flag )
		DoWindow /F PeakViewer
		return -1
	endif
	PeakViewer()
	//DoWindow /T PeakViewer, "Numerical Results"
end


Window PeakViewer() : Panel
	initializeConstraintTable()         //for the lack of an appropriately named function
	PauseUpdate; Silent 1		// building window...
	NewPanel /K=1 /W=(25,60,1230,520) as "Numerical Results" 
	ModifyPanel fixedSize=1
	ListBox NumericList,pos={1,2},win=PeakViewer,size={1200,400},listWave=Numerics, userColumnResize=1
	ListBox NumericList,selWave=selNumerics
	ListBox NumericList,colorWave=root:STFitAssVar:myColors
	ListBox NumericList,widths={80,120,80,120,80,120,80,120,80,150,80,150,80,0,0,0}
	
	Button PlotBtn,pos={2,418},size={160,34},proc=makeReport,title="Export Report"             
	Button PeakParaHelp, pos={166,418}, size={160,34}, proc=callHelpPeakParameters, title="Explain those parameters"
	fancyUpViewer()
	
EndMacro


function callHelpPeakParameters(ctrlName):ButtonControl
	string ctrlName
	DisplayHelpTopic "Peak parameters in XPST"

end


function makeReport(ctrlName):ButtonControl
string ctrlName
//saveFit("foe")
PlotAndSort("void")
end


function fancyUpViewer()
	string ctrlName
	wave /t source = Numerics
	wave sw = selNumerics
	wave myColors = root:STFitAssVar:myColors
	SVAR peakType = root:STFitAssVar:PR_PeakType
	//SetDimLabel 1,0,$"Legend",source
	//SetDimLabel 1,1,$"",source
	//SetDimLabel 1,2,$"Fit Results",source
	//ListBox list0, widths={120,30,100,100,40,100,100,60}
	//SetDimLabel 1,3,$"Initial Guess", source
	//SetDimLabel 1,4,$"hold", source
	//SetDimLabel 1,5,$"Lower Limit", source
	//SetDimLabel 1,6,$"Upper Limit", source
	//SetDimLabel 1,7,$"epsilon", source

	string planeName = "backColors"
	variable plane = FindDimLabel(sw,2,planeName)

	Variable nplanes = max(1,Dimsize(sw,2))
	if (plane <0)
		Redimension /N=(-1,-1,nplanes+1) sw
		plane = nplanes
		SetDimLabel 2,nplanes,$planeName sw
	endif


	//now do the color coding
	//these are the basic colors
	sw[][][plane] = 4   //4th row in the color wave
	//sw[][3][plane] =3
	//sw[][5][plane] =3
	//sw[][6][plane] =3
	//sw[][7][plane] =3


	//now re-colorcode every other peak meaing peak 2, 4, 6, 8, ....
	//first extract the number of peaks
	variable length = DimSize(source,0)
	
	variable numpeaks
	
	variable index, i,j,k
	
	strswitch(peakType)
	case "MultiSK":
		numpeaks= (length)/ 16 
		for (i=0; i<numpeaks; i += 1)
			index = 16*i 
		
			if (mod(i,2) != 0)
				continue
			endif
			for ( j = 0; j < 16; j += 1)
			sw[index + j][][plane] = 0
			//sw[index+1][][plane] = 0
			//sw[index+2][][plane] = 0
			//sw[index+3][][plane] = 0
			endfor
		endfor
	break
	default:
		numpeaks= (length)/ 4 
		for (i=0; i<numpeaks; i += 1)
			index = 4*i 
		
			if (mod(i,2) != 0)
				continue
			endif
			
			sw[index][][plane] = 0
			sw[index+1][][plane] = 0
			sw[index+2][][plane] = 0
			sw[index+3][][plane] = 0
		endfor
	break
	endswitch
	


end



function waves2Setup()
	//that one is actually only concerned with W_Coef
	Wave /t source = STsetup
	wave fitResults = W_Coef
	
	variable length = DimSize(source,0)	
	variable i
	for ( i = 0; i < length; i += 1 )
		source[i][2] = num2str(fitResults[i])
	endfor
end


function setup2Waves()

variable i,j
//print length
//variable Summe = 0
string tempStr
variable item

wave /t source= STsetup
wave sw = selSTsetup

variable length = dimsize(source,0)   //0 for rows, 1 for columns

make /d /n=(length) /o hold, W_Coef, InitializeCoef, epsilon
make /t /n=(length) /o Min_Limit, Max_Limit
make /t /n=(2*length) /o T_Constraints

wave hold = hold
wave W_coef = W_Coef
wave init = InitializeCoef
wave eps = epsilon

wave /t allConstraints = T_Constraints
wave /t minLim = Min_Limit
wave /t maxLim = Max_Limit

hold = sw[p][4]

for (i =0; i < length; i += 1)
	if (hold[i] ==32)
		hold[i] = 0
	else
		hold[i] = 1
	endif
endfor

// take the textwaves
minLim = source[p][5]
maxLim=source[p][6]

string kList = ""
string paraList = ""

//generate a list with the k's and with the parameters used in the setup
for ( i = 0; i < length; i += 1 )
	kList +=   "K" + num2str(i) + ";"
	paraList +=  source[i][1] + ";"
endfor
//print kList, paraList

//now exchange all a1,a2, etc with the correpsonding K0,K1, etc. make sure to add K0< .... and K0> .... in the output waves

for ( j = 0; j < ItemsInList(kList); j += 1 )
	for ( i = 0; i < length; i += 1 )
		minLim[i] =  ReplaceString(StringFromList(j,paraList) ,minLim[i],StringFromList(j,kList))   //remember, if no substitution is done, ReplaceString just returns the original string
		maxLim[i] =  ReplaceString(StringFromList(j,paraList) ,maxLim[i],StringFromList(j,kList))
	endfor
endfor

for ( j = 0; j < ItemsInList(kList); j += 1 )
		minLim[j] = StringFromList(j,kList) + " > " + minLim[j] + "- 0.0001"
		maxLim[j] = StringFromList(j,kList) + " < " + maxLim[j] +  "+ 0.0001"		
endfor
//and now do the numerical waves
for (i =0; i < length; i += 1)
	init[i] = str2num(source[i][3])
	W_Coef[i] = str2num(source[i][2])
	eps[i] = str2num(source[i][7])
endfor

for (i=0; i < numpnts(minLim); i+=1)
	allConstraints[2*i] = minLim[i]
	allConstraints[2*i+1] = maxLim[i]
endfor


end


Window GetAreaFactors() : Panel
	PauseUpdate; Silent 1		// building window...
	//NewPanel /K=2 /W=(150,77,397,256)

	//Button ConfirmBtn,win=GetAreaFactors,pos={120,137},proc=CloseGetAreaFactors,size={112,32},title="Use it and Close"
//	SetVariable setUpperFactor,pos={6,99},size={227,16},title="Factor for upper limit:  B ", limits={0.001,100,0}, value = root:STFitAssVar:AreaLinkFactorHigh
//	SetVariable setUpperFactor1,pos={8,69},size={227,16},title="Factor for lower limit:  A ", limits={0.001,100,0}, value = root:STFitAssVar:AreaLinkFactorLow
//	SetDrawLayer UserBack
//	DrawText 21,27,"(peak area)  >  A  x  (area parent peak)"
//	DrawText 20,47,"(peak area)  <  B  x  (area parent peak)"

	NewPanel /K=2 /W=(150,77,399,350)
	SetDrawLayer UserBack
	DrawRect 9,179,240,261
	DrawText 21,27,"(peak area)  >  A  x  (area parent peak)"
	DrawText 20,47,"(peak area)  <  B  x  (area parent peak)"
	DrawText 20,214,"For multiplet peaks, this factor"
	DrawText 20,229,"acts on the first subpeaks only!"
	Button ConfirmBtn,pos={122,131},size={112,32},proc=CloseGetAreaFactors,title="Use it and Close"
	SetVariable setUpperFactor,pos={6,99},size={227,16},title="Factor for upper limit:  B "
	SetVariable setUpperFactor,limits={0.001,100,0},value= root:STFitAssVar:AreaLinkFactorHigh
	SetVariable setUpperFactor1,pos={8,69},size={227,16},title="Factor for lower limit:  A "
	SetVariable setUpperFactor1,limits={0.001,100,0},value= root:STFitAssVar:AreaLinkFactorLow


	DeActivateButtons(0)
EndMacro


function CloseGetAreaFactors(ctrlName):ButtonControl
	string ctrlName
	NVAR areaFactorMin = root:STFitAssVar:AreaLinkFactorLow
	NVAR areaFactorMax = root:STFitAssVar:AreaLinkFactorHigh
	variable temp
	
	//make sure that high and low limits are sound
	temp = areaFactorMin
	areaFactorMin = min(areaFactorMin,areaFactorMax)
	areaFactorMax = max(temp, areaFactorMax)
	
	//now close
	DoWindow /K GetAreaFactors
	ActivateButtons(0)
end

Window GetOffsetInterval() : Panel
	PauseUpdate; Silent 1		// building window...
	NewPanel /K=2 /W=(150,77,411,255)
	Button ConfirmBtn, win=GetOffsetInterval,pos={120,132},size={112,32},proc=CloseGetOffsetInterval,title="Use it and Close"
	SetVariable setUpperOffset,pos={6,99},size={227,16},title="Upper Limit for Offset:   B ", limits={-100,100,0}, value = root:STFitAssVar:PositionLinkOffsetMax
	SetVariable setLowerOffset,pos={8,69},size={227,16},title="Lower Limit for Offsett:  A", limits={-100,100,0}, value = root:STFitAssVar:PositionLinkOffsetMin
	SetDrawLayer UserBack
	DrawText 6,26,"(peak position)  >  (position parent peak) + A"
	DrawText 6,44,"(peak position)  <  (position parent peak) + B"
	DeActivateButtons(0)
	
EndMacro

function CloseGetOffsetInterval(ctrlName):ButtonControl
	string ctrlName
	NVAR offSetMin = root:STFitAssVar:PositionLinkOffsetMin
	NVAR offSetMax = root:STFitAssVar:PositionLinkOffsetMax
	variable temp
	
	//make sure that high and low limits are sound
	temp = offsetMin
	offsetMin = min(offsetMin,offsetMax)
	offsetMax = max(temp, offsetMax)
	
	DoWindow /K GetOffsetInterval
	ActivateButtons(0)
end

function callHelp(ctrlName):ButtonControl
string ctrlName
DisplayHelpTopic "Generating a stable fit setup"

end

function checkSetup(ctrlName): ButtonControl
string ctrlName
//this has to be called upon checking or unchecking and editing in [5] and [6]

wave /t source= STsetup
variable i,j,k
variable length = dimsize(source,0)   //0 for rows, 1 for columns
string tempStr
variable item

wave sw = selSTsetup
string matchString

string planeName = "backColors"   //used to be backColors
variable plane = FindDimLabel(sw,2,planeName)
variable nplanes = max(1,Dimsize(sw,2))
variable Errors = 0
string tempString
variable refuseHold

if (plane <0)
	Redimension /N=(-1,-1,nplanes+1) sw
	plane = nplanes
	SetDimLabel 2,nplanes,$planeName sw
endif



for ( i = 0; i < length; i += 1)
	if (sw[i][4] == 48)   //this parameter is set to be constant does it show up anywhere else?
		matchString = "*" + source[i][1] +"*"


		for (j=0; j < length; j += 1)
			tempString =source[j][5]
			refuseHold = StringMatch(tempString,matchString)
			if (refuseHold != 0 && sw[j][4] == 32)
				Errors += 1
				sw[j][5][plane] =1
			endif
			tempString =source[j][6]
			refuseHold = StringMatch(tempString,matchString)
			if (refuseHold != 0 && sw[j][4] == 32)
				Errors += 1
				sw[j][6][plane] =1	
			endif
		endfor
	endif	
endfor

		if (Errors != 0)
			tempString = "\rNo parameter which is on hold is allowed to show up as active in 'Lower Limit' or 'Upper Limit'. Remove it from the limits or freeze the depending parameter too."
			Doalert 0, tempString
			
		else
			tempString = "\rThe logic between hold and constraints seems to be fine. \r\rMake sure all your initial conditions are consistent with the constraints.\rNote: Use constraints only 'backwards':\r\r a4 =>2*a2   is   GOOD    ..... a2 =>0.5*a4  is  BAD."
			//DoAlert 0, tempString 
			FancyUp("foe")
		endif
end


function ActivateButtons(set)
	variable set    // 0, 1, 2. ... do define which buttons are to be switched on
	
	if ( set == 0)
	Button LoadNewBtn,  win=CursorPanel, disable= 0			 //title="  New Fit Project  >>"
	Button ImportFitBut,  win=CursorPanel, disable= 0			//title="Import Fit Project    >>"
	Button OpenProjectBtn,  win=CursorPanel, disable= 0		//title="Open Fit Project"
	Button ReportViewerBtn, win=CursorPanel, disable= 0		//title="View Numeric Results"
	Button FinePlotBtn, win=CursorPanel, disable=0 	 			//title="Permanent Plot (saves last)"
	Button addBut, win=CursorPanel, disable=0 					//title=" Add Peak   >>"
	Button DeleteBut, win=CursorPanel, disable=0				//title=" Delete Peak  <<"
	Button CleanBut, win=CursorPanel, disable=0				 //title="Save"
	Button SaveAsBut, win=CursorPanel, disable=0				 //title="Save As"
	Button closeBut, win=CursorPanel, disable=0					//title="Close"
	Button ExportFitBut, win=CursorPanel, disable=0				//title="Export Fit Project"
	Button resetBut, win=CursorPanel, disable=0  				//title="Save as Template "
	Button UpDateBut, win=CursorPanel, disable=0				//title="Show Initial Values"
	Button ConstraintBut, win=CursorPanel, disable=0			//title="Peak Editor"
	Button UpDateButFin, win=CursorPanel, disable=0			//title="Show Final Values"
	Button FitBut, win=CursorPanel, disable=0					//title="Start Fit"
	Button ReUseBut, win=CursorPanel, disable=0				//title="Result ---> Initial"
	Button restorBut, win=CursorPanel, disable=0				//title="Recover last Saved"
	CheckBox RecordCheck,win=CursorPanel, disable = 0 
	CheckBox RobustCheck,win=CursorPanel, disable = 0 
	CheckBox SuppressCheck,win=CursorPanel, disable = 0 
	CheckBox check0, win=CursorPanel, disable = 0				//,title="Area"
	CheckBox check1, win=CursorPanel, disable = 0
	CheckBox check2, win=CursorPanel, disable = 0
	CheckBox check3, win=CursorPanel, disable =0
	CheckBox check4, win=CursorPanel, disable =0
	CheckBox check5, win=CursorPanel, disable = 0
	CheckBox check6, win=CursorPanel, disable = 0
	CheckBox InitCheck, win=CursorPanel, disable =0
	
	
	
	
	
	elseif (set == 1)
	Button LoadNewBtn,  win=CursorPanel, disable= 0			 //title="  New Fit Project  >>"
	Button ImportFitBut,  win=CursorPanel, disable= 0			//title="Import Fit Project    >>"
	Button OpenProjectBtn,  win=CursorPanel, disable= 0	
	Button closeBut, win=CursorPanel, disable=0					//title="Close"
	endif
	
end


function DeActivateButtons(set)
	variable set   // 0, 1, 2. ... do define which buttons are to be switched off
	
	if ( set == 0)   //all of them
	Button LoadNewBtn,  win=CursorPanel, disable= 2			 //title="  New Fit Project  >>"
	Button ImportFitBut,  win=CursorPanel, disable=2			//title="Import Fit Project    >>"
	Button OpenProjectBtn,  win=CursorPanel, disable=2		//title="Open Fit Project"
	Button ReportViewerBtn, win=CursorPanel, disable= 2		//title="View Numeric Results"
	Button FinePlotBtn, win=CursorPanel, disable=2 	 			//title="Permanent Plot (saves last)"
	Button addBut, win=CursorPanel, disable=2 					//title=" Add Peak   >>"
	Button DeleteBut, win=CursorPanel, disable=2				//title=" Delete Peak  <<"
	Button CleanBut, win=CursorPanel, disable=2				 //title="Save"
	Button SaveAsBut, win=CursorPanel, disable=2				 //title="Save As"
	Button closeBut, win=CursorPanel, disable=2					//title="Close"
	Button ExportFitBut, win=CursorPanel, disable=2				//title="Export Fit Project"
	Button resetBut, win=CursorPanel, disable=2  				//title="Save as Template "
	Button UpDateBut, win=CursorPanel, disable=2				//title="Show Initial Values"
	Button ConstraintBut, win=CursorPanel, disable=2			//title="Peak Editor"
	Button UpDateButFin, win=CursorPanel, disable=2			//title="Show Final Values"
	Button FitBut, win=CursorPanel, disable=2					//title="Start Fit"
	Button ReUseBut, win=CursorPanel, disable=2				//title="Result ---> Initial"
	Button restorBut, win=CursorPanel, disable=2				//title="Recover last Saved"
					
	CheckBox RecordCheck,win=CursorPanel, disable = 2 
	CheckBox RobustCheck,win=CursorPanel, disable = 2 
	CheckBox SuppressCheck,win=CursorPanel, disable = 2 
	CheckBox check0, win=CursorPanel, disable = 2				//,title="Area"
	CheckBox check1, win=CursorPanel, disable = 2 
	CheckBox check2, win=CursorPanel, disable = 2 
	CheckBox check3, win=CursorPanel, disable = 2 
	CheckBox check4, win=CursorPanel, disable = 2 
	CheckBox check5, win=CursorPanel, disable = 2 
	CheckBox check6, win=CursorPanel, disable = 2 
	CheckBox InitCheck, win=CursorPanel, disable = 2 
	
	elseif (set == 1)
	Button ReportViewerBtn, win=CursorPanel, disable= 2		//title="View Numeric Results"
	Button FinePlotBtn, win=CursorPanel, disable=2	 			//title="Permanent Plot (saves last)"
	Button addBut, win=CursorPanel, disable=2 					//title=" Add Peak   >>"
	Button DeleteBut, win=CursorPanel, disable=2				//title=" Delete Peak  <<"
	Button CleanBut, win=CursorPanel, disable=2				 //title="Save"
	Button SaveAsBut, win=CursorPanel, disable=2				 //title="Save As"
	Button closeBut, win=CursorPanel, disable=0					//title="Close"
	Button ExportFitBut, win=CursorPanel, disable=2			//title="Export Fit Project"
	Button resetBut, win=CursorPanel, disable=2 				//title="Save as Template "
	Button UpDateBut, win=CursorPanel, disable=2				//title="Show Initial Values"
	Button ConstraintBut, win=CursorPanel, disable=2			//title="Peak Editor"
	Button UpDateButFin, win=CursorPanel, disable=2			//title="Show Final Values"
	Button FitBut, win=CursorPanel, disable=2					//title="Start Fit"
	Button ReUseBut, win=CursorPanel, disable=2				//title="Result ---> Initial"
	Button restorBut, win=CursorPanel, disable=2			
		
	CheckBox RecordCheck,win=CursorPanel, disable = 2 
	CheckBox RobustCheck,win=CursorPanel, disable = 2 
	CheckBox SuppressCheck,win=CursorPanel, disable = 2 
	CheckBox check0, win=CursorPanel, disable = 2				//,title="Area"
	CheckBox check1, win=CursorPanel, disable = 2 
	CheckBox check2, win=CursorPanel, disable = 2 
	CheckBox check3, win=CursorPanel, disable = 2 
	CheckBox check4, win=CursorPanel, disable = 2 
	CheckBox check5, win=CursorPanel, disable = 2 
	CheckBox check6, win=CursorPanel, disable = 2 
	CheckBox InitCheck, win=CursorPanel, disable = 2 
	
	endif
end



function launchCurveFit(ctrlName): ButtonControl
	string ctrlName
	CheckLocation()
	updateCoefs()
	
	
	//////////////////////////////////////////////////////////////////
	//	Make references to global objects
	NVAR toleranceLocal = V_FitTol
	NVAR toleranceGlobal = root:STFitAssVar:V_FitTol
	toleranceLocal = toleranceGlobal
	NVAR savedLast = root:STFitAssVar:savedLast
	savedLast = 0 //this change has not beed saved yet

	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR coefWave = root:STFitAssVar:PR_CoefWave
	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR RawXWave = root:STFitAssVar:PR_XRawDataCursorPanel
	SVAR RawYWave = root:STFitAssVar:PR_nameWorkWave
	NVAR FitMin = root:STFitAssVar:STFitMin
	NVAR epsilonFactor = root:STFitAssVar:EpsilonFactor
	variable FitMinIndex
	NVAR FitMax = root:STFitAssVar:STFitMax
	variable FitMaxIndex
	Wave RawY = $RawYWave
	
	Wave  holdRef = hold
	Wave /t T_Constraints = T_Constraints
	Wave /t Min_Limit = Min_Limit
	Wave /t Max_Limit = Max_Limit
	if ( numpnts(hold) == 0 )
		DoAlert 0, "You have to add peaks before you can do a fit."
		return -1
	endif
	
	//////////////////////////////////////////////////////////
	//	Make auxiliary global objects
	duplicate /o T_Constraints $"EffectiveConstraints"
	Wave /t ConstRef = $"EffectiveConstraints"
	Variable /G V_FitError = 0
	Variable /G V_chisq
	
	////////////////////////////////
	//	Local objects
	variable timerRefNum
	variable microSeconds
	variable index,constraintIndex,k, i
	variable nPeaks
	variable numPntsOutput = 750
	variable normFac = area(RawY)
	variable numCoef = 6 // Voigt as default
	
	string HoldString = ""
	string DestWave = "fit_" + RawYWave                 //keeps the name of the fit-result wave like "fit_Au4f_sample", wave is automatically generated by FitFunc command
	string functionType
	string CheckFitString 			//this string is proceeded via the command "execute" to set up the fit, but don't do anything numerical -> check for 'syntax' errors
	string FitString				//this string is proceeded via the command "execute" to perform the fitting
	//the following variables are needed to implement the temporarily normalization of the intensity constraints
	string FirstPartMin				//hold the text part of a constraints statement
	string FirstPartMax
	string LastPartMin				//hold the numerical part of a constraints statement  
	string LastPartMax
	string NormalizedLowLimit		//hold the constraint statement after normalization
	string NormalizedHighLimit
	variable NumLowLimit
	variable NumHighLimit
	variable operatorPos			//hold the position of > or < in a string
	
	variable BackgroundType = 0     //0: no background, 1: Line-Shirley, 2 ....
	
	///////////////////////////////////////////////////////////////////////////////////
	//	CHECK POINT: Is the wave name too long?
	if ( strlen(RawYWave) >= 26)		  //26 charackters would be okay, but there are names generated form this string, and those get possibly too long
		doalert 0, "The name of the fit-wave is too long! Please shorten the names."
		return 1
	endif
	
	print "\r---------------------------------------------------------------          XPST macro package\rStart curve fit.\r"
	
	/////////////////////////////////////////////////////////////////////
	//	Update the graphical user interface
	Button CleanBut disable =0
	Button ReUseBut disable = 0
	//CheckBox CleanCheck disable = 0
	SetActiveSubwindow CursorPanel#GuiCursorDisplay
	upDateFitDisplay("void")    //refresh the display, since this funciton is usually a button control, give an artificial Buttonname i.e. "void"
	
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	Overwrite the old W_coef, so the entire thing starts from the initial conditions
	wave InitializeCoef = InitializeCoef
	killwaves /z W_coef
	duplicate /o InitializeCoef W_coef    //InitializeCoef contains the coefficients which were set, by pressing the add button
	
	
	///////////////////////////////////////////////////////////////////////////////////////////////////
	//	Get the number of peaks and define the function type
	strswitch(peakType)
	case "SingletSK":
		numCoef = 6
		nPeaks = (numpnts(W_coef )- 5)/numCoef
		functionType = "FitMultiVoigtSK"
		BackgroundType = 2
		break
	case "MultiSK":
		numCoef = 15
		nPeaks = (numpnts(W_coef )- 5)/numCoef
		functionType = "FitMultipletVoigtSK"
		BackgroundType = 2
		break
	default:
		DoAlert 0, "Error in the function launchCurveFit() in the strswitch(peakType) construction."
		return 1
	endswitch
	
	//////////////////////////////////////////////////////////////////////////////////////
	//	CHECK POINT: No peaks given? if yes, exit
	if (nPeaks == 0)
		FitAbort()
		DoAlert 0, "There are no peaks .... "        
		return 1
	endif
	
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	CHECK POINT: Faulty logical constraints with + and - in the area coefficient ... this would possibly conflict with the normalization later on
	//for ( i = 0; i < npeaks; i += 1)
		//figure out, whether the constraints to the area coefficient are numerical or logical, if it is logical then continue with the next area coefficient
	//	if ( ( countStringItems(Min_Limit[numCoef * i],"k") > 1 && countStringItems(Min_Limit[numCoef * i],"+") != 0 ) || ( countStringItems(Min_Limit[numCoef * i],"k") > 1 && countStringItems(Max_Limit[numCoef * i],"+") != 0 ) )         //countStringItems( ) is implemented in MiscFUNC.ipf
	//		DoAlert 0, "You constrained one or more of the area coefficients by using + or -.\rThis is not valid, use multiplication or division instead!"
	//		FitAbort()
	//		return 1
	//	endif
	//	if ( ( countStringItems(Min_Limit[numCoef * i],"k") > 1 && countStringItems(Min_Limit[numCoef * i],"-") != 0 ) || ( countStringItems(Min_Limit[numCoef * i],"k") > 1 && countStringItems(Max_Limit[numCoef * i],"-") != 0 ) )         //countStringItems( ) is implemented in MiscFUNC.ipf
	//		DoAlert 0, "You constrained one or more of the area coefficients by using + or -.\rThis is not valid, use multiplication or division instead!"
	//		FitAbort()
	//		return 1
	//	endif
	//endfor
	
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	CHECK POINT: Is the lower and upper limit of the area compatible with the normalization
	//		the number of k's tells, if the constraint is a logical one, like k6 > 0.5*k0 or a numerical one k6> 10
	//		if it is a numerical one, then split it into the part "k6 >" and the number
	//		normalize the number and rejoin it to "k6 > normalized number "
	//
	//     first check if there are any brackets
	for ( i = 0; i < npeaks; i += 1)
		if (MyGrepString(Min_Limit[numCoef*i],"[;(;);{;};]") == 1 || MyGrepString(Max_Limit[numCoef*i],"[;(;);{;};]" ) == 1 )
			DoAlert 0, "Do not use ( ) ord [ ] in the constraints wave"
			FitAbort()
			return 1
		endif
	endfor
	//if not, go on ....
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	NUMERICAL IMPROVEMENT: Normalize the raw data and the intensity intial values for the fit
	//    do the normalization also for the constraints
	
	for ( i = 0; i < nPeaks; i += 1)
		if (BackgroundType == 0)
			W_coef[numCoef*i] /= normFac
			
			Min_Limit[numCoef*i] = NormAndDeNormConstraints(Min_Limit[numCoef*i],normFac,0)
			Max_Limit[numCoef*i] = NormAndDeNormConstraints(Max_Limit[numCoef*i],normFac,0)
			
		elseif (BackgroundType == 1)
			if (i==0)   //the first peak also has the background attached  // the following lines could be condensed in a for - loop
					// but they are kept like that for clarity
				W_coef[numCoef*i] /= normFac
				W_coef[numCoef*i+1] /= normFac
				W_coef[numCoef*i+2] /= normFac
				W_coef[numCoef*i+3] /= normFac
				Min_Limit[numCoef*i] = NormAndDeNormConstraints(Min_Limit[numCoef*i],normFac,0)
				Max_Limit[numCoef*i] = NormAndDeNormConstraints(Max_Limit[numCoef*i],normFac,0)
				Min_Limit[numCoef*i+1] = NormAndDeNormConstraints(Min_Limit[numCoef*i+1],normFac,0)
				Max_Limit[numCoef*i+1] = NormAndDeNormConstraints(Max_Limit[numCoef*i+1],normFac,0)
				Min_Limit[numCoef*i+2] = NormAndDeNormConstraints(Min_Limit[numCoef*i+2],normFac,0)
				Max_Limit[numCoef*i+2] = NormAndDeNormConstraints(Max_Limit[numCoef*i+2],normFac,0)
				Min_Limit[numCoef*i+3] = NormAndDeNormConstraints(Min_Limit[numCoef*i+3],normFac,0) //this is the first peak amplitude
				Max_Limit[numCoef*i+3] = NormAndDeNormConstraints(Max_Limit[numCoef*i+3],normFac,0) 
			else 
				W_coef[numCoef*i+3] /= normFac
				Min_Limit[numCoef*i+3] = NormAndDeNormConstraints(Min_Limit[numCoef*i+3],normFac,0)
				Max_Limit[numCoef*i+3] = NormAndDeNormConstraints(Max_Limit[numCoef*i+3],normFac,0)
			endif
		elseif (BackgroundType == 2)
			if (i==0)   //the first peak also has the background attached  // the following lines could be condensed in a for - loop
					// but they are kept like that for clarity
				W_coef[numCoef*i] /= normFac //offset
				W_coef[numCoef*i+1] /= normFac //slope
				W_coef[numCoef*i+2] /= normFac //parabola
				W_coef[numCoef*i+3] /= normFac //pseudo-tougaard
				W_coef[numCoef*i+4] /= normFac //shirley
				W_coef[numCoef*i+5] /= normFac //amplitude
				Min_Limit[numCoef*i] = NormAndDeNormConstraints(Min_Limit[numCoef*i],normFac,0)
				Max_Limit[numCoef*i] = NormAndDeNormConstraints(Max_Limit[numCoef*i],normFac,0)
				Min_Limit[numCoef*i+1] = NormAndDeNormConstraints(Min_Limit[numCoef*i+1],normFac,0)
				Max_Limit[numCoef*i+1] = NormAndDeNormConstraints(Max_Limit[numCoef*i+1],normFac,0)
				Min_Limit[numCoef*i+2] = NormAndDeNormConstraints(Min_Limit[numCoef*i+2],normFac,0)
				Max_Limit[numCoef*i+2] = NormAndDeNormConstraints(Max_Limit[numCoef*i+2],normFac,0)
				Min_Limit[numCoef*i+3] = NormAndDeNormConstraints(Min_Limit[numCoef*i+3],normFac,0)
				Max_Limit[numCoef*i+3] = NormAndDeNormConstraints(Max_Limit[numCoef*i+3],normFac,0)
				Min_Limit[numCoef*i+4] = NormAndDeNormConstraints(Min_Limit[numCoef*i+4],normFac,0)
				Max_Limit[numCoef*i+4] = NormAndDeNormConstraints(Max_Limit[numCoef*i+4],normFac,0) 
				Min_Limit[numCoef*i+5] = NormAndDeNormConstraints(Min_Limit[numCoef*i+5],normFac,0)
				Max_Limit[numCoef*i+5] = NormAndDeNormConstraints(Max_Limit[numCoef*i+5],normFac,0) 
			else 
				W_coef[numCoef*i+5] /= normFac
				Min_Limit[numCoef*i+5] = NormAndDeNormConstraints(Min_Limit[numCoef*i+5],normFac,0)
				Max_Limit[numCoef*i+5] = NormAndDeNormConstraints(Max_Limit[numCoef*i+5],normFac,0)
			endif
		
		
		endif 
	endfor
	RawY /= normFac
	
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	Generate the contents of the constraints wave  T_Constraints
	for (i=0; i <  numpnts(Min_Limit); i+=1)
		T_Constraints[2*i] = Min_Limit[i]
		T_Constraints[2*i+1] = Max_Limit[i]
	endfor

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	Ceate a HoldString and create an effective constraints wave (parameters that are set to be constant are omitted in that wave)
	constraintIndex = -2
	k = 0
	for (index = 0; index < numpnts(holdRef); index += 1)
		HoldString +=num2istr(holdRef[index])
		constraintIndex += 2       								 //index der effectiveConstraints
		k = 2*index                           							 //Index von T_constraints
		if ( holdRef[index] == 0 )
			ConstRef[constraintIndex] = T_Constraints[k]
			ConstRef[constraintIndex+1] = T_Constraints[k+1]
		elseif (holdRef[index] != 1 && holdRef[index] != 0)
			DoAlert 0, "Something is wrong with the 'hold' wave! Only 0 and 1 are valid entries."
			FitAbort()
			return 1
		else
			constraintIndex -= 2
		endif
	endfor
	redimension /n=(constraintIndex+2) ConstRef				//ConstRef is a link to the auxiliary wave "EffectiveConstraints" which is used during fitting
	
	
	
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	CHECK POINT: Was a reasonable background subtracted?
	WaveStats /Q RawY //needed later on, so do not uncomment
	//if ( RawY[0] > noiseThreshold*V_max && RawY[numpnts(RawY)-1] > noiseThreshold*V_max)
	//	DoAlert 0, "The spectrum seems to have a significant offset. This will very likely cause an error! Subtract a better background."
	//endif
	

	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	CHECK POINT: Make an epsilon wave in case there is none
	if (! exists("epsilon") )
		duplicate /o W_coef epsilon
		wave epsilon = epsilon
		epsilon = 1e-9
	endif
	
	epsilon *=epsilonFactor
	
	
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	CHECK POINT: Set up the fit, but do nothing numerical, done by flag "/O". This will catch non-numerical errors
	strswitch(RawXWave)
		case "":
			sprintf CheckFitString, "FuncFit /O  /H=\"%s\"  /Q  %s W_coef  '%s' /D /C=EffectiveConstraints  /E=epsilon", HoldString, functionType, RawYWave
			break
		case "_calculated_":
			sprintf CheckFitString, "FuncFit /O  /H=\"%s\"  /Q  %s W_coef  '%s' /D /C=EffectiveConstraints  /E=epsilon", HoldString, functionType, RawYWave
			break
		default:
			sprintf CheckFitString, "FuncFit /O  /H=\"%s\" /Q %s W_coef  '%s' /X='%s' /D /C=EffectiveConstraints  /E=epsilon", HoldString, functionType, RawYWave, RawXWave
			break
	endswitch
	Execute CheckFitString
	if (V_FitError ==1)
		DoAlert 0, "Non-numerical error: There is something wrong in the fit setup.\rCheck constraints and hold wave!\rA paramter that is set to be constant in the hold wave MUST NOT appear anywhere else in the constraints wave!"
		
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//////        THERE IS A PROBLEM   >    DENORMALIZE   EVERYTHING   BEFORE   CONTINUING
		//////
		
		RawY *= normFac             	 //the raw wave is de -normalized
		
		for ( i = 0; i < nPeaks; i += 1)
			if (BackgroundType == 0)
				W_coef[numCoef*i] *= normFac
				Min_Limit[numCoef*i] = NormAndDeNormConstraints(Min_Limit[numCoef*i],normFac,1)
				Max_Limit[numCoef*i] = NormAndDeNormConstraints(Max_Limit[numCoef*i],normFac,1)
			elseif (BackgroundType == 1)
				if (i==0) //first peak + background
					W_coef[numCoef*i] *= normFac
					W_coef[numCoef*i+1] *= normFac
					W_coef[numCoef*i+2] *= normFac
					W_coef[numCoef*i+3] *= normFac
				
					Min_Limit[numCoef*i] = NormAndDeNormConstraints(Min_Limit[numCoef*i],normFac,1)
					Max_Limit[numCoef*i] = NormAndDeNormConstraints(Max_Limit[numCoef*i],normFac,1)
					Min_Limit[numCoef*i+1] = NormAndDeNormConstraints(Min_Limit[numCoef*i+1],normFac,1)
					Max_Limit[numCoef*i+1] = NormAndDeNormConstraints(Max_Limit[numCoef*i+1],normFac,1)
					Min_Limit[numCoef*i+2] = NormAndDeNormConstraints(Min_Limit[numCoef*i+2],normFac,1)
					Max_Limit[numCoef*i+2] = NormAndDeNormConstraints(Max_Limit[numCoef*i+2],normFac,1)
					Min_Limit[numCoef*i+3] = NormAndDeNormConstraints(Min_Limit[numCoef*i+3],normFac,1) 
					Max_Limit[numCoef*i+3] = NormAndDeNormConstraints(Max_Limit[numCoef*i+3],normFac,1) 

				else
					W_coef[numCoef*i+3] *= normFac
					Min_Limit[numCoef*i+3] = NormAndDeNormConstraints(Min_Limit[numCoef*i+3],normFac,1)
					Max_Limit[numCoef*i+3] = NormAndDeNormConstraints(Max_Limit[numCoef*i+3],normFac,1)
				endif
				elseif (BackgroundType == 2)
				if (i==0) //first peak + background
					W_coef[numCoef*i] *= normFac
					W_coef[numCoef*i+1] *= normFac
					W_coef[numCoef*i+2] *= normFac
					W_coef[numCoef*i+3] *= normFac
					W_coef[numCoef*i+4] *= normFac
					W_coef[numCoef*i+5] *= normFac
					
					Min_Limit[numCoef*i] = NormAndDeNormConstraints(Min_Limit[numCoef*i],normFac,1)
					Max_Limit[numCoef*i] = NormAndDeNormConstraints(Max_Limit[numCoef*i],normFac,1)
					Min_Limit[numCoef*i+1] = NormAndDeNormConstraints(Min_Limit[numCoef*i+1],normFac,1)
					Max_Limit[numCoef*i+1] = NormAndDeNormConstraints(Max_Limit[numCoef*i+1],normFac,1)
					Min_Limit[numCoef*i+2] = NormAndDeNormConstraints(Min_Limit[numCoef*i+2],normFac,1)
					Max_Limit[numCoef*i+2] = NormAndDeNormConstraints(Max_Limit[numCoef*i+2],normFac,1)
					Min_Limit[numCoef*i+3] = NormAndDeNormConstraints(Min_Limit[numCoef*i+3],normFac,1) 
					Max_Limit[numCoef*i+3] = NormAndDeNormConstraints(Max_Limit[numCoef*i+3],normFac,1) 
					Min_Limit[numCoef*i+4] = NormAndDeNormConstraints(Min_Limit[numCoef*i+4],normFac,1) //this is the first peak amplitude
					Max_Limit[numCoef*i+4] = NormAndDeNormConstraints(Max_Limit[numCoef*i+4],normFac,1) 
					Min_Limit[numCoef*i+5] = NormAndDeNormConstraints(Min_Limit[numCoef*i+5],normFac,1) //this is the first peak amplitude
					Max_Limit[numCoef*i+5] = NormAndDeNormConstraints(Max_Limit[numCoef*i+5],normFac,1) 

				else
					W_coef[numCoef*i+5] *= normFac
					Min_Limit[numCoef*i+5] = NormAndDeNormConstraints(Min_Limit[numCoef*i+5],normFac,1)
					Max_Limit[numCoef*i+5] = NormAndDeNormConstraints(Max_Limit[numCoef*i+5],normFac,1)
				endif
			endif 	
		endfor
		
		//	Generate the de-normalized contents of the constraints wave  T_Constraints
		for (i=0; i < numpnts(Min_Limit); i+=1)
			T_Constraints[2*i] = Min_Limit[i]
			T_Constraints[2*i+1] = Max_Limit[i]
		endfor
		
		FitAbort()
		return 1   					//leave, if something was wrong
	endif

	///////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////			NOW   DO   THE   REAL   FIT	
	////////
	 
	strswitch(RawXWave)
		case "":
			sprintf FitString, "FuncFit /L=%d /H=\"%s\"  %s W_coef  '%s'(%d,%d) /D /C=EffectiveConstraints  /E=epsilon",numPntsOutput, HoldString, functionType, RawYWave, FitMin, FitMax
			break
		case "_calculated_":
			sprintf FitString, "FuncFit /L=%d /H=\"%s\"  %s W_coef  '%s'(%d,%d)  /D /C=EffectiveConstraints  /E=epsilon", numPntsOutput,HoldString, functionType, RawYWave, FitMin, FitMax
			break
		default:
			//find the indizes of the min and max
			wave rawX = $RawXWave
			
			FitMinIndex=  max ( 0, ceil( (     (  FitMin-WaveMin(rawX)  ) / ( WaveMax(rawX) -WaveMin(rawX)  )  )      *numpnts(rawX) )  )
			FitMaxIndex= min (numpnts(rawX), numpnts(rawX) - floor( (    (  WaveMax(rawX) -FitMax )  / ( WaveMax(rawX) -WaveMin(rawX)  ) )      *numpnts(rawX) )  )
			print FitMinIndex, FitMaxIndex
			sprintf FitString, "FuncFit /L=%d /H=\"%s\"  %s W_coef  '%s'[%d,%d] /X='%s'[%d,%d]  /D /C=EffectiveConstraints  /E=epsilon",numPntsOutput, HoldString, functionType, RawYWave,  FitMinIndex, FitMaxIndex, RawXWave, FitMinIndex, FitMaxIndex
			break
	endswitch
	
	//print "\rTechnical remarks:"
	//print "---------------------------------------------------"
//	print "For a satisfying fit try to set good initial conditions. In the best case, your initial conditions closely resemble the result."
//	print "Make the initial values match the constraints."
//	print "For asymmetric peaks: Try not to vary the 'Asymmetry-Shift' parameter; leave the default value of '1' in the corresponding location in the hold wave"
//	print "(Button: 'Peak Editor'). Let it only vary if necessary."
//	print "\r"
//	print "Please note that the spectrum is internally normalized during the fit. This is done to enhance the numerical stability."
//	print "Therefore, the area coefficients reported here on the commandline are smaller than the final coefficients."
//	print "The final set of area coefficients refers to the original spectrum, while the coefficients reported here on the command line refer to the normalized data."
//	print "Usually, you can ignore those 'normalized' coefficients!"
//	print "\r"
//	print "Please note furthermore that constraining the intensities of ASYMMETRIC peaks by statements like 'k6 > 0.5 * k0' and 'k6 < 0.5*k0'  works only"
//	print "if the peaks have an identical FWHM, asymmetry, and GL-ratio. Otherwise, the resulting area will not follow the constraints you set for the area coefficient."
//	print "This is due to the fact that for asymmetric peaks the area coefficient does not directly represent the area any more, but only a value proportional to it."
//	print "The actual area is measured numerically when you press 'Save + Make Report'"
//	print "\r"
//	print "Some of the default constraint statements are written as 'k8 > k2 - 1e-5' and 'k8 < k2 + 1e-5'. Providing a small intervall of e.g. 2e-5 around k2"
//	print "has shown to be numerically favorable (faster convergence). However, a more strict statement like and 'k8 > k2' and 'k8 < k2' should also work." 
//	print "\r"
//	print "In the case of a 'singular matrix' error, Igor will report on one or more critical coefficients. To avoid this problem increase the 'iteration step' values for"
//	print "those coefficients. The wave 'iteration step' is identical to the epsilon wave as described in the Igor manual. Alternatively, set those critical coefficients"
//	print "to constant values via the hold wave; singular matrix indicates that one or more coefficients show no influence at all to the fit quality. Thus, they can adapt arbitrary values"
//	print "which makes the situation unsolvable for Igor. However, sometimes better initial conditions and constraints help as well ..."
//	print "\r"
//	print "For further information please consult the Igor Manual, in particular the chapter on Curve Fitting."
//	print "The 'Fit Assistant' provides only a front-end for the Igor fitting engine - so if you want to learn more e.g. on constraints or hold waves you have to read"
//	print "through the corresponding chapter in the manual."
//	print "\r"
//	print "---------------------------------------------------"
	
	Execute FitString
	

	
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	////////////						DENORMALIZE    AFTER    EVERYTHING    IS   DONE
	///////////
	
	RawY *= normFac							//the raw wave is de -normalized
	
	Wave OutputWave = $DestWave				//the fit_wave was generated by Igor, simply make a reference to it
	OutputWave *= normFac   						//de-normalize the fit_wave
	
	for ( i = 0; i < nPeaks; i += 1)
		if (BackgroundType == 0)
			W_coef[numCoef*i] *= normFac
			Min_Limit[numCoef*i] = NormAndDeNormConstraints(Min_Limit[numCoef*i],normFac,1)
			Max_Limit[numCoef*i] = NormAndDeNormConstraints(Max_Limit[numCoef*i],normFac,1)
		elseif (BackgroundType == 1)
			if (i==0)   //specific for line-shirley background
				W_coef[numCoef*i] *= normFac
				W_coef[numCoef*i+1] *= normFac
				W_coef[numCoef*i+2] *= normFac
				W_coef[numCoef*i+3] *= normFac			
				Min_Limit[numCoef*i] = NormAndDeNormConstraints(Min_Limit[numCoef*i],normFac,1)
				Max_Limit[numCoef*i] = NormAndDeNormConstraints(Max_Limit[numCoef*i],normFac,1)
				Min_Limit[numCoef*i+1] = NormAndDeNormConstraints(Min_Limit[numCoef*i+1],normFac,1)
				Max_Limit[numCoef*i+1] = NormAndDeNormConstraints(Max_Limit[numCoef*i+1],normFac,1)
				Min_Limit[numCoef*i+2] = NormAndDeNormConstraints(Min_Limit[numCoef*i+2],normFac,1)
				Max_Limit[numCoef*i+2] = NormAndDeNormConstraints(Max_Limit[numCoef*i+2],normFac,1)

				Min_Limit[numCoef*i+3] = NormAndDeNormConstraints(Min_Limit[numCoef*i+3],normFac,1) //this is the first peak amplitude
				Max_Limit[numCoef*i+3] = NormAndDeNormConstraints(Max_Limit[numCoef*i+3],normFac,1) 
			else
				W_coef[numCoef*i+3] *= normFac
				Min_Limit[numCoef*i+3] = NormAndDeNormConstraints(Min_Limit[numCoef*i+3],normFac,1)
				Max_Limit[numCoef*i+3] = NormAndDeNormConstraints(Max_Limit[numCoef*i+3],normFac,1)
			endif
		elseif (BackgroundType == 2)
			if (i==0)   //specific for line-shirley background
				W_coef[numCoef*i] *= normFac
				W_coef[numCoef*i+1] *= normFac
				W_coef[numCoef*i+2] *= normFac
				W_coef[numCoef*i+3] *= normFac
				W_coef[numCoef*i+4] *= normFac
				W_coef[numCoef*i+5] *= normFac
				
				Min_Limit[numCoef*i] = NormAndDeNormConstraints(Min_Limit[numCoef*i],normFac,1)
				Max_Limit[numCoef*i] = NormAndDeNormConstraints(Max_Limit[numCoef*i],normFac,1)
				Min_Limit[numCoef*i+1] = NormAndDeNormConstraints(Min_Limit[numCoef*i+1],normFac,1)
				Max_Limit[numCoef*i+1] = NormAndDeNormConstraints(Max_Limit[numCoef*i+1],normFac,1)
				Min_Limit[numCoef*i+2] = NormAndDeNormConstraints(Min_Limit[numCoef*i+2],normFac,1)
				Max_Limit[numCoef*i+2] = NormAndDeNormConstraints(Max_Limit[numCoef*i+2],normFac,1)
				Min_Limit[numCoef*i+3] = NormAndDeNormConstraints(Min_Limit[numCoef*i+3],normFac,1) 
				Max_Limit[numCoef*i+3] = NormAndDeNormConstraints(Max_Limit[numCoef*i+3],normFac,1)
				Min_Limit[numCoef*i+4] = NormAndDeNormConstraints(Min_Limit[numCoef*i+4],normFac,1) //this is the first peak amplitude
				Max_Limit[numCoef*i+4] = NormAndDeNormConstraints(Max_Limit[numCoef*i+4],normFac,1) 
				Min_Limit[numCoef*i+5] = NormAndDeNormConstraints(Min_Limit[numCoef*i+5],normFac,1) //this is the first peak amplitude
				Max_Limit[numCoef*i+5] = NormAndDeNormConstraints(Max_Limit[numCoef*i+5],normFac,1) 
			else
				W_coef[numCoef*i+5] *= normFac
				Min_Limit[numCoef*i+5] = NormAndDeNormConstraints(Min_Limit[numCoef*i+5],normFac,1)
				Max_Limit[numCoef*i+5] = NormAndDeNormConstraints(Max_Limit[numCoef*i+5],normFac,1)
				
			endif
		endif 	
	endfor
		
	
	
	//	Generate the de-normalized contents of the constraints wave  T_Constraints
	for (i=0; i < numpnts(Min_Limit); i+=1)
		T_Constraints[2*i] = Min_Limit[i]
		T_Constraints[2*i+1] = Max_Limit[i]
	endfor
	
	
	////////////////////////////////////////////////////////
	//	Notify the user about errors
	if (V_FitError != 0)
		if (V_FitError ==1)
			printf "Fit failed ..... Reason: Singular Matrix\r"
			printf "If the fit reported on missing dependencies on some parameters: Open the peak editor (button: 'Edit') and increase the corresponding 'Iteration Step' values."
		elseif (V_FitError == 2)
			printf "Fit failed ..... Reason: Out of Memory\r"
		elseif (V_FitError ==3)
			print "\rFit failed .... The fit function returned NaN of Inf.CHECK constraints and hold waves, or retry in robust mode.\r"
			//DoAlert 0, "The fit function returned NaN or Inf. CHECK constraints and hold waves, or retry in robust mode."
		endif
	else
		print "\rDone ...  "
	endif
	
	if (V_chisq/V_npnts > 0.001 || V_FitError == 3)
		DoAlert 0, "There was something not quite right with the fit."
	//	print "\rUnusual high Chi-square value or crash of the fit\r--------------------------------------------------------------"
	//	print "\rThis can have one of the following numerical reasons:\r"
	//	print "         - noisy data\r"
	//	print "         - bad background subtraction\r"
	//	print "         - to few or far too many peaks -> no unique solution possible\r"
	//	print "\rConflicts due to bad constraints:\r"
	//	print "         - you mixed up the values of upper and lower limits, at least for one coefficient. E.g. K1 < 5  and K1 > 10\r"
	//	print "         - typo somewhere in the waves for upper and lower limit\r"
	//	print "         - you forward-referenced a peak:\t K0 = 0.5 * K6     THIS IS UNSTABLE\r"
	//	print "            INSTEAD:   K6 = 2 * K0\r"
	//	print "         - initial values outside the constraints regions\r"
	//	print "\rSee the Technical remarks above (scroll upwards in this window) for more information."
	
	endif
	
	/////////////////////////////
	//	Delete remains
	killwaves /z EffectiveConstraints, bgyw
	killvariables  V_FitError, V_chisq
	// now, do the aftermath
	//Display the fit results
	SinglePeakDisplay(peakType, RawYWave, RawXWave, coefWave)
	//copy the fit results into the setup wave
	waves2setup()
	// now extract the peaks from the FitTemp folder
	if (V_chisq/V_npnts < 0.001 && V_FitError == 0)
		PullOutPeaksFromFitTemp( )
		AnalyzeWCoefAfterFit()
	endif
	
	epsilon /= epsilonFactor
	print "\r---------------------------------------------------------------\r"
end


function PullOutPeaksFromFitTemp()
	//later on, this function also serves to split doublet/multiplet peaks into single peaks
	string wavesInFitTemp
	variable i
	string parentFolder = GetDataFolder(1)
//	print parentFolder

	string item = ""
	SetDataFolder :FitTemp:
	wavesInFitTemp = WaveList("*",";","DIMS:1") //only get the one-dimensional ones
	SetDataFolder $parentFolder //go back after you made the list

	//first, make a duplicate of those waves to the main folder
	for ( i = 0; i < ItemsInList(wavesInFitTemp); i += 1 )
		item = StringFromList(i,wavesInFitTemp)
		duplicate /o :FitTemp:$item $item
	endfor
end


function AnalyzeWCoefAfterFit()
//this function needs to know the peak-type later on .... for now it is only PVSK (Pseudo-Voigt Shirley Kombi)
	SVAR peakType = root:STFitAssVar:PR_PeakType
	
	strswitch(peakType)
	case "SingletSK":
		EvaluateSingletSK()
		break
	case "MultiSK":
		EvaluateMultipletSK()
		break
	default:
		DoAlert 0, "Peak type not recognized in AnalyzeWCoefAfterFit()"
		break
	endswitch

end

//////////////////////////////////////////////////////////////////////////////////////
////////
//	delete auxiliary global objects if the function has to quit
function FitAbort()
	upDateFitDisplay("void")
	killwaves /z EffectiveConstraints
	killvariables  V_FitError
end

////////////////////////////////////
///////////////////////////////////
// Refresh the fit display
function upDateFitDisplay(ctrlName):ButtonControl 
	string ctrlName
	//CheckLocation()
	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR RawYWave = root:STFitAssVar:PR_nameWorkWave
	SVAR RawXWave = root:STFitAssVar:PR_XRawDataCursorPanel
	string InitializeCoefName = "InitializeCoef"
	wave InitializeCoef = InitializeCoef
	RemoveFromGraph /W=CursorPanel#guiCursorDisplay /Z $"#1"
	updateCoefs()
	SinglePeakDisplay(peakType,RawYWave,RawXWave,InitializeCoefName)
	//duplicate /o InitializeCoef W_coef
end

function upDateFitDisplayFinalVal(ctrlName):ButtonControl 
	string ctrlName
	//CheckLocation()
	SVAR peakType = root:STFitAssVar:PR_PeakType
	SVAR RawYWave = root:STFitAssVar:PR_nameWorkWave
	SVAR RawXWave = root:STFitAssVar:PR_XRawDataCursorPanel
	string InitializeCoefName = "InitializeCoef"
	wave InitializeCoef = InitializeCoef
	variable i
	updateCoefs()
	RemoveFromGraph /W=CursorPanel#guiCursorDisplay /Z $"#1"
	
	SinglePeakDisplay(peakType,RawYWave,RawXWave,"W_Coef")
//	duplicate /o InitializeCoef W_coef
end



function PlotPeaks( coefWave,dataWave,fitWave,peakType,RawXWave,newFolder)

string coefWave
string dataWave
string fitWave
string peakType
string RawXWave
variable newFolder               //if this value is different from 1, no folder for the results will be created

strswitch(peakType)
	case "SingletSK":   //for now, we work only here
		DrawAndEvaluatePseudoVoigtSK(dataWave,fitWave,peakType,RawXWave,newFolder)
	break
	case "MultiSK":
		DrawAndEvaluateMultipletSK(dataWave,fitWave,peakType,RawXWave,newFolder)
	break	
	default:
		DoAlert 0, "Peak type not recognized, something is wrong"
	break
endswitch

end 

//Function to name graphs and other windows in an easy-to-read way
//This function is also accessable from the commandline
function TagWindow(newTag)
	string newTag
	string TempTag
	string commandString
	NVAR V_Flag = V_Flag
	
	//now search the name-string for illegal characters and if there are any, replace them by '_'

	newTag = stringClean(newTag)
	variable i=1   //index variable for the data folder	
	commandString = ""
	sprintf commandString, "DoWindow '%s'", newTag
	Execute  commandString
	commandString =""
	  //V_flag will be set to 0, if this is not the case
	

	if (V_Flag || Exists(newTag))          //is there already a window or some other object with this name?
	//if yes:
	 //Append an index number to the current foldername and check if a folder with this name
	 //is already present. If so, increase the index number by one, append it to the basic foldername
	 //and check again. Repeat this procedure until a "free" name is found
		do
			TempTag =  newTag +"_" + num2istr(i)
			i+=1
			sprintf commandString, "DoWindow '%s'", TempTag
			Execute /Q  commandString
			commandString = ""
		while(V_Flag)
		
		if ( strlen(TempTag) >= 30)	
			printf "Function 'TagWindow' error: name too long!"
			return -1
		else
			sprintf commandString, "DoWindow /C '%s'", TempTag
			Execute /Q   commandString
			commandString =""
			sprintf commandString, "DoWindow /T %s, \"%s\"", TempTag, TempTag
			Execute /Q  commandString
			//RenameDataFolder subfolder, $tempFoldername                   //now rename the peak-folder accordingly
		endif
	
	else
		sprintf commandString, "DoWindow /C '%s'", newTag
		Execute /Q  commandString
		commandString =""
		sprintf commandString, "DoWindow /T %s, \"%s\"", newTag, newTag
		Execute /Q commandString
		killstrings /z commandString
	endif
end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//check a string for illegal characters, and replace them by regular ones
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function /s stringClean(startString)
	string startString
	
	if ( strsearch(startString,".",0) != -1 )      // strsearch returns -1 if  the string contains no "." 
		startString = ReplaceString(".", startString,"_")
	endif
	if ( strsearch(startString,":",0) != -1 )      // strsearch returns -1 if  the string contains no "." 
		startString = ReplaceString(":", startString,"_")
	endif
	if ( strsearch(startString," ",0) != -1 )  
		startString = ReplaceString(" ", startString,"")
	endif
	if ( strsearch(startString,",",0) != -1 )  
		startString = ReplaceString(",", startString,"_")
	endif
	if ( strsearch(startString,";",0) != -1 )  
		startString = ReplaceString(";", startString,"_")
	endif
	if ( strsearch(startString,"-",0) != -1 )  
		startString = ReplaceString("-", startString,"_")
	endif
	if ( strsearch(startString,"#",0) != -1 )  
		startString = ReplaceString("#", startString,"_")
	endif
	//if ( strsearch(startString,"'",0) != -1 )  
//		startString = ReplaceString("'", startString,"_")
//	endif
	if ( strsearch(startString,"*",0) != -1 )  
		startString = ReplaceString("*", startString,"_")
	endif
	if ( strsearch(startString,"~",0) != -1 )  
		startString = ReplaceString("~", startString,"_")
	endif
//	if ( strsearch(startString,".",0) != -1 )  
//		startString = ReplaceString(".", startString,"_")
//	endif
	if ( strsearch(startString,"'",0) != -1 )  
		startString = ReplaceString("'", startString,"")
	endif
	if ( strsearch(startString,"+",0) != -1 )  
		startString = ReplaceString("+", startString,"_")
	endif
	if ( strsearch(startString,",",0) != -1 )  
		startString = ReplaceString(",", startString ,"_")
	endif
	if ( strsearch(startString,"/",0) != -1 )  
		startString = ReplaceString("/", startString ,"_")
	endif
	if ( strsearch(startString,"",0) != -1 )  
		startString = ReplaceString("", startString,"ae")
	endif
	if ( strsearch(startString,"",0) != -1 )  
		startString = ReplaceString("", startString,"Ae")
	endif
	if ( strsearch(startString,"",0) != -1 )  
		startString = ReplaceString("", startString,"oe")
	endif
	if ( strsearch(startString,"",0) != -1 )  
		startString = ReplaceString("", startString,"Oe")
	endif
	if ( strsearch(startString,"",0) != -1 )  
		startString = ReplaceString("", startString,"ue")
	endif
	if ( strsearch(startString,"",0) != -1 )  
		startString = ReplaceString("", startString,"Ue")
	endif
	if ( strsearch(startString,"",0) != -1 )  
		startString = ReplaceString("", startString,"ss")
	endif
	
	return startString
end



