#pragma rtGlobals=1		// Use modern global access method.
#pragma hide=1

//////////////////////////////////////////////////////////////////////////////////
//Licence
//////////////////////////////////////////////////////////////////////////////////
//
//	If you use the 'Fit Assistant' to analyze data, please read and cite the following paper:
//	Surace and Interface Analysis, (2014), 46, 505-511
//	This publication contains more details on the fit function used by XPST.
//
//	XPST is a front-end to the IgorPro fitting engine. Although every possible effort was made 
//	to prevent serious bugs and wrong results, the author can not give any warranty for XPST as 
//	large parts actually depend on the underlying IgorPro code. 
//	Author: Dr. Martin Schmid
//
//////////////////////////////////////////////////////////////////////////////////



//this function works as follows:
// The user is supposed to plot the function
// then the cursors are used to show the range, which should be extracted

menu "XPST"
	"Help", NotifyHelp()
end

function NotifyHelp()
	DisplayHelpTopic "XPST"
	//DoAlert 0, "An overview over further functions was written to the command line."
	//InfoST()

end

proc XPSTUnhide()
	SetIgorOption IndependentModuleDev=1
end

function /s STnum2str(value)
	variable value
	variable digits
	string output
	
	if (abs(value) <= 1 )
		sprintf output,  "%.4 g  " value
	elseif ( abs(value) <= 10 && abs(value) > 1)
		sprintf output,  "%6.3 f " value
	elseif (abs(value) > 10 && abs(value)<100000)
		sprintf output,  "%8.2 f " value
	elseif (abs(value) >= 100000)
		sprintf output,  "%g " value
	endif
	
	return output
end

function getConstraintCorrectionFactor(width1,gl1,as1,at1, width2,gl2, as2, at2)

	variable width1
	variable gl1
	variable as1
	variable at1
	variable width2
	variable gl2
	variable as2
	variable at2
	
	print "This function may take a while to complete. There are many integrations involved ... be patient."
	print "Be aware that there are numericall errors due to the limited integration range."
	print "The relative error of that can be up to ~ 0.5%"
	//print "This function returns the actual constraint factor required to achieve the nominal constraint factor for peaks with different asymmetry"
	//nominalfactor peak2/peak1
	
	variable AnalyticalArea1 = 0
	variable AnalyticalArea2 = 0
	variable realRatio
	variable correctedRatio = 0.09				

      //get a starting guess
      
	AnalyticalArea1 = IntegrateSingleVoigtGLS(1,500,width1,gl1,as1,at1)
	
	AnalyticalArea2 = IntegrateSingleVoigtGLS(1*correctedRatio,500,width2,gl2,as2,at2)
	 
	realRatio = AnalyticalArea2 / AnalyticalArea1
	
//	correctedRatio =   ((1 - realRatio) + 1 )
	 //iterate for all subpeaks
	do
	correctedRatio += 0.2* (abs(realRatio-1)) // ((nominalFactor - realRatio)/nominalFactor + 1 )
	AnalyticalArea1 = IntegrateSingleVoigtGLS(1,500,width1,gl1,as1,at1)
	
	AnalyticalArea2 = IntegrateSingleVoigtGLS(1*correctedRatio,500,width2,gl2,as2,at2)
	
	realRatio = AnalyticalArea2 / AnalyticalArea1
	
	//print correctedRatio, realRatio
	while (abs(realRatio-1) > 0.000001)

	return correctedRatio
	
end

function SetFreeCursors()
	string LastGraphName = WinList("*", "", "WIN:") 
	string name = stringFromList(0,LastGraphName)
	String allNormalTraces=TraceNameList("",";",1)
	string traceName = stringFromList(0,allNormalTraces)
	Cursor /F /P /W=$name A $traceName, 0.2,0.8
	Cursor /F /P /W=$name B $traceName, 0.8,0.8
	//Cursor /F /P  / w=ShirlPanel#guiDisplay A  $"SP_DispWave" ,0.2, 0.5
end

function RemoveUnitString()
	print "This function works on waveform data only and removes the unitstring from the x and y scaling"
	string Aname = CsrWave(A)
	wave Awave = $Aname
	string Bname = CsrWave(B)
	wave Bwave = $Bname
	variable firstPoint
	variable lastPoint
	// if A is on a graph,  remove the units of A
	if ( cmpstr(Aname,"") != 0)   //go into the loop, if Aname is not empty
		//use SetScale to remove the unitstring
		firstPoint = pnt2x(Awave,0)
		lastPoint = pnt2x(Awave,numpnts(Awave)-1)
		SetScale /I x firstPoint,lastPoint, "", Awave
		SetScale y 0,0, "", Awave
	endif
	// if B is on a graph, remove the units of B
	if ( cmpstr(Bname,"") != 0)     // go into the section, if Bname is not empty
		firstPoint = pnt2x(Bwave,0)
		lastPoint = pnt2x(Bwave,numpnts(Bwave)-1)
		SetScale /I x firstPoint,lastPoint, "", Bwave
		SetScale y 0,0, "", Bwave
	
	endif
end



function CursorCut()

	string targetName = CsrWave(A)+"_c"

	// All that stuff  only prevents user errors - maybe Igor has not the most efficient language
	strswitch (CsrWave(A))
		case "":       //A not on grah
			strswitch (CsrWave(B) )
				case "":  // B missing as well
					DoAlert 0, "Please set cursor A and B on the curve to mark the range wich you want to cut. Use Ctrl+I to activate cursors..."
					return 1
				default:
					DoAlert 0, "You also have to use cursor A"
					return 1 // A is not there but B
			endswitch
		break
		default:       // A is on graph
			strswitch (CsrWave(B) )
				case "":  // B is missing
					DoAlert 0, "You have also to use cursor B"
					return 1
				default:
				break // A and B are there, proceed
			endswitch
		endswitch
	string tempTargetName
	variable i=1
	do
		tempTargetName = targetName + num2istr(i)
		i+=1
	while(exists(tempTargetName))
	targetName = tempTargetName
	duplicate /O /R=(hcsr(A),hcsr(B)) $(CsrWave(A)), $targetName
	Display /K=1 $targetName
	ModifyGraph rgb($targetName) = (0,0,65500)  // Color code it accordingly
end

function CursorCutXY()
	strswitch (CsrWave(A))
		case "":       //A not on grah
			strswitch (CsrWave(B) )
				case "":  // B missing as well
					DoAlert 0, "Please set cursor A and B on the curve to mark the range wich you want to cut. Use Ctrl+I to activate cursors..."
					return 1
				default:
					DoAlert 0, "You also have to use cursor A"
					return 1 // A is not there but B
			endswitch
		break
		default:       // A is on graph
			strswitch (CsrWave(B) )
				case "":  // B is missing
					DoAlert 0, "You have also to use cursor B"
					return 1
				default:
				break // A and B are there, proceed
			endswitch
		endswitch
	string LastGraphName = WinList("*", "", "WIN:")
	//string name = XWaveRefFromTrace( LastGraphName,CsrWave(A))
	string xname = XWaveName(LastGraphName,CsrWave(A))
	wave xVals = $xname
	
	string targetName = CsrWave(A) + "_c"
	string targetX = xname+ "_c"
	
	string tempTargetNameY, tempTargetNameX
	variable i=1
	do
		tempTargetNameY = targetName + num2istr(i)
		tempTargetNameX = targetX + num2istr(i)
		i+=1
	while(exists(tempTargetNameY))
	targetName = tempTargetNameY
	targetX = tempTargetNameX
	// here, an adaption to existing waves has to be performed
	
	duplicate /o /r=(pcsr(A),pcsr(B)) $CsrWave(A) $targetName
	duplicate /o /r=(pcsr(A),pcsr(B)) xVals $targetX
	
	display /K=1 $targetName vs $targetX
	ModifyGraph rgb($targetName) = (0,0,65500)  // Color code it accordingly	
end


//Mark a wave with the cursor and then use this function e.g. like
// translate(1.3)
//this will move the function about 1.3 x-units 
function Translate(waveShift)
	variable waveShift
	string targetName
	
	string LastGraphName = WinList("*", "", "WIN:")
	//string name = XWaveRefFromTrace( LastGraphName,CsrWave(A))
	string xname = XWaveName(LastGraphName,CsrWave(A))
	string xname2 = XWaveName(LastGraphName,CsrWave(B))
	strswitch(xname)
		case "":  //proceed normally
		break
		default:   //everything, but not empty
			DoAlert 0, "This function works only with waves with an intrinsical scalling. For waves that require an X wave it is obsolete.\rSimply go to the command line and add the shift to the X wave:\r\r e.g. wave0 += 0.3"
			return 1
	endswitch
	strswitch(xname2)
		case "":  //proceed normally
		break
		default:   //everything, but not empty
			DoAlert 0, "This function works only with waves with an intrinsical scalling. For waves that require an X wave it is obsolete.\rSimply go to the command line and add the shift to the X wave:\r\r e.g. wave0 += 0.3"
			return 1
	endswitch
	
	strswitch (CsrWave(A) )
		case "":                 //obviously A is not on the graph, so B has to do the job
			strswitch (CsrWave(B))
			case "":       // uuups, there is no cursor at all there, inform the user
				DoAlert 0, "You have to place a cursor on the wave you want to shift ..."
				return 1
			endswitch
			targetName = CsrWave(B)+"_t" 
			duplicate /o $(CsrWave(B)), $targetName
			SetScale /I x, leftx($CsrWave(B)) +waveShift , rightx($CsrWave(B) ) + waveShift , $targetName
			break
		default:       // A is on the graph
			strswitch (CsrWave(B))
			case "":       //B not
				targetName = CsrWave(A)+"_t" 
				duplicate /o $(CsrWave(A)), $targetName
				SetScale /I x, leftx($CsrWave(A)) +waveShift , rightx($CsrWave(A) ) + waveShift , $targetName
				break
			default:      // A and B are both on the graph, B has to be ignored and removed
				targetName = CsrWave(A)+"_t" 
				duplicate /o $(CsrWave(A)), $targetName
				SetScale /I x, leftx($CsrWave(A)) +waveShift , rightx($CsrWave(A) ) + waveShift , $targetName
				DoAlert 0, "You placed one cursor to much on the graph, ignoring Cusor B"
				Cursor /K B
				break
			endswitch
			break
	endswitch
	// If there is already the wave displayed, remove it, if not /Z quenches the error message
	RemoveFromGraph /Z $targetName
	AppendToGraph $targetName
	ModifyGraph rgb($targetName) = (0,0,65500)  // Color code it accordingly
end


//this function returns the path to the selected wave

function WhereIs()
	strswitch (CsrWave(A) )
		case "":                 //obviously A is not on the graph, so B has to do the job
			strswitch (CsrWave(B))
			case "":       // uuups, there is no cursor at all there, inform the user
				DoAlert 0, "You have to place a cursor on the wave you want to shift ..."
				return 1
			endswitch
			break
		default:       // A is on the graph
			strswitch (CsrWave(B))
			case "":       //B not
				wave test = CsrWaveRef(A)
				print "Cursor A is on: "
				print GetWavesDataFolder(test,2)
				break
			default:      // A and B are both on the graph, B has to be ignored and removed
				wave test = CsrWaveRef(B)
				print "Cursor B is on: "
				print GetWavesDataFolder(test,2)
				print "Cursor A is on: "
				wave test2 = CsrWaveRef(A)
				print GetWavesDataFolder(test2,2)
				break
			endswitch
			break
	endswitch
end




/////////////////////////////////////////////////////////////////////////////////////
//Auxiliary string functions

function countStringItems(Specimen,item)
	string Specimen
	string item
	//Do the counting only with upper case letter
	Specimen = UpperStr(Specimen)
	item = UpperStr(item)
	variable countItem = 0
	variable position = -1
	do
		position = strsearch(Specimen,item,position+1)
		if (position >=0)
			countItem += 1
		endif
	while (position != -1)
	return countItem
end

function Integrator()
	string targetName = CsrWave(A)+"_I"
	string waveTag = CsrWave(A)
	wave original = $waveTag
	variable width = DimSize($waveTag,0)
	variable IntStart, IntEnd, j,k

	// All that stuff  only prevents user errors - maybe Igor has not the most efficient language
	strswitch (CsrWave(A))
		case "":       //A not on grah
			strswitch (CsrWave(B) )
				case "":  // B missing as well
					DoAlert 0, "Please set cursor A and B on the curve to mark the range wich you want to cut. Use Ctrl+I to activate cursors..."
					return 1
				default:
					DoAlert 0, "You also have to use cursor A"
					return 1 // A is not there but B
			endswitch
		break
		default:       // A is on graph
			strswitch (CsrWave(B) )
				case "":  // B is missing
					DoAlert 0, "You have also to use cursor B"
					return 1
				default:
				break // A and B are there, proceed
			endswitch
		endswitch
	string tempTargetName
	variable i=1
	do
		tempTargetName = targetName + num2istr(i)
		i+=1
	while(exists(tempTargetName))
	
	targetName = tempTargetName
	
	//duplicate /O  $(CsrWave(A)), $targetName
	//make targetName one-dimensional
	//Redimension /N=(width,1) $targetName
	Make /O /D /N=(width) $targetName
	SetScale /I x,leftx($CsrWave(A)),rightx($CsrWave(A)),   $targetName
	wave target = $targetName
	intStart = min(qcsr(A),qcsr(B))
	intEnd = max(qcsr(A),qcsr(B))
	//print intStart, intEnd
	for (j = intStart; j < intEnd; j+=1)
		for (k =0; k < width; k += 1)
			target[k] += original[k][j]
		endfor
	endfor
	
	Display /K=1 $targetName
	
	
	//ModifyGraph rgb($targetName) = (0,0,65500)  // Color code it accordingly

end

function alignCurves(limitJump)
	variable limitJump
	variable IndexOfMaximum
	//make an artificial wave

	
	
	string waveTag = CsrWave(A)
	wave original = $waveTag
	IndexOfMaximum = pcsr(A)
	variable width = DimSize(original,0)
	variable height = DimSize(original,1)

	variable i,j,k,line
	
	make /o /d /n=(width)  tempWave
	wave tempWaveR = tempWave
	
//	setScale /I x, -10,10, tempWave           //similar scaling as sample wave

	//now make a wave which finally holds the results
	//duplicate /o sample, result
	duplicate /o original, result
	wave resultR = result   
	
	variable PntsToShift
	variable averageAux = 3
	variable average = 0
	variable averageBest = 0.1
	variable waveMaximum
	for (line = 0; line <height; line+=1)

		for ( i = 0; i < width; i += 1)
			tempWaveR[i] = original[i][line]
		endfor 
		//now we have tempwave and must shift it
		
		duplicate /o tempWaveR, tempWave2
		smooth 10, tempWave2
		WaveStats /Q tempWave2
		
	
		//print V_MaxRowLoc //this is the index of the maximum, which should be zero
		//find point with the maximum averaged environment
		average = 0
		averageBest=0
		waveMaximum = 0
		for ( i = averageAux; i < width-averageAux; i += 1)
			for (j =0; j< 2*averageAux; j+=1)
				average += tempWaveR[i-averageAux+j]
			endfor
			average *= 1/(2*averageAux)
			if (average > averageBest)
				averageBest = average
				waveMaximum = i
			endif
		endfor 
		
		
		PntsToShift = IndexOfMaximum - V_MaxRowLoc
		if (abs(PntsToShift) > limitJump)
		continue
		endif
		if (PntsToShift > 0)
			//for (k=width-1 ; k > (IndexOfMaximum - V_MaxRowLoc-1); k -= 1)
			for (k=width-1 ; k > PntsToShift-1; k -= 1)
				//tempWave[k] = tempWave[k-(IndexOfMaximum-V_MaxRowLoc)]
				tempWaveR[k] = tempWaveR[k-PntsToShift]
			
			endfor
			for (k = 0; k< (PntsToShift); k+=1)
				tempWaveR[k] = -0.1 //mark as invalid for later integrations
			endfor
		else 
			
			for (k=0 ; k < width + PntsToShift; k += 1)
				tempWaveR[k] = tempWaveR[k-PntsToShift]   //PntsToShift < 0  
			endfor
			
			for (k = width-1; k > (width - 1 + PntsToShift); k -= 1)
				tempWaveR[k] = -0.1 //mark as invalid for later integrations
			endfor
		endif
	
		for (j=0; j<width; j+=1)
			resultR[j][line] = tempWaveR[j]
		endfor

	endfor //line
	string outputname = waveTag +"_a"
	duplicate /o result, $outputname
	NewImage /K=1 $outputname
	killwaves /z tempWave, result, tempWave2
end

function Cut2D()
	string targetName = CsrWave(A)+"_c"
	string waveTag = CsrWave(A)
	wave original = $waveTag
	variable width = DimSize($waveTag,0)
	variable IntStart, IntEnd, j,k

	// All that stuff  only prevents user errors - maybe Igor has not the most efficient language
	strswitch (CsrWave(A))
		case "":       //A not on grah
			strswitch (CsrWave(B) )
				case "":  // B missing as well
					DoAlert 0, "Please set cursor A and B on the curve to mark the range wich you want to cut. Use Ctrl+I to activate cursors..."
					return 1
				default:
					DoAlert 0, "You also have to use cursor A"
					return 1 // A is not there but B
			endswitch
		break
		default:       // A is on graph
			strswitch (CsrWave(B) )
				case "":  // B is missing
					DoAlert 0, "You have also to use cursor B"
					return 1
				default:
				break // A and B are there, proceed
			endswitch
		endswitch
	string tempTargetName
	variable i=1
	do
		tempTargetName = targetName + num2istr(i)
		i+=1
	while(exists(tempTargetName))
	
	targetName = tempTargetName
	intStart = min(qcsr(A),qcsr(B))
	intEnd = max(qcsr(A),qcsr(B))
	
	duplicate /O  /R=[][intStart, intEnd] original, $targetName
	NewImage /K=1 $targetName
end

// calculate the Tougaard background of an XPS spectrum
function Tougaard()
	string targetName = CsrWave(A)+"_T"
	variable sampleLen,highX, lowX, step, offset, b
	variable c = 1643
	variable i,j
	// All that stuff  only prevents user errors - maybe Igor has not the most efficient language
	strswitch (CsrWave(A))
		case "":       //A not on grah
			strswitch (CsrWave(B) )
				case "":  // B missing as well
					DoAlert 0, "Please set cursor A and B on the curve to mark the range wich you want to cut. Use Ctrl+I to activate cursors..."
					return 1
				default:
					DoAlert 0, "You also have to use cursor A"
					return 1 // A is not there but B
			endswitch
		break
		default:       // A is on graph
			strswitch (CsrWave(B) )
				case "":  // B is missing
					DoAlert 0, "You have also to use cursor B"
					return 1
				default:
				break // A and B are there, proceed
			endswitch
		endswitch
	string tempTargetName
	i=1
	do
		tempTargetName = targetName + num2istr(i)
		i+=1
	while(exists(tempTargetName))
	targetName = tempTargetName
	duplicate  /O $(CsrWave(A)), $targetName
	duplicate /O $targetName temp
	wave temp = temp
	wave target = $targetName
	wave original = $(CsrWave(A))
	//now do the actual calculation
	sampleLen = numpnts($(CsrWave(A)))
	print "Calculating Tougaard background, Loss parameter: 1643 eV^2, see:"
	print "S.Tougaard; 1989, 216, 343 - 360"
	print "Please note, that this algorithm does not correct for the transmission function of"
	print "the analyzer and the integrals in Tougaards paper are replaced by discrete sums."
	print "Therefore, one should be carefull about relating the paramters of the background to physical quantities"
	if (sampleLen > 1000)
		print "Many points to be considered, calculation may takes a while ..."
	endif
	string xname
	xname = CsrXWave(A)
	if ( stringmatch(CsrXWave(A), "") == 1)
	highX = max( leftx($(CsrWave(A))), rightx($(CsrWave(A))))
	lowX = min( leftx($(CsrWave(A))), rightx($(CsrWave(A))) )
	else
	print "with x-axis"
	highX = leftx($(CsrXwave(A)))
	lowX = rightx($(CsrXwave(A)))
	endif
	if ( leftx($(CsrWave(A) ) )  <= rightx($(CsrWave(A) ) ) )
		for (i = 0; i < sampleLen; i+= 1)   //reverse the wave contents
			temp[i] = target[sampleLen-i-1]
		endfor
		for (i = 0; i < sampleLen; i+= 1)   //reverse the wave contents
			target[i] = temp[i]
		endfor
	endif
	step = abs(highX - lowX)/sampleLen
	
	//get the average value from the cursor
	
	offset=min( vcsr(A), vcsr(B))
	//print offset
	target -= offset  //target contains the same values as the original wave 
	
	variable Integral = 0
	for ( i = 0; i < sampleLen; i += 1)
		Integral += (i*step)/(c+(i*step)^2)^2 * target[i] 
	
	endfor

	//b = mean(sampleWOoffset, xcsr(B)-3, xcsr(B))
	b =  (max (vcsr(A),vcsr(B)) - offset) /Integral 
	variable IntegralUP = Integral
	//variable IntegralDown =Integral
	for ( i = 0; i < sampleLen; i += 1)
		//IntegralDown=0
		IntegralUP = 0
		for ( j = i; j < sampleLen; j += 1)
			IntegralUP += ( ( j - i )*step)/(c+( ( j - i ) * step)^2)^2 *(original[j] - offset)
		endfor
		//for (j=0; j< i; j+= 1)
		//	IntegralDown += (j*step)/(c+(j*step)^2)^2 * (original[j] - offset)
		//endfor
		//IntegralUP = Integral -IntegralDwon
		target[i] = original[i] - offset - b* IntegralUP
		
	endfor
	//print Integral, IntegralUP + IntegralDown
	if ( stringmatch(CsrXWave(A), "") == 1)
		Display /K=1 $targetName
	else
		Display /K=1 $targetName vs $xname
	endif

	ModifyGraph rgb($targetName) = (0,26112,39168)  // Color code it accordingly
	killwaves /Z temp
	SetAxis /A /R bottom
end



//this is only necessary to be downwards-compatible to Igor 5.02
function MyGrepString(testString,ListWithItemsToFind)
	string testString
	string ListWithItemsToFind
	variable found = 0
	variable numItems
	variable i
	numItems = ItemsInList(ListWithItemsToFind)
	string item
	for ( i = 0; i < numItems; i += 1)
		item = ""
		item = StringFromList(i,ListWithItemsToFind, ";")
		//now check if this is contained in testString
		item = "*"+item + "*"
		found = stringmatch(testString,item)
		if (found != 0)
			break
		endif
	endfor

	return found   //0 if not found, 1 if found  0 is returned only if nothing was found in the teststring
	
end

