#pragma rtGlobals=1		// Use modern global access method.
#pragma hide=1

 //   XPST - a macro package for peak fitting with Igor Pro
 //   Copyright (C) 2011  Martin Schmid, University Erlangen

 //   This program is free software: you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License as published by
 //   the Free Software Foundation, either version 3 of the License, or
 //   (at your option) any later version.

 //   This program is distributed in the hope that it will be useful,
 //   but WITHOUT ANY WARRANTY; without even the implied warranty of
 //   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //   GNU General Public License for more details.



//Fit function definition

//The gauss curves are defined in the following way:
//				
//  [ Area *  sqrt{ 4ln(2) / (pi) }   / fwhm  ] * exp( - 4ln(2) * { ( x-x0 ) / fwhm }^2 )  =  Amplitude*exp( - { (x-x0) / width }^2 )
//
//this is possible since the area of a Gauss curve is:    Amplitude*width*sqrt(pi)

//Functions in this file
// Gauss1    to    Gauss4
// VoigtGLS1 to VoigtGLS4



function FitMultiVoigtSK(pw,yw,xw): FitFunc
	wave pw, yw, xw
	duplicate /o yw ShirleyStep  //this wave needs to be removed after the fit has completed
	wave ShirleyStep = ShirleyStep
	duplicate /o yw HGB
	wave hgb=HGB
	duplicate /o yw individualPeak
	wave peak = individualPeak
	
	variable i, index
	variable numPeaks = (numpnts(pw)-5)/6    //use the length of the coefficient wave
	variable totalArea
	variable FWHMoffset = 0.01
	variable partialArea =0
	variable pointlength
	variable temp
	variable areaPeak
	yw = 0
	//those are only the peaks
	for (i = 0; i < numPeaks; i +=1)
		index = 6*i +5

		temp = pw[index + 3]// 0.02 + 0.98* ((pw[index+3]) - floor((pw[index+3])) )   //exactly 0 or 1 are really bad
		//pw[index] = abs(pw[index])
		//pw[index+2] = (pw[index+2]) + 0.001
		peak = 0
		peak += (temp + 0.00001)* ( 2 * (1) / pi ) * ( ( 2 * pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5]) ) ) ) / ( ( FWHMoffset + 2 * pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5] ) ) ) )^2 + 4 * ( xw - pw[index+1] )^2 ) )  
		peak += (1 - 0.99999*temp) * ((1) / ( gPf * ( 2 * pw[index+2]  / ( 1 + exp( -pw[index+4]*( xw - pw[index+1] - pw[index+5] ) ) ) ) ) ) * exp ( -gAf * ( ( xw - pw[index+1] ) / ( FWHMoffset + 2 * pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5]) ) ) ) )^2 ) 
		if (abs(pw[index+4]) > 0.01)  //otherwise it is normalized anyway
			areaPeak= IntegrateSingleVoigtGLS(1,pw[index+1],pw[index+2],pw[index+3],pw[index+4],pw[index+5])
			peak /= areaPeak
			peak *= pw[index]
		else
			peak *= integrationCorrection*pw[index]  //this correction factor accounts for the limited accuracy of the numerical integration here
		endif
		
		yw += peak
	endfor
	
	
	pointLength = numpnts(yw)
	//print "Laenge der yw:", pointLength
	totalArea = sum(yw) 
	partialArea = 0		
	if (xw[0] < xw[2] )  //binding energy increases with point index
		for (i=0; i<pointLength; i += 1) 

			partialArea += abs(yw[i]) 
			ShirleyStep[i] =partialArea/totalArea
		endfor
	else                     //binding energy decreaeses with point index
		for (i=0; i<pointLength; i += 1)
			partialArea += abs(yw[pointLength-1-i])
			ShirleyStep[pointLength-1-i] =partialArea/totalArea 
		endfor
	endif
	
	
	//for ( i = 0; i < numPeaks; i += 1)
	//index = 6*i +5
	//yw += 1e-3*pw[3]*pw[index]*(xw-pw[index+1])^2*(xw>pw[index+1])
	//endfor
	
	//integrate the shirely step again to obtain the HGBackground with the parameter pw[3]
	partialArea = 0.000001
	totalArea = sum(ShirleyStep) 
	if (xw[0] < xw[2] )   //binding energy increases with point index
		for ( i = 0; i < pointLength; i += 1)
			partialArea += abs(ShirleyStep[i])
			hgb[i] = partialArea/totalArea	
		endfor
	else                     //binding energy decreaeses with point index
		for( i = 0; i < pointLength; i += 1)
			partialArea += abs(ShirleyStep[pointLength-1-i])
			hgb[pointLength-1-i] = partialArea/totalArea	
		endfor
	endif
	
	//hgb *= pw[3]
	//ShirleyStep *= pw[4]
	yw +=  pw[3]*hgb
	yw +=  pw[4]*ShirleyStep   //this is necessary, as bgywStart always comes in a range between 0 and 1
	////and now the slope
	yw += pw[0]  + pw[1]*xw + pw[2]*xw^2
	//print numpnts(yw)
//	for (i=0; i<numpnts(yw); i +=1)
	//	temp=yw[i]
	//	if (numtype(temp) != 0)
	//		print "Nan detected"
	//		if (i==0) 
	//			yw[i]=yw[1]	
	//		else		
	//			yw[i]=yw[i-1]
	//		endif
	//	endif
//	endfor
	
		
 	killwaves /Z ShirleyStep,HGB,individualPeak
end


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////			Multiplet
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////			Multiplet
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////			Multiplet
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////			Multiplet

function FitMultipletVoigtSK(pw,yw,xw): FitFunc
	wave pw, yw, xw
	
	//fit a multiplet with 4 peaks
	duplicate /o yw ShirleyStep  //this wave needs to be removed after the fit has completed
	wave ShirleyStep = ShirleyStep
	duplicate /o yw HGB
	wave hgb=HGB
	duplicate /o yw individualPeak
	wave peak=individualPeak
	
	
	variable i, index
	variable numPeaks = (numpnts(pw)-5)/15    //use the length of the coefficient wave
	variable totalArea
	variable FWHMoffset = 0.01
	variable partialArea =0
	variable pointlength
	variable temp
	variable areaPeak = 0
	yw = 0
	//those are only the peaks
	for (i = 0; i < numPeaks; i +=1)
		index = 15*i +5

		peak = 0
		temp = pw[index + 3] // 0.02 + 0.98* ((pw[index+3]) - floor((pw[index+3])) )   //exactly 0 or 1 are really bad,   this has to be corrected

		peak += (temp + 0.01)* ( 2 * (1) / pi ) * ( ( 2 * pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5]) ) ) ) / ( ( FWHMoffset + 2 * pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5] ) ) ) )^2 + 4 * ( xw - pw[index+1] )^2 ) )  
		peak += (1 - 0.99*temp) * ((1) / ( gPf * ( 2 * pw[index+2]  / ( 1 + exp( -pw[index+4]*( xw - pw[index+1] - pw[index+5] ) ) ) ) ) ) * exp ( -gAf * ( ( xw - pw[index+1] ) / ( FWHMoffset + 2 * pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5]) ) ) ) )^2 ) 
		
		if (abs(pw[index+4]) >= 0.01)   //otherwise it is normalized anyway
			areaPeak= IntegrateSingleVoigtGLS(1,pw[index+1],pw[index+2],pw[index+3],pw[index+4],pw[index+5])
			peak /= areaPeak
			peak *= pw[index]
		else
			peak *= integrationCorrection*pw[index]
		endif
		
		yw += peak
		peak = 0
		
		
		//now add the second multiplet peak
		//pw[index+6] = ratio21
		//pw[index+7] = dist21
		//pw[index+8] = broad21
		peak += (temp + 0.01)* ( 2 * (1) / pi ) * ( ( 2 * pw[index+8] * pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5] - pw[index+7] ) ) ) ) / ( ( FWHMoffset + 2 * pw[index+8] * pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5] - pw[index+7]) ) ) )^2 + 4 * ( xw - pw[index+1] - pw[index+7] )^2 ) )  
		peak += (1 - 0.99*temp) * ((1) / ( gPf * ( 2 *  pw[index+8] * pw[index+2]  / ( 1 + exp( -pw[index+4]*( xw - pw[index+1] - pw[index+5] - pw[index+7]) ) ) ) ) ) * exp ( -gAf * ( ( xw - pw[index+1] - pw[index+7]) / ( FWHMoffset + 2 * pw[index+8] *pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5] - pw[index+7] ) ) ) ) )^2 ) 
		if (pw[index+6] != 0 && abs(pw[index+4]) >= 0.01)
			areaPeak= IntegrateSingleVoigtGLS(1,pw[index+1]-pw[index+7],pw[index+2]*pw[index+8],pw[index+3],pw[index+4],pw[index+5])
			peak /= areaPeak
			peak *= pw[index]*pw[index+6]
		else
			peak *= integrationCorrection*pw[index]*pw[index+6]
		endif
		yw += peak
		peak =0
		
		//now do the third peak
		//6 > 9
		//7 >10
		//8 > 11
		peak += (temp + 0.01)* ( 2 * (1) / pi ) * ( ( 2 * pw[index+11] * pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5] - pw[index+10] ) ) ) ) / ( ( FWHMoffset + 2 * pw[index+11] * pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5] -pw[index+10]) ) ) )^2 + 4 * ( xw - pw[index+1]  -  pw[index+10] )^2 ) )  
		peak += (1 - 0.99*temp) * ((1) / ( gPf * ( 2 *  pw[index+11] * pw[index+2]  / ( 1 + exp( -pw[index+4]*( xw - pw[index+1] - pw[index+5]  - pw[index+10]) ) ) ) ) ) * exp ( -gAf * ( ( xw - pw[index+1] -  pw[index+10]) / ( FWHMoffset + 2 * pw[index+11] *pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5]  -pw[index+10] ) ) ) ) )^2 ) 
		if (pw[index+9] != 0  && abs(pw[index+4]) >= 0.01)
			areaPeak= IntegrateSingleVoigtGLS(1,pw[index+1]-pw[index+10],pw[index+2]*pw[index+11],pw[index+3],pw[index+4],pw[index+5])
			peak /= areaPeak
			peak *= pw[index]*pw[index+9]
		else
			peak *= integrationCorrection*pw[index]*pw[index+9]
		endif
		yw += peak
		peak = 0
		//and now the fourth
		// 9>12
		//10>13
		//11>14
		peak += (temp + 0.01)* ( 2 * (1) / pi ) * ( ( 2 * pw[index+14] * pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5]   - pw[index+13] ) ) ) ) / ( ( FWHMoffset + 2 * pw[index+14] * pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5] -  pw[index+13]) ) ) )^2 + 4 * ( xw - pw[index+1]  - pw[index+13] )^2 ) )  
		peak += (1 - 0.99*temp) * ((1) / ( gPf * ( 2 *  pw[index+14] * pw[index+2]  / ( 1 + exp( -pw[index+4]*( xw - pw[index+1] - pw[index+5]  - pw[index+13]) ) ) ) ) ) * exp ( -gAf * ( ( xw - pw[index+1] - pw[index+13]) / ( FWHMoffset + 2 * pw[index+14] *pw[index+2] / ( 1 + exp( - pw[index+4]*( xw - pw[index+1] - pw[index+5]  - pw[index+13] ) ) ) ) )^2 ) 
		if (pw[index+12] != 0 &&  abs(pw[index+4]) >= 0.01)
			areaPeak= IntegrateSingleVoigtGLS(1,pw[index+1]-pw[index+13],pw[index+2]*pw[index+14],pw[index+3],pw[index+4],pw[index+5])
			peak /= areaPeak
			peak *= pw[index]*pw[index+12]
		else
			peak *= integrationCorrection*pw[index]*pw[index+12]
		endif
		
		yw += peak
		peak = 0
	endfor
	
	
	pointLength = numpnts(yw)
	//print "Laenge der yw:", pointLength
	totalArea = sum(yw) 
	partialArea = 0		
	if (xw[0] < xw[2] )  //binding energy increases with point index
		for (i=0; i<pointLength; i += 1) 

			partialArea += abs(yw[i]) 
			ShirleyStep[i] =partialArea/totalArea
		endfor
	else                     //binding energy decreaeses with point index
		for (i=0; i<pointLength; i += 1)
			partialArea += abs(yw[pointLength-1-i])
			ShirleyStep[pointLength-1-i] =partialArea/totalArea 
		endfor
	endif
	
	
	//integrate the shirely step again to obtain the HGBackground with the parameter pw[3]
	partialArea = 0.000001
	totalArea = sum(ShirleyStep) 
	if (xw[0] < xw[2] )   //binding energy increases with point index
		for ( i = 0; i < pointLength; i += 1)
			partialArea += abs(ShirleyStep[i])
			hgb[i] = partialArea/totalArea	
		endfor
	else                     //binding energy decreaeses with point index
		for( i = 0; i < pointLength; i += 1)
			partialArea += abs(ShirleyStep[pointLength-1-i])
			hgb[pointLength-1-i] = partialArea/totalArea	
		endfor
	endif
	

	yw +=  pw[3]*hgb
	yw +=  pw[4]*ShirleyStep   //this is necessary, as bgywStart always comes in a range between 0 and 1
	////and now the slope
	yw += pw[0]  + pw[1]*xw + pw[2]*xw^2

 	killwaves /Z ShirleyStep,HGB, individualPeaks
end
