#pragma rtGlobals=1	// Use modern global access method and strict wave access.
#pragma hide=1

//////////////////////////////////////////////////////////////////////////////////
//Licence
//////////////////////////////////////////////////////////////////////////////////
//
//	If you use the 'Fit Assistant' to analyze data, please read and cite the following paper:
//	Surace and Interface Analysis, (2014), 46, 505-511
//	This publication contains more details on the fit function used by XPST.
//
//	XPST is a front-end to the IgorPro fitting engine. Although every possible effort was made 
//	to prevent serious bugs and wrong results, the author can not give any warranty for XPST as 
//	large parts actually depend on the underlying IgorPro code. 
//	Author: Dr. Martin Schmid
//
//////////////////////////////////////////////////////////////////////////////////




//this file contains the following instructions
//  1. adding a peak to the peak editor
//  2. displaying a peak in the fit window
//  3. instructions how to plot and evaluate the fit results
//  4. Auxiliary functions
//add a simple voigt function

function AddVoigtPeak(coefWave,indepCheck,heightA,posA,left,right,Wcoef_length)

string coefWave
variable indepCheck                //decide whether to link the new peak to peak 1 or not
variable heightA
variable posA	
variable left
variable right
variable Wcoef_length

wave locRefW_coef = W_Coef
wave /t Min_Limit = Min_Limit   //just create a local refernce, do not get confused by the identical names
wave /t Max_Limit = Max_Limit 
wave /t locRefConstraints = T_Constraints
wave /t locRefLegend = LegendWave
wave /t CoefLegendRef = CoefLegend
wave holdRef = hold                      //The global wave is named "hold"
wave epsilon = epsilon
	
variable nPeaks,i,numPara,EstimatedPeakArea
variable epsilonVal=1e-5
			numPara = 6
			nPeaks =WCoef_length/numPara
			redimension /n=(WCoef_length+numPara)  locRefW_coef
			redimension /n=(WCoef_length+numPara) Min_Limit
			redimension /n=(WCoef_length+numPara) Max_Limit
			redimension /n=(WCoef_length+numPara) CoefLegendRef        // This approach is much easier, than the other, using Temporary waves
			redimension /n=(WCoef_length+numPara) holdRef
			redimension /n=(WCoef_length+numPara) epsilon
			redimension /n=(2*(WCoef_length+numPara)) locRefConstraints
			
			if (WCoef_length == 0 )                                //the first peak gets these values
				EstimatedPeakArea = EstimatePeakArea( heightA , Width_Start, 5 * GLratio_Min, 5 * Asym_Min , 5 * Asym_Shift_Min)	
				locRefW_coef[0] = EstimatedPeakArea  		//now append the current cursor settings
				locRefW_coef[1]  = posA
				locRefW_coef[2] = Width_Start
				locRefW_coef[3] = GLratio_Start
				locRefW_coef[4] = Asym_Start
				locRefW_coef[5] = Asym_Shift_Start
			else                                                              // all further peaks are related to those values
				 //now append the current cursor settings
				locRefW_coef[WCoef_length+1]  = posA
				if ( indepCheck == 1 )
					// This is a trick to calculate a good estimator for the peak area form the cursor height
					//     so that the resulting peak has its maximum also roughly at the cursor height
					//  First assume that the fit coefficient would be 1
					// Calculate the maximum peak height with this coefficient at the peak position
					// Compare this with the desired peak height wich is given by the cursor
					// Multiply the initial fit coefficient of 1 with number how many times the y value of the cursor is higher
					EstimatedPeakArea = EstimatePeakArea ( heightA , locRefW_coef[2] , locRefW_coef[3] , locRefW_coef[4] , locRefW_coef[5] )						
					locRefW_coef[WCoef_length] = EstimatedPeakArea  
					locRefW_coef[WCoef_length+2] = locRefW_Coef[2]                 //by default, everything is linked to peak 1	
					locRefW_coef[WCoef_length+3] = locRefW_Coef[3]
					locRefW_coef[WCoef_length+4] = locRefW_Coef[4]
					locRefW_coef[WCoef_length+5] = locRefW_Coef[5]
				else
					EstimatedPeakArea = EstimatePeakArea( heightA , Width_Start, 5 * GLratio_Min, 5 * Asym_Min , 5 * Asym_Shift_Min)						
					locRefW_coef[WCoef_length] = EstimatedPeakArea
					locRefW_coef[WCoef_length+2] = Width_Start             
					locRefW_coef[WCoef_length+3] = GLratio_Start
					locRefW_coef[WCoef_length+4] = Asym_Start
					locRefW_coef[WCoef_length+5] = Asym_Shift_Start
				endif
			endif
			
			if (WCoef_length == 0)
				Min_Limit[0] = "K0 >  " + num2str(min(1e-5,0.2 * EstimatedPeakArea ))  // the first one has to be finite not zero
				Max_Limit[0] ="K0 <  " + num2str(max(10,10 * EstimatedPeakArea ))
				Min_Limit[1] = "K1 >  " + num2str(right)
				Max_Limit[1] ="K1 <  " + num2str(left )
				Min_Limit[2] = "K2 >  " + num2str(Width_Min)
				Max_Limit[2] ="K2 <  " + num2str(Width_Max )
				Min_Limit[3] = "K3 >  " + num2str(GLratio_Min)
				Max_Limit[3] = "K3 <  " + num2str(GLratio_Max)
				Min_Limit[4] = "K4 >  " + num2str(Asym_Min)
				Max_Limit[4] = "K4 <  " + num2str(Asym_Max)
				Min_Limit[5] = "K5 >  " + num2str(Asym_Shift_Min)
				Max_Limit[5] = "K5 <  " + num2str(Asym_Shift_Max)
			else 
				Min_Limit[WCoef_length] = "K"  + num2istr(WCoef_length) + " >  "+ num2str(min(1e-5,0.2 * EstimatedPeakArea ) ) 
				Max_Limit[WCoef_length] ="K"  + num2istr(WCoef_length) + " <  " + num2str(max(10,10 * EstimatedPeakArea ) )
				Min_Limit[WCoef_length+1] =  "K" + num2istr(WCoef_length+1) + " >  "+ num2str(right)
				Max_Limit[WCoef_length+1] ="K" + num2istr(WCoef_length+1) + " <  " +num2str(left)
				
				if ( indepCheck == 1 )
					Min_Limit[WCoef_length+2] = "K" + num2istr(WCoef_length+2) + " >  K2 - " + num2str(interval)
					Max_Limit[WCoef_length+2] ="K" + num2istr(WCoef_length+2) + " <  K2 + " + num2str(interval)
					Min_Limit[WCoef_length+3] = "K" + num2istr(WCoef_length+3) + " >  K3 - " + num2str(interval)
					Max_Limit[WCoef_length+3] ="K" + num2istr(WCoef_length+3) + " <  K3 +" + num2str(interval)
					Min_Limit[WCoef_length+4] = "K" + num2istr(WCoef_length+4) + " >  K4 - " + num2str(interval)
					Max_Limit[WCoef_length+4] ="K" + num2istr(WCoef_length+4) + " <  K4 + " + num2str(interval)
					Min_Limit[WCoef_length+5] = "K" + num2istr(WCoef_length+5) + " >  K5 - " + num2str(interval)
					Max_Limit[WCoef_length+5] ="K" + num2istr(WCoef_length+5) + " <  K5 + " + num2str(interval)
				else
					Min_Limit[WCoef_length+2] = "K" + num2istr(WCoef_length+2) + " >  " + num2str(Width_Min)
					Max_Limit[WCoef_length+2] = "K" + num2istr(WCoef_length+2) + " <  " + num2str(Width_Max)
					Min_Limit[WCoef_length+3] = "K" + num2istr(WCoef_length+3) + " >  " + num2str(GLratio_Min)
					Max_Limit[WCoef_length+3] = "K" + num2istr(WCoef_length+3) + " <  " + num2str(GLratio_Max)
					Min_Limit[WCoef_length+4] = "K" + num2istr(WCoef_length+4) + " >  " + num2str(Asym_Min)
					Max_Limit[WCoef_length+4] = "K" + num2istr(WCoef_length+4) + " <  " + num2str(Asym_Max)
					Min_Limit[WCoef_length+5] = "K" + num2istr(WCoef_length+5) + " >  " + num2str(Asym_Shift_Min)
					Max_Limit[WCoef_length+5] = "K" + num2istr(WCoef_length+5) + " <  " + num2str(Asym_Shift_Max)	
				endif			
			endif
			
			for (i=0; i <  numpnts(Min_Limit); i+=1)
				locRefConstraints[2*i] = Min_Limit[i]
				locRefConstraints[2*i+1] = Max_Limit[i]
			endfor

			epsilon[Wcoef_length] = epsilonVal
			epsilon[Wcoef_length+1] =epsilonVal
			epsilon[Wcoef_length+2] = epsilonVal
			epsilon[Wcoef_length+3] =epsilonVal
			epsilon[Wcoef_length+4] = epsilonVal
			epsilon[Wcoef_length+5] = epsilonVal
			
			holdRef[Wcoef_length] = 0
			holdRef[Wcoef_length+1] = 0
			holdRef[Wcoef_length+2] = 0
			holdRef[Wcoef_length+3] = 0
			holdRef[Wcoef_length+4] = 1
			holdRef[Wcoef_length+5] = 1

			CoefLegendRef[WCoef_length] = " K" + num2istr(WCoef_length) + " : Area ------------- Peak: #"  + num2istr(nPeaks+1)
			CoefLegendRef[WCoef_length+1] = " K" + num2istr(WCoef_length+1) + " : Position"
			CoefLegendRef[WCoef_length+2] = " K" + num2istr(WCoef_length+2) + " : Width"
			CoefLegendRef[WCoef_length+3] = " K" + num2istr(WCoef_length+3) + " : GL ratio"
			CoefLegendRef[WCoef_length+4] = " K" + num2istr(WCoef_length+4) + " : Asymmetry"
			CoefLegendRef[WCoef_length+5] = " K" + num2istr(WCoef_length+5) + " : Asym. Shift"
end

// 2.////////////////////////////// Display the peak in the fit window correctly

function PlotPseudoVoigtDisplay(peakType,RawYWave, RawXWave,coefWave)

	//SVAR peakType = root:S_PeakType
	//SVAR RawYWave = root:S_nameWorkWave
	//SVAR coefWave = root:S_CoefWave
	//SVAR RawXWave = root:S_XRawDataCursorPanel
	
	string peakType
	string RawYWave
	string RawXWave
	string coefWave
	string TagName    // the Tag in the result window
	string PeakTag     // text in this tag
	string PkName, parentDataFolder //, cleanUpString=""		
	string BGName //background
	string PeakSumName
	NVAR FitMin = STFitMin
	NVAR FitMax = STFitMax
	
	wave cwave = $coefWave
	wave raw = $RawYWave
//	wave xraw = $RawXWave
	variable LenCoefWave = DimSize(cwave,0)
	
	//create some waves, to display the peak
	variable nPeaks = 0
	variable numCoef
	variable i,index
	variable xmin, xmax, step
	variable TagPosition   //the position of the tag in the result window
	variable totalPeakSumArea, partialPeakSumArea
	
	duplicate /o /R=(FitMin,FitMax) raw WorkingDuplicate   
	strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
		case "":                                             //if empty
			                     
			break
		case "_calculated_":
			 
			break
		default:                                                 // if not empty, x-axis wave neccessary
			//read in the start x-value and the step size from the x-axis wave
			wave xraw = $RawXWave
			xmax = max(xraw[0],xraw[numpnts(xraw)-1] )
			xmin = min(xraw[0],xraw[numpnts(xraw)-1] )
			step = (xmax - xmin ) / DimSize(xraw,0)
			// now change the scaling of the y-wave duplicate, so it gets equivalent to a data-wave imported from an igor-text file
			duplicate /o raw tempWaveForCutting  
			SetScale /I x, xmin, xmax, tempWaveForCutting  //OKAY, NOW THE SCALING IS ON THE ENTIRE RANGE
			duplicate /o /R=(FitMin,FitMax) tempWaveForCutting WorkingDuplicate  
			killwaves /z tempWaveForCutting
			break
	endswitch
	
	parentDataFolder = GetDataFolder(1)
	
	
	//now make tabular rasa in the case of background functions
	string ListOfCurves = TraceNameList("CursorPanel#guiCursorDisplayFit",";",1)
	variable numberCurves = ItemsInList(ListOfCurves)
	//remove only the very last curve, if there are e.g. 3 curves on the graph it has the index #2
	
	// If a wave is given which needs an external x-axis (from an ASCII-file) create a duplicate which receives a proper x-scaling later on
	// the original wave will not be changed
	              
	NewDataFolder /O /S FitTemp

	
 			numCoef = 6   //Voigt
			nPeaks = LenCoefWave/numCoef
			for (i =0; i<nPeaks;i+=1)
				index = numCoef*i
				PkName = "peak_" + num2istr(i+1)   //make a propper name
			 	//create a wave with this name and the correct  scaling and number of datapoints -> copy the fitted wave
				duplicate /o WorkingDuplicate $PkName			
				wave tempDisplay = $PkName        //This needs some explanation, see commentary at the end of the file                                        
			 
				 //overwrite the original values in the wave with the values of a single peak
				tempDisplay = CalcSingleVoigtGLS(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5],x)
			
				RemoveFromGraph /W=CursorPanel#guiCursorDisplayFit /Z  $PkName#0
				AppendToGraph /w= CursorPanel#guiCursorDisplayFit tempDisplay                           //now plot it
				// and add a tag
				WaveStats /Q tempDisplay
				tagName = PkName+num2istr(i)
				PeakTag = num2istr(i+1)
				TagPosition = V_maxloc
				
				Tag /w= CursorPanel#guiCursorDisplayFit /C /N= $tagName  /F=0 /L=1  /Y =2.0  $PkName, TagPosition ,PeakTag
				ModifyGraph /w= CursorPanel#guiCursorDisplayFit rgb($PkName)=(0,0,0)       // and color-code it	
			endfor
			
		
	WaveStats /Q WorkingDuplicate
//	SetAxis /w = CursorPanel#guiCursorDisplayFit left -0.1*V_max, 1.1*V_max
	ModifyGraph /w= CursorPanel#guiCursorDisplayFit zero(left)=2 
	SetAxis/A/R /w = CursorPanel#guiCursorDisplayFit bottom
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit mirror(left)=2
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit mirror(bottom)=2
	Label  /w = CursorPanel#guiCursorDisplayFit Bottom "\\f01 binding energy (eV)"
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit minor(bottom)=1,sep(bottom)=2
	SetDataFolder parentDataFolder 
	killwaves /Z WorkingDuplicate
end







// 3 //////////////////////    draw and evaluate the fit results from Voigt peaks ////////////////////////////////////////////////////////////////////////////////////////////////////////

function DrawAndEvaluatePseudoVoigt(dataWave,fitWave,peakType,RawXWave,newFolder)

string dataWave
string fitWave
string peakType
string RawXWave
variable newFolder               //if this value is different from 1, no folder for the results will be created

wave cwave = W_coef
wave origWave = $dataWave
wave fitted = $fitWave
wave epsilon = epsilon
wave hold = hold
wave InitializeCoef = InitializeCoef
wave Min_Limit = Min_Limit
wave Max_Limit = Max_Limit
wave T_Constraints = T_Constraints
wave  CoefLegend = CoefLegend

if ( strlen(fitWave) >= 30)	
	doalert 0, "The name of the fit-wave is too long! Please shorten the names."
	return -1
endif


//define further local variables
variable LenCoefWave = DimSize(cwave,0)	
variable nPeaks
variable index
variable i =0                               //general counting variable
variable numCoef                       //variable to keep the number of coefficients of the selected peak type
							  // numCoef = 3   for Gauss Singlet     and numCoef =5 for VoigtGLS
variable pointLength, totalArea, partialArea
variable peakMax 
variable TagPosition
variable AnalyticalArea
variable EffectiveFWHM
variable GeneralAsymmetry        //  = 1 - (fwhm_right)/(fwhm_left)

string PkName                          //string to keep the name of a single peak wave
string foldername                       //string to keep the name of the datafolder, which is created later on for the single peak waves
string tempFoldername               //help-string to avoid naming conflicts
string parentDataFolder
string TagName
string PeakTag
string LastGraphName
string NotebookName = "Report"     //this is the initial notebook name, it is changed afterwards
string tempNotebookName
string tempString                          // for a formated output to the notebook
string BGName
//The following switch construct is necessary in order to plot waveform data (usually from igor-text files , *.itx) as well as
//raw spectra which need an extra x-axis (such data come usually from an x-y ASCII file)

strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
	case "":                                             //if empty
		display /K=1 origWave                 //the raw-data wave is in true waveform and has an intrinsic scale                     
		break
	case "_calculated_":
		display /K=1 origWave
		break
	default:
		wave xraw = $RawXWave                                                 // if not empty
		display /K=1 origWave vs xraw        // the raw-data wave has no intrinsic scale and needs a propper x-axis 
		break
endswitch
ModifyGraph mode($dataWave)=3,msize($dataWave)=1.3, marker($dataWave)=8
ModifyGraph mrkThick($dataWave)=0.7
ModifyGraph rgb($dataWave)=(0,0,0)           //color-code it
    
LastGraphName = WinList("*", "", "WIN:")    //get the name of the graph

//check if this Notebook already exists
V_Flag = 0
DoWindow $NotebookName   
// if yes, construct a new name
if (V_Flag)
	i = 1
	do 
		tempNoteBookName = NotebookName + num2istr(i)
		DoWindow $tempNotebookName
		i += 1
	while (V_Flag)
	NotebookName = tempNotebookName 
endif
//if not, just proceed

NewNotebook /F=1 /K=1 /N=$NotebookName      //make a new notebook to hold the fit report
Notebook $NoteBookName ,fsize=8
//Notebook $NoteBookName ,text="\r\r \t\t --- if necessary, insert plot by copy and paste ----    "
Notebook $NoteBookName ,text="\r\r\rPeak Shape:    "+ peakType

//prepare a new datafolder for the fitting results, in particular the single peaks
parentDataFolder = GetDataFolder(1)    //get the name of the current data folder

if (newFolder == 1)
	NewDataFolder /O /S subfolder
	//now, this folder is the actual data folder, all writing is done here and not in root
endif

duplicate /o fitted tempFitWave
wave locRefTempFit = tempFitWave
locRefTempFit = 0

strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
	case "":                                             //if empty
		 sprintf  tempString, "\r\rIntegral area of the raw spectrum (wave area): %20.2f " ,  area(origWave)           
		break
	case "_calculated_":
		sprintf tempString, "\r\rIntegral area of the raw spectrum (wave area): %20.2f " ,  area(origWave)
		break
	default:                                                 // if not empty
		sprintf tempString, "\r\rIntegral area of the raw spectrum (wave area): %20.2f " ,  areaXY(xraw,origWave)
		break
endswitch
Notebook $NoteBookName ,text=tempString

//take the fit-result and analyze the maximum, get the maximum signal, so a significance threshold can be calculated
//WaveStats /Q fitted
//peakMax = V_max

//now decompose the fit into single peaks --- if a further fit function is added, a further "case" has to be attached

		numCoef = 6
		nPeaks = LenCoefWave/numCoef                                             //get the number of  peaks from the output wave of the fit
		
		if ( mod(LenCoefWave,numCoef) != 0 )
			DoAlert 0, "Missmatch, probably wrong peak type selected or wrong coefficient file, retry "
			SetDataFolder parentDataFolder 
			KillDataFolder /Z subfolder
			print " ******* Peak type mismatch - check your fit and peak type ******"
			return 1
		endif 
		
		Notebook $NoteBookName ,text="\rNumber of peaks:    " + num2istr(nPeaks)
		//Notebook $NoteBookName ,text="\rPeak No \t  Area\t\t\tPosition \t  FWHM \t  G-L ratio (0:Gauss)  Asymmetryfactor \t Asym. Shift\r " 
		
		for (i =0; i<nPeaks;i+=1)
			index = numCoef*i
			//make a proper name for the single peak wave
			PkName = "p" + num2istr(i+1)+"_" + dataWave
			 //create a wave with this name and the correct scaling 
			 //and number of datapoints -> copy the fit wave and give the copy the name PkName    
			duplicate /o fitted $PkName	                                                  									  
			wave W = $PkName     //This needs explanation, see comments at the end of the file                                                 
 			//overwrite the original values in the wave with the values of a single peak   
			//The function CalcSingleVoigtGLS is defined in the file FitAssistFunc.ipf 
			W = CalcSingleVoigtGLS(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5],x)
			locRefTempFit += W
			AppendToGraph $PkName                                                        //now plot it
			
			//append the peak-tags to the graph, let the arrow point to a maximum
			WaveStats /Q W                             // get the location of the maximum
			TagName = "tag"+num2istr(i)           //each tag has to have a name
			PeakTag = num2istr(i+1)                 // The tag displays the peak index
			TagPosition = V_maxloc                 // and is located at the maximum
			Tag  /C /N= $TagName  /F=0 /L=1  /Y =2.0  $PkName, TagPosition ,PeakTag    // Now put the tag there
			
			ModifyGraph rgb($PkName)=(30464,30464,30464)              // color code the peak
				
			AnalyticalArea =  IntegrateSingleVoigtGLS(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5])
			
			EffectiveFWHM = CalcFWHMSingleVoigtGLS(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5])
			GeneralAsymmetry = CalcGeneralAsymmetry(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5])   
			
			sprintf  tempString, "\r\r %1g	Area                                   |  Position |  FWHM   | GL-ratio  |   Asym.   | Asym. S. |\r",(i+1)
			Notebook $NoteBookName ,text=tempString
			sprintf  tempString, "---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------" 
			Notebook $NoteBookName ,text=tempString
			sprintf  tempString, "\r       %16.4g   'Fit-coefficient'  |  %8.2f  |  %8.2f  |  %8.2f  |  %8.2f  |  %8.2f    |\r" ,  cwave[index], cwave[index+1] ,cwave[index+2] ,cwave[index+3] , cwave[index+4],cwave[index+5]
			Notebook $NoteBookName ,text=tempString
			sprintf  tempString, "   %20.3g   'Fit wave area' \t Effective maximum position %8.2f \r",  area(W), V_maxloc  // "-> In case of asymmetry, this value does not represent an area any more"
			Notebook $NoteBookName ,text=tempString
			sprintf tempString, "   %20.3g   'Analytical area' \t Effective Asymmetry = 1 - (fwhm_right)/(fwhm_left) %8.2f \r" ,	AnalyticalArea, GeneralAsymmetry
			Notebook $NoteBookName ,text=tempString
			sprintf  tempString, "   \t\t\t \t Effective FWHM: %8.2f \r"	EffectiveFWHM	
			Notebook $NoteBookName ,text=tempString
		endfor
		
		sprintf tempString, "\r'Singlet area':     Identical to the Fit coefficient - not significant for asymmetric peaks!"
		Notebook $NoteBookName ,text=tempString
		sprintf  tempString,"\r'Fit wave area':    Peak area within the measured energy range"
		Notebook $NoteBookName ,text=tempString																			    
		sprintf tempString,"\r'Analytical area': Obtained by numerically integrating the peak from 0 to (position + 1000eV)\r                        Significant also for asymmetric peaks - approximates the peak area from -INF to + INF\r"
		Notebook $NoteBookName ,text=tempString
		sprintf tempString,"\r\rPlease note that for asymmetric peaks the fit coefficients do not describe the physically relevant quantities any more. \rPlease refer to the respective 'effective' or 'analytical' values.\r"      
		Notebook $NoteBookName ,text=tempString
		

killwaves /z  fitted     //this applies to the original fit-wave of Igor, since it is a reference to the wave root:Igor-FitWave, the original wave is possibly wrong
duplicate /o locRefTempFit, $fitWave               //but we are still in the subfolder, 
killwaves /z locRefTempFit
wave fitted = $fitWave

//go back to the parent data folder
if (newFolder ==1)
	SetDataFolder parentDataFolder 

	//create a copy of the coefficient wave in the subfolder, so the waves 
	//and the complete fitting results are within that folder
	duplicate /o :$dataWave, :subfolder:$dataWave
	if (WaveExists($RawXWave))
		duplicate /o :$RawXWave, :subfolder:$RawXWave
	endif

	//duplicate  :$fitWave, :subfolder:$fitWave
	killwaves /z :$fitWave            //probably fails, if the fit wave is displayed in the main panel as well
	AppendToGraph :subfolder:$fitWave                                 //draw the complete fit
	ModifyGraph rgb($fitWave) = (0,0,0)       //color-code it
	//Remove the original wave, which is located in the parent directory and replace it by the copy in the subfolder
	RemoveFromGraph $"#0" 
	strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
		case "":                                             //if empty
			AppendToGraph :subfolder:$dataWave                 //the raw-data wave is in true waveform and has an intrinsic scale                     
			break
		case "_calculated_":
			AppendToGraph  :subfolder:$dataWave
			break
		default:                                                 // if not empty
			AppendToGraph :subfolder:$dataWave vs :subfolder:$RawXWave        // the raw-data wave has no intrinsic scale and needs a propper x-axis 
		break
	endswitch

	
	duplicate :T_Constraints, :subfolder:T_Constraints
	duplicate :Min_Limit, :subfolder:Min_Limit
	duplicate :Max_Limit, :subfolder:Max_Limit
	//if (WaveExists(LegendWave))
	//	duplicate :LegendWave, :subfolder:LegendWave
	//endif

	if (WaveExists(InitializeCoef))
		duplicate InitializeCoef, :subfolder:InitializeCoef
	endif
 	if (exists("epsilon"))
 		duplicate epsilon :subfolder:epsilon
 	endif
 	duplicate hold, :subfolder:hold
 	duplicate CoefLegend, :subfolder:CoefLegend
	duplicate cwave, :subfolder:W_Coef                //now create that copy, leave the original 
														//coefficient wave where it is, maybe the 
														//user wants to try another fit

//Now rename the subfolder accordingly
//If there are any datafolders with peak-waves from previous runs are present, do
//not overwrite them, but create a foldername with a running index
// if a folder named "test" was present, name the new folder "test1", the next one "test2" and so on

	foldername = "V"+num2istr(nPeaks)+ "_" + dataWave
	
	//foldername ="Fit_"+ dataWave
	i=1   //index variable for the data folder	
	tempFoldername = foldername            //used also for the notebook
	if (DataFolderExists(foldername))    //is there already a folder with this name
	//if yes:
	 //Append an index number to the current foldername and check if a folder with this name
	 //is already present. If so, increase the index number by one, append it to the basic foldername
	 //and check again. Repeat this procedure until a "free" name is found
		do
			tempFoldername = foldername + num2istr(i)
			i+=1
		while(DataFolderExists(tempFoldername))
		
		if ( strlen(tempFoldername) >= 30)	
			doalert 0, "The output folder name is too long! Please shorten the names. The output folder of the current run is named 'subfolder'."
		else
			RenameDataFolder subfolder, $tempFoldername                   //now rename the peak-folder accordingly
			Notebook  $NoteBookName, text="\r------------------------------------------------------------------------------"
			//remove illegal characters from the string
			tempFoldername = stringClean(tempFoldername)
			DoWindow /C $tempFoldername
			DoWindow /F $LastGraphName
			DoWindow /C $tempFoldername + "_graph"
			//DoWindow /C /W=$LastGraphName $tempFoldername + "_graph"   //valid in Igor 6.x
		endif
		//TextBox/N=text0/A=LT tempFoldername        //prints into the graph
	else 
		//if no:
		RenameDataFolder subfolder, $foldername 
		//TagWindow(foldername)
		Notebook  $NoteBookName, text="\r------------------------------------------------------------------------------"
		//if ( strsearch(foldername,".",0) != -1 )      // strsearch returns -1 if  the string contains no "." 
		//	foldername = ReplaceString(".", foldername,"_")
		//endif
		foldername = stringClean(foldername)
		tempFoldername = stringClean(tempFoldername)
		DoWindow /C $foldername
		DoWindow /F $LastGraphName
		DoWindow /C $tempFoldername + "_graph"
		//DoWindow /C /W=$LastGraphName $tempFoldername + "_graph"
		//TextBox/N=text0/A=LT foldername               //prints into the graph
	endif
	//everything up to now was done in a subfolder
else          //varialbe newFolder different from 1  create no new folder
	AppendToGraph $fitWave
	ModifyGraph rgb($fitWave) = (0,0,0)
	SetAxis/A/R bottom
	//find a name for the graph and the notebook, that reflects the location of the corresponding data
	String location = GetDataFolder (0)
	strswitch(location)
		case "root":
			tempFoldername = "fit" + dataWave
			break
		default:
			tempFoldername = location
			break
	endswitch

	Notebook  $NoteBookName, text="\r------------------------------------------------------------------------------"
	
	//if ( strsearch(tempFoldername,".",0) != -1 )      // strsearch returns -1 if  the string contains no "." 
	//	tempFoldername = ReplaceString(".", tempFoldername,"_")
	//endif
	tempFoldername = stringClean(tempFoldername)
	V_Flag = 0
	DoWindow $tempFoldername    //check if it is there, if yes V_Flag is set to 1
	if (V_Flag)
		do
			V_Flag = 0   // it should be possible to use the cancel button
			Prompt tempFoldername, "There is already a report on this folder, please enter a new for the new report."
			DoPrompt /Help="Please use only letters, numbers and an underscore!" "Please enter a new name", tempFoldername
			if (V_Flag)  //user pressed cancel
				DoWindow /K Report //kill the notebook
				DoWindow /K $LastGraphName
				
				killvariables  /Z V_chisq, V_numNaNs, V_numINFs, V_npnts, V_nterms,V_nheld,V_startRow, V_Rab, V_Pr
				killvariables  /Z V_endRow, V_startCol, V_endCol, V_startLayer, V_endLayer, V_startChunk, V_endChunk, V_siga, V_sigb,V_q,VPr
				return -1
			endif
			tempFoldername = stringClean(tempFoldername)
			DoWindow $tempFoldername    //check again and
		while (V_Flag)                                 //repeat until
		DoWindow /C $tempFoldername
		DoWindow /F $LastGraphName
		DoWindow /C $tempFoldername + "_graph"
		//DoWindow /C /W=$LastGraphName $tempFoldername + "_graph"
	else
		DoWindow /C $tempFoldername
		DoWindow /F $LastGraphName
		DoWindow /C $tempFoldername + "_graph"
		//DoWindow /C /W=$LastGraphName $tempFoldername + "_graph" 
	endif
	//TextBox/N=text0/A=LT "fit"+dataWave
endif
 
//make the graph look good
ModifyGraph mode($dataWave)=3 ,msize($dataWave)=1.3 // ,marker($dataWave)=8, opaque($dataWave)=1
ModifyGraph opaque=1,marker($dataWave)=19
ModifyGraph rgb($dataWave)=(60928,60928,60928)
ModifyGraph useMrkStrokeRGB($dataWave)=1
ModifyGraph mrkStrokeRGB($dataWave)=(0,0,0)
//ModifyGraph mrkThick($dataWave)=0.7
//ModifyGraph rgb($dataWave)=(0,0,0)           //color-code it
ModifyGraph mirror=2,minor(bottom)=1
Label left "\\f01 intensity (counts)"
Label bottom "\\f01  binding energy (eV)"	
ModifyGraph width=255.118,height=157.465, standoff = 0, gfSize=11

//The following command works easily, but then the resulting graph is not displayed properly in the notebook
//SetAxis/A/R bottom
//instead do it like this:
variable left,right

strswitch(RawXWave)
	 	case "":
	 		 left = max(leftx($dataWave),rightx($dataWave))
	 		 right = min(leftx($dataWave),rightx($dataWave))
	 	break
	 	case "_calculated_":
	 		 left = max(leftx($dataWave),rightx($dataWave))
	 		 right = min(leftx($dataWave),rightx($dataWave))
	 	break
	 	default:
	 		waveStats /Q $RawXWave
	 		left = V_max
	 		right = V_min
	 	break
 endswitch
 SetAxis bottom left,right
//okay this is not perfectly elegant ... but

WaveStats /Q $dataWave
SetAxis left V_min-0.05*(V_max-V_min), 1.02*V_max
LastGraphName = WinList("*", "", "WIN:")



Notebook  $tempFoldername, selection = {startOfFile,startOfFile}
tempString = "Fitting results for: " + tempFoldername + "\r"
Notebook  $tempFoldername, text=tempString
Notebook  $tempFoldername, selection = {startOfPrevParagraph,endOfPrevParagraph}, fsize = 12, fstyle = 0

Notebook  $tempFoldername, selection = {startOfNextParagraph,startOfNextParagraph}
Notebook  $tempFoldername, picture={$LastGraphName, 0, 1} , text="\r" 




//Notebook  $tempFoldername, text="\r \r"  

//now clean up
killvariables  /Z V_chisq, V_numNaNs, V_numINFs, V_npnts, V_nterms,V_nheld,V_startRow, V_Rab, V_Pr
killvariables  /Z V_endRow, V_startCol, V_endCol, V_startLayer, V_endLayer, V_startChunk, V_endChunk, V_siga, V_sigb,V_q,VPr

end 


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Doublet //Doublet  //Doublet  //Doublet  //Doublet  //Doublet  //Doublet  //Doublet  //Doublet  //Doublet  //Doublet
 //Doublet  //Doublet  //Doublet  //Doublet  //Doublet  //Doublet  //Doublet  //Doublet  //Doublet  //Doublet  //Doublet

function AddVoigtDoublet(coefWave,indepCheck,heightA,posA,left,right,Wcoef_length)

string coefWave
variable indepCheck                //decide whether to link the new peak to peak 1 or not
variable heightA
variable posA	
variable left
variable right
variable Wcoef_length

wave locRefW_coef = W_Coef
wave /t Min_Limit = Min_Limit   //just create a local refernce, do not get confused by the identical names
wave /t Max_Limit = Max_Limit 
wave /t locRefConstraints = T_Constraints
wave /t locRefLegend = LegendWave
wave /t CoefLegendRef = CoefLegend
wave holdRef = hold                      //The global wave is named "hold"
wave epsilon = epsilon
	
variable nPeaks,i,numPara,EstimatedPeakArea
variable epsilonVal=1e-5
numPara = 8	
			nPeaks =WCoef_length/numPara
			redimension /n=(WCoef_length+numPara)  locRefW_coef
			redimension /n=(WCoef_length+numPara) Min_Limit
			redimension /n=(WCoef_length+numPara) Max_Limit
			redimension /n=(WCoef_length+numPara) CoefLegendRef        // This approach is much easier, than the other, using Temporary waves
			redimension /n=(WCoef_length+numPara) holdRef
			redimension /n=(WCoef_length+numPara) epsilon
			redimension /n=(2*(WCoef_length+numPara)) locRefConstraints
			
			if (WCoef_length == 0 )                                //the first peak gets these values
				EstimatedPeakArea = EstimatePeakArea( heightA , Width_Start, 5 * GLratio_Min, 5 * Asym_Min , 5 * Asym_Shift_Min)	
				locRefW_coef[0] = EstimatedPeakArea 		//now append the current cursor settings
				locRefW_coef[1]  = posA
				locRefW_coef[2] = Width_Start	   //standard guess
				locRefW_coef[3] = GLratio_Start
				locRefW_coef[4] =  Asym_Start
				locRefW_coef[5] = Asym_Shift_Start
				locRefW_coef[6] = 2
				locRefW_coef[7] = 0.5
			else                                                              // all further peaks are related to those values
				locRefW_coef[WCoef_length+1]  = posA
				if ( indepCheck == 1 )
					EstimatedPeakArea = EstimatePeakArea ( heightA , locRefW_coef[2] , locRefW_coef[3] , locRefW_coef[4] , locRefW_coef[5] )						
					locRefW_coef[WCoef_length] = EstimatedPeakArea  
					locRefW_coef[WCoef_length+2] = locRefW_Coef[2]                 //by default, everything is linked to peak 1	
					locRefW_coef[WCoef_length+3] = locRefW_Coef[3]
					locRefW_coef[WCoef_length+4] = locRefW_Coef[4]
					locRefW_coef[WCoef_length+5] = locRefW_Coef[5]
					locRefW_coef[WCoef_length+6] = locRefW_Coef[6]
					locRefW_coef[WCoef_length+7] = locRefW_Coef[7]
				else
					EstimatedPeakArea = EstimatePeakArea( heightA , Width_Start, 5 * GLratio_Min, 5 * Asym_Min , 5 * Asym_Shift_Min)						
					locRefW_coef[WCoef_length] = EstimatedPeakArea
					locRefW_coef[WCoef_length+2] = Width_Start         
					locRefW_coef[WCoef_length+3] = GLratio_Start
					locRefW_coef[WCoef_length+4] = Asym_Start
					locRefW_coef[WCoef_length+5] = Asym_Shift_Start
					locRefW_coef[WCoef_length+6] = 2
					locRefW_coef[WCoef_length+7] =0.5
				endif
			endif
			
			if (WCoef_length == 0)
				Min_Limit[0] = "K0 >  " + num2str(min(1e-5,0.2 * EstimatedPeakArea ))  //the first one should not become zero, its the reference peak
				Max_Limit[0] ="K0 <  " + num2str(max(10,10*heightA ) )
				Min_Limit[1] = "K1 >  " + num2str(right)
				Max_Limit[1] ="K1 <  " + num2str(left )
				Min_Limit[2] = "K2 >  " + num2str(Width_Min)
				Max_Limit[2] ="K2 <  " + num2str(Width_Max )
				Min_Limit[3] = "K3 >  " + num2str(GLratio_Min)
				Max_Limit[3] = "K3 <  " + num2str(GLratio_Max)
				Min_Limit[4] = "K4 >  " + num2str(Asym_Min)
				Max_Limit[4] = "K4 <  " + num2str(Asym_Max)
				Min_Limit[5] = "K5 >  " + num2str(Asym_Shift_Min)
				Max_Limit[5] = "K5 <  " + num2str(Asym_Shift_Max)
				Min_Limit[6] = "K6 >  " + num2str(Spin_Split_Min)
				Max_Limit[6] = "K6 <  " + num2str(Spin_Split_Max)
				Min_Limit[7] = "K7 >  " + num2str(Spin_Ratio_Min)
				Max_Limit[7] = "K7 <  " + num2str(Spin_Ratio_Max)

			else 
				Min_Limit[WCoef_length] = "K"  + num2istr(WCoef_length) + " >  "+ num2str(min(1e-5,0.2 * EstimatedPeakArea )) 
				Max_Limit[WCoef_length] ="K"  + num2istr(WCoef_length) + " <  " + num2str(max(10,10*heightA) )
				Min_Limit[WCoef_length+1] =  "K" + num2istr(WCoef_length+1) + " >  "+ num2str(right)
				Max_Limit[WCoef_length+1] ="K" + num2istr(WCoef_length+1) + " <  " +num2str(left)
				
				if ( indepCheck == 1 )
					Min_Limit[WCoef_length+2] = "K" + num2istr(WCoef_length+2) + " >  K2 - "+ num2str(interval)
					Max_Limit[WCoef_length+2] ="K" + num2istr(WCoef_length+2) + " <  K2 + "+ num2str(interval)
					Min_Limit[WCoef_length+3] = "K" + num2istr(WCoef_length+3) + " >  K3 - "+ num2str(interval)
					Max_Limit[WCoef_length+3] ="K" + num2istr(WCoef_length+3) + " <  K3 + "+ num2str(interval)
					Min_Limit[WCoef_length+4] = "K" + num2istr(WCoef_length+4) + " >  K4 - "+ num2str(interval)
					Max_Limit[WCoef_length+4] ="K" + num2istr(WCoef_length+4) + " <  K4 + "+ num2str(interval)
					Min_Limit[WCoef_length+5] = "K" + num2istr(WCoef_length+5) + " >  K5 - "+ num2str(interval)
					Max_Limit[WCoef_length+5] ="K" + num2istr(WCoef_length+5) + " <  K5 + "+ num2str(interval)
					Min_Limit[WCoef_length+6] = "K" + num2istr(WCoef_length+6) + " >  K6 - "+ num2str(interval)
					Max_Limit[WCoef_length+6] ="K" + num2istr(WCoef_length+6) + " <  K6 + "+ num2str(interval)
					Min_Limit[WCoef_length+7] = "K" + num2istr(WCoef_length+7) + " >  K7 - "+ num2str(interval)
					Max_Limit[WCoef_length+7] ="K" + num2istr(WCoef_length+7) + " <  K7 + "+ num2str(interval)
				else
					Min_Limit[WCoef_length+2] = "K" + num2istr(WCoef_length+2) + " >  " + num2str(Width_Min)
					Max_Limit[WCoef_length+2] = "K" + num2istr(WCoef_length+2) + " <  " + num2str(Width_Max)
					Min_Limit[WCoef_length+3] = "K" + num2istr(WCoef_length+3) + " >  " + num2str(GLratio_Min)
					Max_Limit[WCoef_length+3] = "K" + num2istr(WCoef_length+3) + " <  " + num2str(GLratio_Max)
					Min_Limit[WCoef_length+4] = "K" + num2istr(WCoef_length+4) + " >  " + num2str(Asym_Min)
					Max_Limit[WCoef_length+4] = "K" + num2istr(WCoef_length+4) + " <  " + num2str(Asym_Max)
					Min_Limit[WCoef_length+5] = "K" + num2istr(WCoef_length+5) + " >  " + num2str(Asym_Shift_Min)
					Max_Limit[WCoef_length+5] = "K" + num2istr(WCoef_length+5) + " <  " + num2str(Asym_Shift_Max)	
					Min_Limit[WCoef_length+6] = "K" + num2istr(WCoef_length+6) + " >  " + num2str(Spin_Split_Min)
					Max_Limit[WCoef_length+6] = "K" + num2istr(WCoef_length+6) + " <  " + num2str(Spin_Split_Max)
					Min_Limit[WCoef_length+7] = "K" + num2istr(WCoef_length+7) + " >  " + num2str(Spin_Ratio_Min)
					Max_Limit[WCoef_length+7] = "K" + num2istr(WCoef_length+7) + " <  " + num2str(Spin_Ratio_Max)
				endif			
			endif
			
			for (i=0; i <  numpnts(Min_Limit); i+=1)
				locRefConstraints[2*i] = Min_Limit[i]
				locRefConstraints[2*i+1] = Max_Limit[i]
			endfor

			epsilon[Wcoef_length] = epsilonVal
			epsilon[Wcoef_length+1] = epsilonVal
			epsilon[Wcoef_length+2] =epsilonVal
			epsilon[Wcoef_length+3] = epsilonVal
			epsilon[Wcoef_length+4] =epsilonVal
			epsilon[Wcoef_length+5] =epsilonVal
			epsilon[Wcoef_length+6] =epsilonVal
			epsilon[Wcoef_length+7] = epsilonVal
			
			holdRef[Wcoef_length] = 0
			holdRef[Wcoef_length+1] = 0
			holdRef[Wcoef_length+2] = 0
			holdRef[Wcoef_length+3] = 0
			holdRef[Wcoef_length+4] = 1
			holdRef[Wcoef_length+5] = 1
			holdRef[Wcoef_length+6] = 1
			holdRef[Wcoef_length+7] = 1
			
			CoefLegendRef[WCoef_length] = " K" + num2istr(WCoef_length) + " : Area -------- Peak: #"  + num2istr(nPeaks+1)
			CoefLegendRef[WCoef_length+1] = " K" + num2istr(WCoef_length+1) + " : Position"
			CoefLegendRef[WCoef_length+2] = " K" + num2istr(WCoef_length+2) + " : Width"
			CoefLegendRef[WCoef_length+3] = " K" + num2istr(WCoef_length+3) + " : GL ratio"
			CoefLegendRef[WCoef_length+4] = " K" + num2istr(WCoef_length+4) + " : Asymmetry"
			CoefLegendRef[WCoef_length+5] = " K" + num2istr(WCoef_length+5) + " : Asym. Shift"
			CoefLegendRef[WCoef_length+6] = " K" + num2istr(WCoef_length+6) + " : Spin Orbit Split"
			CoefLegendRef[WCoef_length+7] = " K" + num2istr(WCoef_length+7) + " : Spin Orbit ratio"
					
end

///2    // //////////////////////////////// Display
// fill in the case for doublet
function PlotPVDoubletDisplay(peakType,RawYWave, RawXWave,coefWave)

	//SVAR peakType = root:S_PeakType
	//SVAR RawYWave = root:S_nameWorkWave
	//SVAR coefWave = root:S_CoefWave
	//SVAR RawXWave = root:S_XRawDataCursorPanel
	
	string peakType
	string RawYWave
	string RawXWave
	string coefWave
	string TagName    // the Tag in the result window
	string PeakTag     // text in this tag
	string PkName, parentDataFolder //, cleanUpString=""		
	string BGName //background
	string PeakSumName
	NVAR FitMin = STFitMin
	NVAR FitMax =STFitMax
	
	wave cwave = $coefWave
	wave raw = $RawYWave
//	wave xraw = $RawXWave
	variable LenCoefWave = DimSize(cwave,0)
	
	//create some waves, to display the peak
	variable nPeaks = 0
	variable numCoef
	variable i,index
	variable xmin, xmax, step
	variable TagPosition   //the position of the tag in the result window
	variable totalPeakSumArea, partialPeakSumArea
	
	
	duplicate /o /R=(FitMin,FitMax) raw WorkingDuplicate   
	strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
		case "":                                             //if empty
			                     
			break
		case "_calculated_":
			 
			break
		default:                                                 // if not empty, x-axis wave neccessary
			//read in the start x-value and the step size from the x-axis wave
			wave xraw = $RawXWave
			wave xraw = $RawXWave
			xmax = max(xraw[0],xraw[numpnts(xraw)-1] )
			xmin = min(xraw[0],xraw[numpnts(xraw)-1] )
			step = (xmax - xmin ) / DimSize(xraw,0)
			// now change the scaling of the y-wave duplicate, so it gets equivalent to a data-wave imported from an igor-text file
			SetScale /I x, xmin, xmax, WorkingDuplicate
			
			
			//xmax = rightx(xraw)
			//step = (xraw[0] - xraw[xmax] ) / DimSize(xraw,0)
			//// now change the scaling of the y-wave duplicate, so it gets equivalent to a data-wave imported from an igor-text file
			//SetScale /P x, xraw[xmax], step, WorkingDuplicate
			break
	endswitch
	
	parentDataFolder = GetDataFolder(1)
	
	
	//now make tabular rasa in the case of background functions
	string ListOfCurves = TraceNameList("CursorPanel#guiCursorDisplayFit",";",1)
	variable numberCurves = ItemsInList(ListOfCurves)
	//remove only the very last curve, if there are e.g. 3 curves on the graph it has the index #2
	
	// If a wave is given which needs an external x-axis (from an ASCII-file) create a duplicate which receives a proper x-scaling later on
	// the original wave will not be changed
	              
	NewDataFolder /O /S FitTemp

	numCoef = 8   //Voigt Doublet
			nPeaks = LenCoefWave/numCoef
			for (i =0; i<nPeaks;i+=1)
				index = numCoef*i
				PkName = "peak_" + num2istr(i+1)   //make a propper name
			 	//create a wave with this name and the correct  scaling and number of datapoints -> copy the fitted wave
				duplicate /o WorkingDuplicate $PkName			
				wave tempDisplay = $PkName        //This needs some explanation, see commentary at the end of the file                                        
			 
				 //overwrite the original values in the wave with the values of a single peak
				tempDisplay = CalcVoigtDoublet(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5], cwave[index+6],cwave[index+7], x)
			
				RemoveFromGraph /W=CursorPanel#guiCursorDisplayFit /Z  $PkName#0
				AppendToGraph /w= CursorPanel#guiCursorDisplayFit tempDisplay                           //now plot it
				
				WaveStats /Q tempDisplay
				tagName = PkName+num2istr(i)
				PeakTag = num2istr(i+1)
				TagPosition = V_maxloc
				
				Tag /w= CursorPanel#guiCursorDisplayFit /C /N= $tagName  /F=0 /L=1  /Y =2.0  $PkName, TagPosition ,PeakTag
				ModifyGraph /w= CursorPanel#guiCursorDisplayFit rgb($PkName)=(0,0,0)       // and color-code it	
			endfor		

 					
	WaveStats /Q WorkingDuplicate
//	SetAxis /w = CursorPanel#guiCursorDisplayFit left -0.1*V_max, 1.1*V_max
	ModifyGraph /w= CursorPanel#guiCursorDisplayFit zero(left)=2 
	SetAxis/A/R /w = CursorPanel#guiCursorDisplayFit bottom
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit mirror(left)=2
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit mirror(bottom)=2
	Label  /w = CursorPanel#guiCursorDisplayFit Bottom "\\f01 binding energy (eV)"
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit minor(bottom)=1,sep(bottom)=2
	SetDataFolder parentDataFolder 
	killwaves /Z WorkingDuplicate
end


  

function DrawAndEvaluatePVDoublet(dataWave,fitWave,peakType,RawXWave,newFolder)

string dataWave
string fitWave
string peakType
string RawXWave
variable newFolder               //if this value is different from 1, no folder for the results will be created

wave cwave = W_coef
wave origWave = $dataWave
wave fitted = $fitWave
wave epsilon = epsilon
wave hold = hold
wave InitializeCoef = InitializeCoef
wave Min_Limit = Min_Limit
wave Max_Limit = Max_Limit
wave T_Constraints = T_Constraints
wave  CoefLegend = CoefLegend

if ( strlen(fitWave) >= 30)	
	doalert 0, "The name of the fit-wave is too long! Please shorten the names."
	return -1
endif


//define further local variables
variable LenCoefWave = DimSize(cwave,0)	
variable nPeaks
variable index
variable i =0                               //general counting variable
variable numCoef                       //variable to keep the number of coefficients of the selected peak type
							  // numCoef = 3   for Gauss Singlet     and numCoef =5 for VoigtGLS
variable pointLength, totalArea, partialArea
variable peakMax 
variable TagPosition
variable AnalyticalArea
variable EffectiveFWHM
variable GeneralAsymmetry        //  = 1 - (fwhm_right)/(fwhm_left)

string PkName                          //string to keep the name of a single peak wave
string foldername                       //string to keep the name of the datafolder, which is created later on for the single peak waves
string tempFoldername               //help-string to avoid naming conflicts
string parentDataFolder
string TagName
string PeakTag
string LastGraphName
string NotebookName = "Report"     //this is the initial notebook name, it is changed afterwards
string tempNotebookName
string tempString                          // for a formated output to the notebook
string BGName
//The following switch construct is necessary in order to plot waveform data (usually from igor-text files , *.itx) as well as
//raw spectra which need an extra x-axis (such data come usually from an x-y ASCII file)

strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
	case "":                                             //if empty
		display /K=1 origWave                 //the raw-data wave is in true waveform and has an intrinsic scale                     
		break
	case "_calculated_":
		display /K=1 origWave
		break
	default:
		wave xraw = $RawXWave                                                 // if not empty
		display /K=1 origWave vs xraw        // the raw-data wave has no intrinsic scale and needs a propper x-axis 
		break
endswitch
ModifyGraph mode($dataWave)=3,msize($dataWave)=1.3, marker($dataWave)=8
ModifyGraph mrkThick($dataWave)=0.7
ModifyGraph rgb($dataWave)=(0,0,0)           //color-code it
    
LastGraphName = WinList("*", "", "WIN:")    //get the name of the graph

//check if this Notebook already exists
V_Flag = 0
DoWindow $NotebookName   
// if yes, construct a new name
if (V_Flag)
	i = 1
	do 
		tempNoteBookName = NotebookName + num2istr(i)
		DoWindow $tempNotebookName
		i += 1
	while (V_Flag)
	NotebookName = tempNotebookName 
endif
//if not, just proceed

NewNotebook /F=1 /K=1 /N=$NotebookName      //make a new notebook to hold the fit report
Notebook $NoteBookName ,fsize=8
//Notebook $NoteBookName ,text="\r\r \t\t --- if necessary, insert plot by copy and paste ----    "
Notebook $NoteBookName ,text="\r\r\rPeak Shape:    "+ peakType

//prepare a new datafolder for the fitting results, in particular the single peaks
parentDataFolder = GetDataFolder(1)    //get the name of the current data folder

if (newFolder == 1)
	NewDataFolder /O /S subfolder
	//now, this folder is the actual data folder, all writing is done here and not in root
endif

duplicate /o fitted tempFitWave
wave locRefTempFit = tempFitWave
locRefTempFit = 0

strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
	case "":                                             //if empty
		 sprintf  tempString, "\r\rIntegral area of the raw spectrum (wave area): %20.2f " ,  area(origWave)           
		break
	case "_calculated_":
		sprintf tempString, "\r\rIntegral area of the raw spectrum (wave area): %20.2f " ,  area(origWave)
		break
	default:                                                 // if not empty
		sprintf tempString, "\r\rIntegral area of the raw spectrum (wave area): %20.2f " ,  areaXY(xraw,origWave)
		break
endswitch
Notebook $NoteBookName ,text=tempString

//take the fit-result and analyze the maximum, get the maximum signal, so a significance threshold can be calculated
//WaveStats /Q fitted
//peakMax = V_max

//now decompose the fit into single peaks --- if a further fit function is added, a further "case" has to be attached
		numCoef = 8
		nPeaks = LenCoefWave/numCoef                                             //get the number of  peaks from the output wave of the fit
		
		if ( mod(LenCoefWave,numCoef) != 0 )
			DoAlert 0, "Missmatch, probably wrong peak type selected or wrong coefficient file, retry "
			SetDataFolder parentDataFolder 
			KillDataFolder /Z subfolder
			print " ******* Peak type mismatch - check your fit and peak type ******"
			return 1
		endif 
		
		Notebook $NoteBookName ,text="\rNumber of peaks:    " + num2istr(nPeaks)
		//Notebook $NoteBookName ,text="\r\rPeak No  ||  Area  |  Position  |  FWHM  |  G-L ratio  |  Asymmetry  |  Asym. shift  |  Doublet ratio  |  Doublet splitting \r " 
		
		for (i =0; i<nPeaks;i+=1)
			
			index = numCoef*i

			//make a proper name for the single peak wave
			PkName = "p" + num2istr(i+1)+"_" + dataWave
			 //create a wave with this name and the correct scaling 
			 //and number of datapoints -> copy the fit wave and give the copy the name PkName    
			duplicate /o fitted $PkName                                                  
												  
			wave W = $PkName                                                 
		
 			//overwrite the original values in the wave with the values of a single peak   
			//The function CalcSingleVoigtGLS is defined in the file FitAssistFunc.ipf 
			W = CalcVoigtDoublet(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5], cwave[index+6],cwave[index+7],x)
			locRefTempFit += W
			AppendToGraph $PkName                                                        //now plot it
			//append the peak-tags to the graph, let the arrow point to a maximum
			WaveStats /Q W				 // get the location of the maximum
			TagName = "tag"+num2istr(i)     //each tag has to have a name
			PeakTag = num2istr(i+1)           // The tag displays the peak index
			TagPosition = V_maxloc           // and is located at the maximum
			Tag  /C /N= $TagName  /F=0 /L=1  /Y =2.0  $PkName, TagPosition ,PeakTag  //Now put the tag there
			
			ModifyGraph rgb($PkName)=(30464,30464,30464)              // and color-code the graph
			
			//The FWHM is for both peaks in the doublet the same, so the Singlet function is re-usable
			EffectiveFWHM = CalcFWHMSingleVoigtGLS(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5])
			GeneralAsymmetry = CalcGeneralAsymmetry(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5]) 
			
			AnalyticalArea =  IntegrateVoigtDoublet(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5], cwave[index+6], cwave[index+7])

			//WaveStats /Q $PkName
			//peakMax = V_max     //in this way, each peak has a numerically different threshold
			sprintf  tempString, "\r\r %1g	Area                                   |  Position |  FWHM   | GL-ratio  |   Asym.   | Asym. S. | Db. split  |  Db. ratio |\r",(i+1)
			Notebook $NoteBookName ,text=tempString
			sprintf  tempString, "---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------" 
			Notebook $NoteBookName ,text=tempString
			sprintf  tempString, "\r       %16.4g   'Fit-coefficient'  |  %8.2f  |  %8.2f  |  %8.2f  |  %8.2f  |  %8.2f    |  %8.2f  |  %8.2f  |\r" ,  cwave[index], cwave[index+1] ,cwave[index+2] ,cwave[index+3] , cwave[index+4],cwave[index+5], cwave[index+6], cwave[index+7]
			Notebook $NoteBookName ,text=tempString
			sprintf  tempString, "   %20.4g   'Fit wave area' \t Effective maximum position %8.2f \r",  area(W), V_maxloc // "-> In case of asymmetry, this value does not represent an area any more"
			Notebook $NoteBookName ,text=tempString
			sprintf tempString, "   %20.4g   'Analytical area' \t Effective Asymmetry = 1 - (fwhm_right)/(fwhm_left) %8.2f \r" ,	AnalyticalArea, GeneralAsymmetry
			Notebook $NoteBookName ,text=tempString
			sprintf  tempString, "   %20.4g  'Doublet area' \t Effective FWHM %8.2f\r", cwave[index]*(1+cwave[index+7]),EffectiveFWHM     // "-> In case of asymmetry, this value does not represent an area any more"
			Notebook $NoteBookName ,text=tempString
			//sprintf tempString, "   %20.4g   'Analytical area'\r\r" ,AnalyticalArea
			 //Notebook $NoteBookName ,text=tempString
						
		endfor
		
		sprintf tempString, "\rThe 'area' fit-coefficient corresponds only to the area of one component of the doublet. The complete doublet area can be calculated according to: Doublet Area  = Area Fit-Coefficient ( 1 + Doublet Ratio)\r"
		Notebook $NoteBookName ,text=tempString
		sprintf tempString, "\r'Doublet area':    Calculated from the fit coefficients - not significant for asymmetric peaks!"
		Notebook $NoteBookName ,text=tempString
		sprintf  tempString,"\r'Fit wave area':    Peak area within the measured energy range"
		Notebook $NoteBookName ,text=tempString																			    
		sprintf tempString,"\r'Analytical area': Obtained by numerically integrating the peak from 0 to (position + 1000eV)\r                        Significant also for asymmetric peaks - approximates the peak area from -INF to + INF\r"
		Notebook $NoteBookName ,text=tempString
		sprintf tempString,"\r\rPlease note that for asymmetric peaks the fit coefficients do not describe the physically relevant quantities any more. \rPlease refer to the respective 'effective' or 'analytical' values.\r"      
		Notebook $NoteBookName ,text=tempString
	

killwaves /z  fitted     //this applies to the original fit-wave of Igor, since it is a reference to the wave root:Igor-FitWave, the original wave is possibly wrong
duplicate /o locRefTempFit, $fitWave               //but we are still in the subfolder, 
killwaves /z locRefTempFit
wave fitted = $fitWave

//go back to the parent data folder
if (newFolder ==1)
	SetDataFolder parentDataFolder 

	//create a copy of the coefficient wave in the subfolder, so the waves 
	//and the complete fitting results are within that folder
	duplicate /o :$dataWave, :subfolder:$dataWave
	if (WaveExists($RawXWave))
		duplicate /o :$RawXWave, :subfolder:$RawXWave
	endif

	//duplicate  :$fitWave, :subfolder:$fitWave
	killwaves /z :$fitWave            //probably fails, if the fit wave is displayed in the main panel as well
	AppendToGraph :subfolder:$fitWave                                 //draw the complete fit
	ModifyGraph rgb($fitWave) = (0,0,0)       //color-code it
	//Remove the original wave, which is located in the parent directory and replace it by the copy in the subfolder
	RemoveFromGraph $"#0" 
	strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
		case "":                                             //if empty
			AppendToGraph :subfolder:$dataWave                 //the raw-data wave is in true waveform and has an intrinsic scale                     
			break
		case "_calculated_":
			AppendToGraph  :subfolder:$dataWave
			break
		default:                                                 // if not empty
			AppendToGraph :subfolder:$dataWave vs :subfolder:$RawXWave        // the raw-data wave has no intrinsic scale and needs a propper x-axis 
		break
	endswitch

	
	duplicate :T_Constraints, :subfolder:T_Constraints
	duplicate :Min_Limit, :subfolder:Min_Limit
	duplicate :Max_Limit, :subfolder:Max_Limit
	//if (WaveExists(LegendWave))
	//	duplicate :LegendWave, :subfolder:LegendWave
	//endif

	if (WaveExists(InitializeCoef))
		duplicate InitializeCoef, :subfolder:InitializeCoef
	endif
 	if (exists("epsilon"))
 		duplicate epsilon :subfolder:epsilon
 	endif
 	duplicate hold, :subfolder:hold
 	duplicate CoefLegend, :subfolder:CoefLegend
	duplicate cwave, :subfolder:W_Coef                 //now create that copy, leave the original 
														//coefficient wave where it is, maybe the 
														//user wants to try another fit

//Now rename the subfolder accordingly
//If there are any datafolders with peak-waves from previous runs are present, do
//not overwrite them, but create a foldername with a running index
// if a folder named "test" was present, name the new folder "test1", the next one "test2" and so on

	foldername = "VD"+num2istr(nPeaks)+ "_" + dataWave
	
	//foldername ="Fit_"+ dataWave
	i=1   //index variable for the data folder	
	tempFoldername = foldername            //used also for the notebook
	if (DataFolderExists(foldername))    //is there already a folder with this name
	//if yes:
	 //Append an index number to the current foldername and check if a folder with this name
	 //is already present. If so, increase the index number by one, append it to the basic foldername
	 //and check again. Repeat this procedure until a "free" name is found
		do
			tempFoldername = foldername + num2istr(i)
			i+=1
		while(DataFolderExists(tempFoldername))
		
		if ( strlen(tempFoldername) >= 30)	
			doalert 0, "The output folder name is too long! Please shorten the names. The output folder of the current run is named 'subfolder'."
		else
			RenameDataFolder subfolder, $tempFoldername                   //now rename the peak-folder accordingly
			Notebook  $NoteBookName, text="\r------------------------------------------------------------------------------"
			//remove illegal characters from the string
			tempFoldername = stringClean(tempFoldername)
			DoWindow /C $tempFoldername
			DoWindow /F $LastGraphName
			DoWindow /C $tempFoldername + "_graph"
			//DoWindow /C /W=$LastGraphName $tempFoldername + "_graph"   //valid in Igor 6.x
		endif
		//TextBox/N=text0/A=LT tempFoldername        //prints into the graph
	else 
		//if no:
		RenameDataFolder subfolder, $foldername 
		//TagWindow(foldername)
		Notebook  $NoteBookName, text="\r------------------------------------------------------------------------------"
		//if ( strsearch(foldername,".",0) != -1 )      // strsearch returns -1 if  the string contains no "." 
		//	foldername = ReplaceString(".", foldername,"_")
		//endif
		foldername = stringClean(foldername)
		tempFoldername = stringClean(tempFoldername)
		DoWindow /C $foldername
		DoWindow /F $LastGraphName
		DoWindow /C $tempFoldername + "_graph"
		//DoWindow /C /W=$LastGraphName $tempFoldername + "_graph"
		//TextBox/N=text0/A=LT foldername               //prints into the graph
	endif
	//everything up to now was done in a subfolder
else          //varialbe newFolder different from 1  create no new folder
	AppendToGraph $fitWave
	ModifyGraph rgb($fitWave) = (0,0,0)
	SetAxis/A/R bottom
	//find a name for the graph and the notebook, that reflects the location of the corresponding data
	String location = GetDataFolder (0)
	strswitch(location)
		case "root":
			tempFoldername = "fit" + dataWave
			break
		default:
			tempFoldername = location
			break
	endswitch

	Notebook  $NoteBookName, text="\r------------------------------------------------------------------------------"
	
	//if ( strsearch(tempFoldername,".",0) != -1 )      // strsearch returns -1 if  the string contains no "." 
	//	tempFoldername = ReplaceString(".", tempFoldername,"_")
	//endif
	tempFoldername = stringClean(tempFoldername)
	V_Flag = 0
	DoWindow $tempFoldername    //check if it is there, if yes V_Flag is set to 1
	if (V_Flag)
		do
			V_Flag = 0   // it should be possible to use the cancel button
			Prompt tempFoldername, "There is already a report on this folder, please enter a new for the new report."
			DoPrompt /Help="Please use only letters, numbers and an underscore!" "Please enter a new name", tempFoldername
			if (V_Flag)  //user pressed cancel
				DoWindow /K Report //kill the notebook
				DoWindow /K $LastGraphName
				
				killvariables  /Z V_chisq, V_numNaNs, V_numINFs, V_npnts, V_nterms,V_nheld,V_startRow, V_Rab, V_Pr
				killvariables  /Z V_endRow, V_startCol, V_endCol, V_startLayer, V_endLayer, V_startChunk, V_endChunk, V_siga, V_sigb,V_q,VPr
				return -1
			endif
			tempFoldername = stringClean(tempFoldername)
			DoWindow $tempFoldername    //check again and
		while (V_Flag)                                 //repeat until
		DoWindow /C $tempFoldername
		DoWindow /F $LastGraphName
		DoWindow /C $tempFoldername + "_graph"
		//DoWindow /C /W=$LastGraphName $tempFoldername + "_graph"
	else
		DoWindow /C $tempFoldername
		DoWindow /F $LastGraphName
		DoWindow /C $tempFoldername + "_graph"
		//DoWindow /C /W=$LastGraphName $tempFoldername + "_graph" 
	endif
	//TextBox/N=text0/A=LT "fit"+dataWave
endif
 
//make the graph look good
ModifyGraph mode($dataWave)=3 ,msize($dataWave)=1.3 // ,marker($dataWave)=8, opaque($dataWave)=1
ModifyGraph opaque=1,marker($dataWave)=19
ModifyGraph rgb($dataWave)=(60928,60928,60928)
ModifyGraph useMrkStrokeRGB($dataWave)=1
ModifyGraph mrkStrokeRGB($dataWave)=(0,0,0)
//ModifyGraph mrkThick($dataWave)=0.7
//ModifyGraph rgb($dataWave)=(0,0,0)           //color-code it
ModifyGraph mirror=2,minor(bottom)=1
Label left "\\f01 intensity (counts)"
Label bottom "\\f01  binding energy (eV)"	
ModifyGraph width=255.118,height=157.465, standoff = 0, gfSize=11

//The following command works easily, but then the resulting graph is not displayed properly in the notebook
//SetAxis/A/R bottom
//instead do it like this:
variable left,right

strswitch(RawXWave)
	 	case "":
	 		 left = max(leftx($dataWave),rightx($dataWave))
	 		 right = min(leftx($dataWave),rightx($dataWave))
	 	break
	 	case "_calculated_":
	 		 left = max(leftx($dataWave),rightx($dataWave))
	 		 right = min(leftx($dataWave),rightx($dataWave))
	 	break
	 	default:
	 		waveStats /Q $RawXWave
	 		left = V_max
	 		right = V_min
	 	break
 endswitch
 SetAxis bottom left,right
//okay this is not perfectly elegant ... but

WaveStats /Q $dataWave
SetAxis left V_min-0.05*(V_max-V_min), 1.02*V_max
LastGraphName = WinList("*", "", "WIN:")



Notebook  $tempFoldername, selection = {startOfFile,startOfFile}
tempString = "Fitting results for: " + tempFoldername + "\r"
Notebook  $tempFoldername, text=tempString
Notebook  $tempFoldername, selection = {startOfPrevParagraph,endOfPrevParagraph}, fsize = 12, fstyle = 0

Notebook  $tempFoldername, selection = {startOfNextParagraph,startOfNextParagraph}
Notebook  $tempFoldername, picture={$LastGraphName, 0, 1} , text="\r" 




//Notebook  $tempFoldername, text="\r \r"  

//now clean up
killvariables  /Z V_chisq, V_numNaNs, V_numINFs, V_npnts, V_nterms,V_nheld,V_startRow, V_Rab, V_Pr
killvariables  /Z V_endRow, V_startCol, V_endCol, V_startLayer, V_endLayer, V_startChunk, V_endChunk, V_siga, V_sigb,V_q,VPr

end 


//auxiliary functions, used by all of the memebers of the Pseudo-Voigt family
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
				//Aux: Peak Analysis Functions
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

//this expects a wave to be duplicated ... strange ... to be changed
function CalcVoigtWaveManual(w0,w1,w2,w3,w4,w5,w6,w7,originalWave)
	//this order is identical to the order of the parameters in the fit function
	variable w0 //area
	variable w1 //position
	variable w2 //width
	variable w3 //GLRatio
	variable w4 //asymmetry
	variable w5 //asymmetry shift
	variable w6 //spin-orbit splitting
   	variable w7 //spin-orbit ratio
	//those values are obtained from the doublet fit
	wave originalWave
	
	string firstName = "outwave"
	string secondName = "outwave2"
	
	duplicate /o originalWave $firstName
	duplicate /o originalWave $secondName
	
	wave outWave = $firstName                                                  									  
	wave outWave2 = $secondName
	//CalcSingleVoigtGLS( area,position , width , GLratio , asymmetry , asymmetry shift )
	outWave = CalcSingleVoigtGLS( w0 / (1+w7) ,w1,w2,w3,w4,w5,x)
	outWave2 = CalcSingleVoigtGLS( w0*w7 / (1+w7) , w1 + w6 ,w2,w3,w4,w5,x)
	
end



function IntegrateSingleVoigtGLS(w0, w1, w2, w3, w4,w5)
	variable w0
	variable w1
	variable w2
	variable w3
	variable w4
	variable w5
	
	variable  result = 0
	
	//now create a set of global variables
	killvariables /z root:tempW0,root:tempW1,root:tempW2,root:tempW3,root:tempW4, root:tempW5
	variable /G root:tempW0 = w0
	variable /G root:tempW1 = w1
	variable /G root:tempW2 = w2
	variable /G root:tempW3 = w3
	variable /G root:tempW4 = w4
	variable /G root:tempW5 = w5
	string resultString
	
	result = integrate1D(VoigtGLSFunction,w1-IntegrationInterval*w2, w1+IntegrationInterval*w2,1)    //2:Gauss//Romberg integration is much faster //integration stopped at zero for stability
	resultString= num2str(result)
	
	
	if (numType(result)!=0)   //something went wrong during integration
		print "Igors integrate1D function had a problem! If the output makes no sense ( NaN displayed) you have possibly to modify the file IntegrationFunc.ipf."
		result = integrate1D(VoigtGLSFunction,max(0,w1-IntegrationInterval*w2), w1+IntegrationInterval*w2,1) 
		//result = integrate1D(VoigtGLSFunction,0, w1+2000,1)         //try this, if the integration does continuosly give NaN, also try to vary the integration method, see the help on integrate1D()
		
	endif
	//clean up afterwards
	killvariables /Z root:tempW0, root:tempW1, root:tempW2, root:tempW3, root:tempW4, root:tempW5
	return result
end

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// Calculate the effective FWHM and general asymmetry

function CalcFWHMSingleVoigtGLS(Varea,pos,fwhm,glR,asym,asymShift)
	variable Varea
	variable pos
	variable fwhm
	variable glR
	variable asym
	variable asymShift
	
	variable i=0
	variable step = 1e-3
	//When global variables are used, the function for the integration can be re-used
	killvariables /z root:tempW0,root:tempW1,root:tempW2,root:tempW3,root:tempW4, root:tempW5
	variable /G root:tempW0 = Varea
	variable /G root:tempW1 = pos
	variable /G root:tempW2 = fwhm
	variable /G root:tempW3 = glR
	variable /G root:tempW4 = asym
	variable /G root:tempW5 = asymShift
	
	// Get suitable variables for determining the maximum height and position
	variable PreviousStepFunctionValue 
	variable CurrentStepFunctionValue 
	variable CurrentPos
	variable maxPos
	variable maxHeight
	
	//Get the real maximum and its location, do not use WaveStats /Q; it is applied to a discrete approximation of the function, not the real thing --> truncation errors
	i = 1
	do 
		CurrentStepFunctionValue = VoigtGLSFunction(pos - 0.5*fwhm + i*step)
		PreviousStepFunctionValue = VoigtGLSFunction(pos - 0.5*fwhm + (i-1)*step)
		CurrentPos = (pos - 0.5*fwhm + i*step)
		i+= 1
	while (CurrentStepFunctionValue > PreviousStepFunctionValue)
	maxPos = CurrentPos
	maxHeight = PreviousStepFunctionValue     // in the last step, CurrentStepFunctionValue was already smaller then PreviousStepFunctionValue
	i = 0	
	
	
	variable upperPos = maxPos  //initialize at the "center"
	variable lowerPos = maxPos  //initialize at the "center"
	do
		upperPos = maxPos + i*step
		i+=1
	while (VoigtGLSFunction(upperPos) > 0.5*maxHeight)
	//now make a loop to determine where the Intensity is halfway down	
	do
		lowerPos = maxPos - i*step
		i+=1
	while (VoigtGLSFunction(lowerPos) > 0.5*maxHeight)
	
	
	
	//clean up afterwards
	killvariables /Z root:tempW0, root:tempW1, root:tempW2, root:tempW3, root:tempW4, root:tempW5
	return abs(upperPos - lowerPos)
end


// General asymmetry =  1 - (fwhm_right)/(fwhm_left)
function CalcGeneralAsymmetry(Varea,pos,fwhm,glR,asym,asymShift)
	variable Varea
	variable pos
	variable fwhm
	variable glR
	variable asym
	variable asymShift
	
	variable i=0
	variable step = 1e-4
	//When global variables are used, the function for the integration can be re-used
	killvariables /z root:tempW0,root:tempW1,root:tempW2,root:tempW3,root:tempW4, root:tempW5
	variable /G root:tempW0 = Varea
	variable /G root:tempW1 = pos
	variable /G root:tempW2 = fwhm
	variable /G root:tempW3 = glR
	variable /G root:tempW4 = asym
	variable /G root:tempW5 = asymShift
	

	// Get suitable variables for determining the maximum height and position
	variable PreviousStepFunctionValue 
	variable CurrentStepFunctionValue 
	variable CurrentPos
	variable maxPos
	variable maxHeight
	
	//Get the real maximum and its location, do not use WaveStats /Q; it is applied to a discrete approximation of the function, not the real thing --> truncation errors
	i = 1
	do 
		CurrentStepFunctionValue = VoigtGLSFunction(pos - 0.5*fwhm + i*step)
		PreviousStepFunctionValue = VoigtGLSFunction(pos - 0.5*fwhm + (i-1)*step)
		CurrentPos = (pos - 0.5*fwhm + i*step)
		i+= 1
	while (CurrentStepFunctionValue > PreviousStepFunctionValue)
	maxPos = CurrentPos
	maxHeight = PreviousStepFunctionValue     // in the last step, CurrentStepFunctionValue was already smaller then PreviousStepFunctionValue
	i = 0	
	
	variable upperPos = maxPos  //initialize at the "center"
	variable lowerPos = maxPos  //initialize at the "center"
	
	do
		upperPos = maxPos + i*step
		i+=1
	while (VoigtGLSFunction(upperPos) > 0.5*maxHeight)
	//now make a loop to determine where the Intensity is halfway down
	i = 0	
	do
		lowerPos = maxPos - i*step
		i+=1
	while (VoigtGLSFunction(lowerPos) >0.5*maxHeight)
	
	
	
	//clean up afterwards
	killvariables /Z root:tempW0, root:tempW1, root:tempW2, root:tempW3, root:tempW4, root:tempW5
	variable Gasym =  (abs(lowerPos - maxPos)/abs(upperPos - maxPos))
	return 1 - Gasym
end


function VoigtGLSFunction(x)
	variable x
	NVAR w0 = root:tempW0
	NVAR w1 = root:tempW1
	NVAR w2 = root:tempW2
	NVAR w3 = root:tempW3
	NVAR w4 = root:tempW4
	NVAR w5 = root:tempW5
	variable result
	result = 0
	variable loc
	variable width
	
	//loc = x - w1
	//width = 2*w2/ ( 1+ exp( -w4 * (loc - w5 )))
	result  =  ( w3 *  w0 * 2 / pi ) *  ( 2*w2/ ( 1+ exp(- w4 *(x - w1-w5))) ) / ( ( 2*w2/ ( 1+ exp( - w4* (x - w1-w5))) )^2 + 4 * ( x - w1 )^2 )   
	result += ( 1 - w3 ) * ( w0 / ( gPf * ( 2*w2/ ( 1+ exp( - w4 * (x - w1-w5))) ) ) ) * exp ( -gAf  * ( ( x - w1 ) / ( 2*w2/ ( 1+ exp( - w4 * (x - w1-w5))) ) )^2 ) 
	
	//result  =  ( w3 *  0.636619772 ) *  ( width) / ( ( width )^2 + 4 * (loc )^2 )   
	//result += ( 1 - w3 ) * (1 / ( gPf * (  width ) ) ) * exp ( - gAf  * ( ( loc ) / ( width ) )^2 ) 
	//result *= w0
	
	return result
end
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// The following two functions enable a numerical integration of the voigt-function over an arbitrary range

function IntegrateVoigtDoublet(w0, w1, w2, w3, w4,w5,w6,w7)
	variable w0  //area
	variable w1  //position
	variable w2  //width
	variable w3  //GL ratio
	variable w4  //asymmetry
	variable w5  //asymmetry shift
	variable w6 //spin splitting
	variable w7 //spin ratio
	variable  result
	//now create a set of global variables
	killvariables /z root:tempW0,root:tempW1,root:tempW2,root:tempW3,root:tempW4, root:tempW5, root:tempW6, root:tempW7  //just in case ...make tabula rasa
	variable /G root:tempW0 = w0
	variable /G root:tempW1 = w1
	variable /G root:tempW2 = w2
	variable /G root:tempW3 = w3
	variable /G root:tempW4 = w4
	variable /G root:tempW5 = w5
	variable /G root:tempW6 = w6
	variable /G root:tempW7 = w7
		
	result = integrate1D(VoigtDoubletFunction,0, w1+w6+1000,1)    //Romberg integration is much faster

	//clean up afterwards
	killvariables /Z root:tempW0, root:tempW1, root:tempW2, root:tempW3, root:tempW4, root:tempW5, root:tempW6, root:tempW7
	return result
end

function VoigtDoubletFunction(x)
	variable x
	NVAR w0 = root:tempW0   //width
	NVAR w1 = root:tempW1   //position
	NVAR w2 = root:tempW2   //width
	NVAR w3 = root:tempW3   //GL ratio
	NVAR w4 = root:tempW4   //asymmetry
	NVAR w5 = root:tempW5   //asymmetry shift
	NVAR w6 = root:tempW6   //spin splitting
	NVAR w7 = root:tempW7   //spin ratio
	
	variable result
	result = 0
	//first peak
	result  =  ( w3 *  w0 * 2 / pi ) *  ( 2*w2/ ( 1+ exp(- w4 *(x - w1-w5))) ) / ( ( 2*w2/ ( 1+ exp( - w4* (x - w1-w5))) )^2 + 4 * ( x - w1 )^2 )   
	result += ( 1 - w3 ) * ( w0 / ( gPf * ( 2*w2/ ( 1+ exp( - w4 * (x - w1-w5))) ) ) ) * exp ( -gAf  * ( ( x - w1 ) / ( 2*w2/ ( 1+ exp( - w4 * (x - w1-w5))) ) )^2 ) 
	//second peak
	result +=  ( w3 * w7 * w0 * 2 / pi ) *  ( 2*w2/ ( 1+ exp( - w4 * (x - w6 - w1- w5))) ) / ( (2*w2/ ( 1+ exp( - w4 * (x - w6 - w1 - w5))) )^2 + 4 * ( x -  w6 - w1 )^2 )   
	result += ( 1 - w3 ) * ( w0 * w7 / ( gPf * (  2*w2/ ( 1+ exp( - w4 * (x - w6 - w1 -w5 ))) ) ) ) * exp ( -gAf  * ( ( x - w6 - w1 ) / ( 2*w2/ ( 1+ exp( - w4 * (x - w6 - w1-w5))) ) )^2 ) 
	return result
end




// the following two functions enable a numerical integration of the voigt-function over an arbitrary range
function IntegrateSingleVoigtGLS2(w0, w1, w2, w3, w4,peakMax)
	variable w0
	variable w1
	variable w2
	variable w3
	variable w4
	variable peakMax
	variable start, stop
	variable  result
	variable threshold
	variable tempValue
	//now create a set of global variables
	killvariables /z root:tempW0,root:tempW1,root:tempW2,root:tempW3,root:tempW4
	variable /G root:tempW0 = w0
	variable /G root:tempW1 = w1
	variable /G root:tempW2 = w2
	variable /G root:tempW3 = w3
	variable /G root:tempW4 = w4
	variable step = 0.1
	//Now implement an algorithm, that searches for the the range, where the amplitude is larger than 1% of the maximum value
	threshold = 0.01* peakMax
	start = w1
	stop = w1
	
	if (w4 > 2)
		step = 5
	elseif ( w4 > 1 && w4 <= 2 )
		step = 2
	elseif ( w4 >0.75 && w4 <=1)
		step = 1
	elseif ( w4 > 0.5 && w4 <= 0.75)
		step = 0.2
	endif
	
	do
		start = start - step
		tempValue = VoigtGLS2Function(start) 
	while ( tempValue > threshold)

	do
		stop = stop + step
		tempValue = VoigtGLS2Function(stop) 
	while ( tempValue > threshold)

	result = integrate1D(VoigtGLS2Function,start, stop,1)    //Romberg integration is much faster

	//clean up afterwards
	killvariables /Z root:tempW0, root:tempW1, root:tempW2, root:tempW3, root:tempW4
	return result
end



function VoigtGLS2Function(x)
	variable x
	NVAR w0 = root:tempW0
	NVAR w1 = root:tempW1
	NVAR w2 = root:tempW2
	NVAR w3 = root:tempW3
	NVAR w4 = root:tempW4
	variable result
	result = 0
	if (x >= w1 )
		result  =  ( w3 *  w0 * 2 / pi ) *  ( 2 * w4 * ( x - w1 ) + w2 ) / ( ( w2 + 2 * w4 * ( x - w1 ) )^2 + 4 * ( x - w1 )^2 )   
		result += ( 1 - w3 ) * ( w0 / ( gPf * (  2 * w4 * ( x - w1 ) + w2 ) ) ) * exp ( -gAf  * ( ( x - w1 ) / ( w2 + 2 * w4* ( x - w1 ) ) )^2 ) 
	else
		result  =  ( 2 * w3 * w0 * (w2*exp(w4*(x-w1) ) ) / pi )  / (  ( w2 *exp(w4*(x-w1) ))^2 + 4* ( x - w1 )^2 ) 
		result += ( 1 - w3 ) * ( w0 / ( gPf * ( w2*exp(w4*(x-w1 )) ) ) ) * exp ( - gAf * ( ( x - w1 ) / ( w2 *exp(w4*(x-w1 )) ) )^2 ) 
	endif
	return result
end
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// The following two functions enable a numerical integration of the voigt-function over an arbitrary range

function IntegrateVoigtDublet2(w0, w1, w2, w3, w4,w5,w6,peakMax)
	variable w0
	variable w1
	variable w2
	variable w3
	variable w4
	variable w5
	variable w6
	variable peakMax
	variable start, stop
	variable  result
	variable threshold
	variable tempValue
	//now create a set of global variables
	killvariables /z root:tempW0,root:tempW1,root:tempW2,root:tempW3,root:tempW4
	variable /G root:tempW0 = w0
	variable /G root:tempW1 = w1
	variable /G root:tempW2 = w2
	variable /G root:tempW3 = w3
	variable /G root:tempW4 = w4
	variable /G root:tempW5 = w5
	variable /G root:tempW6 = w6
	variable step = 0.1
	//Now implement an algorithm, that searches for the the range, where the amplitude is larger than 1% of the maximum value
	threshold = 0.01* peakMax
	start = w1
	stop = w1 + w6
	
	if (w4 > 2)
		step = 5
	elseif ( w4 > 1 && w4 <= 2 )
		step = 2
	elseif ( w4 >0.75 && w4 <=1)
		step = 1
	elseif ( w4 > 0.5 && w4 <= 0.75)
		step = 0.2
	endif
	
	do
		start = start - step
		tempValue = VoigtDublet2Function(start) 
	while ( tempValue > threshold)

	do
		stop = stop + step
		tempValue = VoigtDublet2Function(stop) 
	while ( tempValue > threshold)

	result = integrate1D(VoigtDublet2Function,start, stop,1)    //Romberg integration is much faster

	//clean up afterwards
	killvariables /Z root:tempW0, root:tempW1, root:tempW2, root:tempW3, root:tempW4, root:tempW5, root:tempW6
	return result
end

function VoigtDublet2Function(x)
	variable x
	NVAR w0 = root:tempW0
	NVAR w1 = root:tempW1
	NVAR w2 = root:tempW2
	NVAR w3 = root:tempW3
	NVAR w4 = root:tempW4
	NVAR w5 = root:tempW5
	NVAR w6 = root:tempW6
	
	variable result
	result = 0
	
	if (x >= (w1 + w6) )   // both peaks asymmetric
		//first peak
		result  =  ( w3 *  w0 * 2 / pi ) *  ( 2 * w4 * ( x - w1 ) + w2 ) / ( ( w2 + 2 * w4 * ( x - w1 ) )^2 + 4 * ( x - w1 )^2 )   
		result += ( 1 - w3 ) * ( w0 / ( gPf * (  2 * w4 * ( x - w1 ) + w2 ) ) ) * exp ( -gAf  * ( ( x - w1 ) / ( w2 + 2 * w4* ( x - w1 ) ) )^2 ) 
		//second peak
		result +=  ( w3 * w5 * w0 * 2 / pi ) *  ( 2 * w4 * ( x - w6 - w1 ) + w2 ) / ( ( w2 + 2 * w4 * ( x - w6 - w1 ) )^2 + 4 * ( x -  w6 - w1 )^2 )   
		result += ( 1 - w3 ) * ( w0 * w5 / ( gPf * (  2 * w4 * ( x - w6 - w1 ) + w2 ) ) ) * exp ( -gAf  * ( ( x - w6 - w1 ) / ( w2 + 2 * w4* ( x - w6 - w1 ) ) )^2 ) 
	
	elseif  ( x >=	w1 && x < (w1 + w6) )     //the peak at w1 still asymmetric, the other one symmetric
		//first peak at w1 still asymmetric
		result  =  ( w3 *  w0 * 2 / pi ) *  ( 2 * w4 * ( x - w1 ) + w2 ) / ( ( w2 + 2 * w4 * ( x - w1 ) )^2 + 4 * ( x - w1 )^2 )   
		result += ( 1 - w3 ) * ( w0 / ( gPf * (  2 * w4 * ( x - w1 ) + w2 ) ) ) * exp ( -gAf  * ( ( x - w1 ) / ( w2 + 2 * w4* ( x - w1 ) ) )^2 ) 
		//second peak at w1+w6 , now symmetric 
		result +=  ( 2 * w3 * w0 * w5* w2 / pi )  / (  w2^2 + 4* ( x - w6 - w1 )^2 ) 
		result += ( 1 - w3 ) * ( w0 *w5 / ( gPf * w2 ) ) * exp ( - gAf * ( ( x - w6 - w1 ) / w2 )^2 ) 
	else  //both symmetric
		result  =  ( 2 * w3 * w0 * w2 / pi )  / (  w2^2 + 4* ( x - w1 )^2 ) 
		result += ( 1 - w3 ) * ( w0 / ( gPf * w2 ) ) * exp ( - gAf * ( ( x - w1 ) / w2 )^2 ) 
		
		result +=  ( 2 * w3 * w0 * w5* w2 / pi )  / (  w2^2 + 4* ( x - w6 - w1 )^2 ) 
		result += ( 1 - w3 ) * ( w0 *w5 / ( gPf * w2 ) ) * exp ( - gAf * ( ( x - w6 - w1 ) / w2 )^2 ) 
	endif
	
	return result
end



function CalcSingleVoigtGLS(w0,w1,w2,w3,w4,w5,x)
	variable w0     //area
	variable w1    //position
	variable w2    //width
	variable w3    // GL-ratio
	variable w4    // Asymmetry
	variable w5 // asymmetry  shift
	variable x
	//variable k
	variable result
	result = 0

	//k = 2 * w2 / ( 1 + exp (-w4*(x - w1 - w5) ) )
	//result  =  w3* ( w0 * 2 / pi ) *  ( k )  /  (   ( k )^2 + 4 * ( x - w1 )^2    ) 
	 result  =  w3* ( w0 * 2 / pi ) *  ( 2 * w2 / ( 1 + exp (- w4* ( x - w1 - w5 )  )   )  )  /  (   ( 2 * w2 / ( 1 + exp (- w4*(x-w1-w5) ) ) )^2 + 4 * ( x - w1 )^2    )   
	 result += ( 1 - w3 ) * ( w0 / ( gPf * ( 2 * w2 / ( 1 + exp ( - w4*(x-w1-w5) ) ) ) ) ) * exp ( -gAf  * ( ( x - w1 ) / ( 2 * w2 / ( 1 + exp ( - w4*(x-w1-w5) ) ) ) )^2 ) 
	//linresult =  w3 * ( 2 * w0 / pi ) * ( (  w2 + w4*( x - w1 ) ) / ( ( w2 + w4*( x - w1 ) )^2 + 4 * ( x - w1 )^2 ) )  
	 //linresult += ( 1 - w3 ) * ( w0 / ( gPf * (  w2 + w4*( x - w1 ) ) ) ) * exp ( -gAf * ( ( x - w1) / ( w2 + w4*( x - w1 ) ) )^2 ) 
	//lin result += w5  //just as offset
	return result
end


function CalcVoigtDoublet(w0,w1,w2,w3,w4,w5,w6,w7,x)
	variable w0     //area
	variable w1    //position
	variable w2    //width
	variable w3    // GL-ratio
	variable w4    // Asymmetry
	variable w5    //asymmetry shift
	variable w6   //peak splitting
	variable w7 //peak ratio raio
	variable x
	variable result
	result = 0

	//first peak
	result +=  w3* (  (w0 * 2) / pi ) *  ( 2 * w2 / ( 1 + exp ( - w4*(x-w1 - w5 ) ) ) ) / ( ( 2 * w2 / ( 1 + exp ( - w4*(x-w1- w5) ) ) )^2 + 4 * ( x - w1 )^2 )   
	result += ( 1 - w3 ) * ( w0 / ( gPf * (  2 * w2 / ( 1 + exp ( - w4*(x-w1- w5) ) ) ) ) ) * exp ( -gAf  * ( ( x - w1 ) / ( 2 * w2 / ( 1 + exp ( - w4*(x-w1- w5) ) ) ) )^2 ) 
	//second peak
	result +=  ( w3 * w7 * w0 * 2 / pi ) *  ( 2 * w2 / ( 1 + exp ( - w4*(x - w6 - w1 - w5) ) ) ) / ( ( 2 * w2 / ( 1 + exp ( - w4*(x - w6 - w1 - w5) ) ) )^2 + 4 * ( x -  w6 - w1 )^2 )   
	result += ( 1 - w3 ) * ( w0 * w7 / ( gPf * (  2 * w2 / ( 1 + exp ( - w4*(x - w6 - w1 - w5 ) ) )) ) ) * exp ( -gAf  * ( ( x - w6 - w1 ) / (2 * w2 / ( 1 + exp ( - w4*(x - w6 - w1 -w5) ) ) ) )^2 ) 

	return result
end


// function used after pressing Add peak -> get an estimator for the peak area from the height of the cursor on the display

function EstimatePeakArea( height , width , glR , asym , asymShift )	
	variable height , width , glR , asym , asymShift
	variable valueAtMaximum
	variable NewWidth = 2 * width / ( 1 + exp ( asym * asymShift  ) )

	valueAtMaximum =  glR* (  2 / pi ) *  ( NewWidth ) / ( ( NewWidth )^2 )   
	valueAtMaximum += ( 1 - glR ) * (1 / ( gPf * ( NewWidth ) ) ) 
	//peak with an fit coefficient of exactly 1


	return  height / valueAtMaximum // by so many times the fit coefficient is too low

end
