#pragma rtGlobals=1		// Use modern global access method and strict wave access.
#pragma hide=1
//////////////////////////////////////////////////////////////////////////////////
//Licence
//////////////////////////////////////////////////////////////////////////////////
//
//	If you use the 'Fit Assistant' to analyze data, please read and cite the following paper:
//	Surace and Interface Analysis, (2014), 46, 505-511
//	This publication contains more details on the fit function used by XPST.
//
//	XPST is a front-end to the IgorPro fitting engine. Although every possible effort was made 
//	to prevent serious bugs and wrong results, the author can not give any warranty for XPST as 
//	large parts actually depend on the underlying IgorPro code. 
//	Author: Dr. Martin Schmid
//
//////////////////////////////////////////////////////////////////////////////////


function AddVoigtSKPeak(coefWave,RawYWave,indepCheck,heightA,posA,left,right,Wcoef_length)

string coefWave     //this won't be needed in the future ... for now, leave it here
string RawYWave
variable indepCheck
variable heightA
variable posA	
variable left
variable right
variable Wcoef_length

NVAR peakToLink = root:STFitAssVar:STPeakToLink
string parameterList = "a;p;w;g;as;at"

NVAR linkArea = root:STFitAssVar:AreaLink
NVAR linkPosition = root:STFitAssVar:PositionLink
NVAR linkWidth = root:STFitAssVar:WidthLink
NVAR linkGL = root:STFitAssVar:GLLink
NVAR linkAsym = root:STFitAssVar:AsymLink

NVAR areaLinkUpperFactor = root:STFitAssVar:AreaLinkFactorHigh
NVAR areaLinkLowerFactor = root:STFitAssVar:AreaLinkFactorLow
NVAR positionLinkOffsetMax = root:STFitAssVar:PositionLinkOffsetMax
NVAR positionLinkOffsetMin = root:STFitAssVar:PositionLinkOffsetMin

string name = "CursorPanel#guiCursorDisplay" 	
variable nPeaks,i,numPara,EstimatedPeakArea
variable epsilonVal=1e-5

wave /t source = STsetup  //everything is in the setup
wave sw = selSTSetup

wave /t  numerics = Numerics
wave selNumerics = selNumerics

variable length = DimSize(source,0)
variable NumLength = DimSize(numerics,0)

variable numpeaks = 0
//print numpeaks
variable index
variable newLength


WaveStats /Q $RawYWave
heightA = vcsr(A,name) - V_min						
EstimatedPeakArea = EstimatePeakArea( heightA , Width_Start, 5 * GLratio_Min, 5 * Asym_Min , 5 * Asym_Shift_Min)	


if (length == 0)
	Redimension /n=(length+11,-1) source
	Redimension /n=(length+11,-1) sw
	
	numpeaks = 0
	//set writing permissions and checkbox controls
	for ( i= 0; i<length + 11; i+=1)
		sw[i][0][0] = 0                //legende
		sw[i][1][0] = 0           //coef kuerzel
		sw[i][2][0] = 0           //endergebnis
		sw[i][3][0] = (0x02)   //anfangswerte
		sw[i][4][0] = (0x20)    //hold
		sw[i][5][0] = (0x02)    //Min Limit
		sw[i][6][0] = (0x02)    //Max Limit
		sw[i][7][0] = (0x02)    //epsilon
	endfor

	source[length][0] = "Offset at E = 0 eV"
	source[length + 1][0] = "Slope"
	source[length + 2][0] = "Parabola"
	source[length + 3][0] = "Pseudo Tougaard (Herrera-Gomez)"
	source[length + 4][0] = "Shirley Step Height"
	source[length + 5][0] = "Area   -------------------------- Peak " + num2str(numpeaks+1)
	source[length + 6][0] = "Position"
	source[length + 7][0] = "Width"
	source[length + 8][0] = "Gauss-Lorentz Ratio"
	source[length + 9][0] = "Asymmetry"
	source[length + 10][0] = "Asymmetry Translation"

	source[length][1] = "off" 
	source[length + 1][1] = "sl"
	source[length + 2][1] = "prb" 
	source[length + 3][1] = "tgd" 
	source[length + 4][1] = "srl"
	source[length + 5][1] = "a" + num2str(numpeaks+1)
	source[length + 6][1] = "p" + num2str(numpeaks+1)
	source[length + 7][1] = "w" + num2str(numpeaks+1)
	source[length + 8][1] = "g" + num2str(numpeaks+1)
	source[length + 9][1] = "as" + num2str(numpeaks+1)
	source[length + 10][1] = "at" + num2str(numpeaks+1)
	
	source[length][4] = "off" 
	source[length + 1][4] = "sl"
	source[length + 2][4] = "prb" 
	source[length + 3][4] = "tgd" 
	source[length + 4][4] = "srl"
	source[length + 5][4] = "a" + num2str(numpeaks+1)
	source[length + 6][4] = "p" + num2str(numpeaks+1)
	source[length + 7][4] = "w" + num2str(numpeaks+1)
	source[length + 8][4] = "g" + num2str(numpeaks+1)
	source[length + 9][4] = "as" + num2str(numpeaks+1)
	source[length + 10][4] = "at" + num2str(numpeaks+1)
	

	EstimatedPeakArea = EstimatePeakArea( heightA , Width_Start, 5 * GLratio_Min, 5 * Asym_Min , 5 * Asym_Shift_Min)
	source[length][3] = num2str(max(0,V_min)) 
	source[length + 1][3] = "0"
	source[length + 2][3] = "0" 
	source[length + 3][3] = "0" 
	source[length + 4][3] = num2str(0.1*abs(heightA-V_min))
	source[length + 5][3] = num2str(EstimatedPeakArea)
	source[length + 6][3] = num2str(posA)
	source[length + 7][3] = num2str(Width_Start)
	source[length + 8][3] = num2str(GLratio_Start)
	source[length + 9][3] = num2str(Asym_Start)
	source[length + 10][3] = num2str(Asym_Shift_Start)

	
	sw[length +2][4][0] = 48
	sw[length +3][4][0] = 48   //check checkboxes
	sw[length +8][4][0] = 48
	sw[length +9][4][0] = 48
	sw[length +10][4][0] = 48
	
	sw[length+2][5][0] = 0   
	sw[length+3][6][0] = 0    
	
	
	source[length][5] = "-500000" 
	source[length + 1][5] = "-200"
	source[length + 2][5] = "-100" 
	source[length + 3][5] = "-1000" 
	source[length + 4][5] = "0.01"
	source[length + 5][5] = num2str(min(10,0.1 * EstimatedPeakArea ))  //this is the first peak
	source[length + 6][5] = num2str(posA-1.5)//num2str(right)
	source[length + 7][5] = num2str(Width_Min)
	source[length + 8][5] =  num2str(GLratio_Min)
	source[length + 9][5] = num2str(Asym_Min)
	source[length + 10][5] = num2str(Asym_Shift_Min)

	
	source[length][6] = "500000" 
	source[length + 1][6] = "200"
	source[length + 2][6] = "100" 
	source[length + 3][6] = "1000" 
	source[length + 4][6] = num2str(0.7*abs(V_min-heightA))
	source[length + 5][6] =  num2str(max(10,30 * EstimatedPeakArea ))
	source[length + 6][6] =  num2str(posA+1.5)//num2str(left )
	source[length + 7][6] = num2str(Width_Max )
	source[length + 8][6] = num2str(GLratio_Max)
	source[length + 9][6] = num2str(Asym_Max)
	source[length + 10][6] = num2str(Asym_Shift_Max)

	source[length][7] = "1e-9" 
	source[length + 1][7] = "1e-9"
	source[length + 2][7] = "1e-9" 
	source[length + 3][7] = "1e-9" 
	source[length + 4][7] = "1e-9"
	source[length + 5][7] = "1e-8"
	source[length + 6][7] = "1e-9"
	source[length + 7][7] = "1e-9"
	source[length + 8][7] = "1e-9" 
	source[length + 9][7] = "1e-9"
	source[length + 10][7] = "1e-9"
else
	//now, linking can come into the game ... it will affect the columns 3,4,5,6
	
	Redimension /n=(length+6,-1) source
	Redimension /n=(length+6,-1) sw	
	
	newLength = length+ 6
	
	numpeaks = floor((length-5)/6)
	for ( i= length; i<newLength; i+=1)
		sw[i][0][0] = 0                //legende
		sw[i][1][0] = 0           //coef kuerzel
		sw[i][2][0] = 0           //endergebnis
		sw[i][3][0] = (0x02)   //anfangswerte
		sw[i][4][0] = (0x20)    //hold
		sw[i][5][0] = (0x02)    //Min Limit
		sw[i][6][0] = (0x02)    //Max Limit
		sw[i][7][0] = (0x02)    //epsilon
	endfor
	
	source[length][0] = "Area   ----------------------------- Peak " + num2str(numpeaks+1)
	source[length + 1][0] = "Position"
	source[length + 2][0] = "Width"
	source[length + 3][0] = "Gauss-Lorentz Ratio"
	source[length + 4][0] = "Asymmetry"
	source[length + 5][0] = "Asymmetry Translation"

	source[length][1] = "a" + num2str(numpeaks+1)
	source[length + 1][1] = "p" + num2str(numpeaks+1)
	source[length + 2][1] = "w" + num2str(numpeaks+1)
	source[length + 3][1] = "g" + num2str(numpeaks+1)
	source[length + 4][1] = "as" + num2str(numpeaks+1)
	source[length + 5][1] = "at" + num2str(numpeaks+1)


     // start: take care of linking
     if (peakToLink == 0)
		EstimatedPeakArea = EstimatePeakArea( heightA , Width_Start, 5 * GLratio_Min, 5 * Asym_Min , 5 * Asym_Shift_Min)
		source[length][3] = num2str(EstimatedPeakArea)
		source[length + 1][3] = num2str(posA)
		source[length + 2][3] = num2str(Width_Start)
		source[length + 3][3] = num2str(GLratio_Start)
		source[length + 4][3] = num2str(Asym_Start)
		source[length + 5][3] = num2str(Asym_Shift_Start)
	
		sw[length +3][4][0] = 48
		sw[length +4][4][0] = 48
		sw[length +5][4][0] = 48   //check checkboxes

	
		source[length][5] = num2str(min(10,0.1 * EstimatedPeakArea ))  //this is the first peak
		source[length + 1][5] = num2str(posA-1.5)// num2str(right)
		source[length + 2][5] = num2str(Width_Min)
		source[length + 3][5] =  num2str(GLratio_Min)
		source[length + 4][5] = num2str(Asym_Min)
		source[length + 5][5] = num2str(Asym_Shift_Min)
	

		source[length ][6] =  num2str(max(10,30 * EstimatedPeakArea ))
		source[length + 1][6] =  num2str(posA+1.5)//num2str(left )
		source[length + 2][6] = num2str(Width_Max )
		source[length + 3][6] = num2str(GLratio_Max)
		source[length + 4][6] = num2str(Asym_Max)
		source[length + 5][6] = num2str(Asym_Shift_Max)
	else
		//get the startingIndex of the target peak
		variable startIndexParentPeak = 6 * (peakToLink -1 ) + 5		
	
		if ( linkArea == 0 )
			EstimatedPeakArea = EstimatePeakArea( heightA , Width_Start, 5 * GLratio_Min, 5 * Asym_Min , 5 * Asym_Shift_Min)
			source[length][3] = num2str(EstimatedPeakArea)
			sw[length][4][0] = 32
			source[length][5] = num2str(min(10,0.2 * EstimatedPeakArea ))  
			source[length ][6] =  num2str(max(10,30 * EstimatedPeakArea ))
		else
			
			source[length][3] = num2str( areaLinkLowerFactor * str2num(source[startIndexParentPeak][3]) )    //start at the lower boundary
			sw[length][4][0] = sw[startIndexParentPeak][4][0]
			source[length][5] = num2str(areaLinkLowerFactor - 0.001) + " * " + StringFromList(0,parameterList) + num2str(peakToLink)
			source[length ][6] = num2str(areaLinkUpperFactor + 0.001) + " * " + StringFromList(0,parameterList) + num2str(peakToLink)
		endif
		
		if ( linkPosition == 0 )
			source[length + 1][3] = num2str(posA)
			sw[length + 1][4][0] = 32
			source[length + 1][5] = num2str(posA-1.5) //num2str(right)
			source[length + 1][6] =  num2str(posA+1.5)//num2str(left )
		else
			source[length + 1][3] = num2str( str2num( source[startIndexParentPeak + 1][3] ) + positionLinkOffsetMin )
			sw[length + 1][4][0] = sw[startIndexParentPeak +1][4][0]
			source[length + 1][5] = StringFromList(1,parameterList) + num2str(peakToLink) + " + " + num2str(positionLinkOffsetMin-0.01)
			source[length + 1][6] = StringFromList(1,parameterList) + num2str(peakToLink) + " + " + num2str(positionLinkOffsetMax + 0.01)
		endif
		
		if ( linkWidth == 0 )
			source[length + 2][3] = num2str(Width_Start)
			sw[length + 2][4][0] = 32
			source[length + 2][5] = num2str(Width_Min)
			source[length + 2][6] = num2str(Width_Max )
		else
			source[length + 2][3] = source[startIndexParentPeak + 2][3]
			sw[length + 2][4][0] = sw[startIndexParentPeak +2][4][0]
			source[length + 2][5] = StringFromList(2,parameterList) + num2str(peakToLink)
			source[length + 2][6] = StringFromList(2,parameterList) + num2str(peakToLink)
		endif
		
		if ( linkGL == 0 )
			source[length + 3][3] = num2str(GLratio_Start)
			sw[length + 2][4][0] = 32
			source[length + 3][5] =  num2str(GLratio_Min)
			source[length + 3][6] = num2str(GLratio_Max)
		else
			source[length + 3][3] = source[startIndexParentPeak + 3][3]
			sw[length + 3][4][0] = sw[startIndexParentPeak +3][4][0]
			source[length + 3][5] =  StringFromList(3,parameterList) + num2str(peakToLink)
			source[length + 3][6] = StringFromList(3,parameterList) + num2str(peakToLink)
		endif
		
		if ( linkAsym == 0 )
			source[length + 4][3] = num2str(Asym_Start)
			source[length + 5][3] = num2str(Asym_Shift_Start)
			sw[length +3][4][0] = 48
			sw[length +4][4][0] =  48
			sw[length +5][4][0] =  48   //check checkboxes
			source[length + 4][5] = num2str(Asym_Min)
			source[length + 5][5] = num2str(Asym_Shift_Min)
			source[length + 4][6] = num2str(Asym_Min)
			source[length + 5][6] = num2str(Asym_Shift_Max)
		else
			source[length + 4][3] = source[startIndexParentPeak + 4][3]
			source[length + 5][3] = source[startIndexParentPeak + 5][3]
			sw[length +3][4][0] = sw[startIndexParentPeak +3][4][0]
			sw[length +4][4][0] = sw[startIndexParentPeak +4][4][0]
			sw[length +5][4][0] = sw[startIndexParentPeak +5][4][0]
			source[length + 4][5] = StringFromList(4,parameterList) + num2str(peakToLink)
			source[length + 5][5] = StringFromList(5,parameterList) + num2str(peakToLink)
			source[length + 4][6] = StringFromList(4,parameterList) + num2str(peakToLink)
			source[length + 5][6] = StringFromList(5,parameterList) + num2str(peakToLink)
		endif
	endif
	// stop: take care of linking
	
	
	source[length][4] = "a" + num2str(numpeaks+1) 
	source[length + 1][4] = "p" + num2str(numpeaks+1)
	source[length + 2][4] = "w" + num2str(numpeaks+1)
	source[length + 3][4] = "g" + num2str(numpeaks+1) 
	source[length + 4][4] = "as" + num2str(numpeaks+1) 
	source[length + 5][4] = "at" + num2str(numpeaks+1) 
	
	
	
	source[length][7] = "1e-8"
	source[length + 1][7] = "1e-9"
	source[length + 2][7] = "1e-9"
	source[length + 3][7] = "1e-9" 
	source[length + 4][7] = "1e-9"
	source[length + 5][7] = "1e-9"
endif

	Redimension /n=(NumLength+4,-1) numerics
	Redimension /n=(NumLength+4,-1) selNumerics
	
	numerics[NumLength][0] = "Peak " + num2str(numpeaks+1)
	numerics[NumLength + 1][1] = "Area"
	//numerics[NumLength +2 ][1] = "Visible Area"
	//numerics[NumLength +3 ][1] = "Analytical Area"
	
	numerics[NumLength + 1][3] = "Position (Coef.)"
	numerics[NumLength + 2][3] = "Effective Position"
	
	numerics[NumLength + 1][5] = "Width (Coef.)"
	numerics[NumLength + 2][5] = "Effective Width"

	numerics[NumLength + 1][7] = "Gauss-Lorentz Ratio"
	
	numerics[NumLength + 1][9] = "Asymmetry (coef)"
	numerics[NumLength + 2][9] = "Effective Asymmetry:"
	numerics[NumLength + 3][9] = "1 - (fwhm_right)/(fwhm_left):"
	 
	 numerics[NumLength + 1][11] = "Asymmetry translation (coef)"
	
	
	FancyUp("foe")
	setup2Waves()	
end

/// 2  ////////////////////////////////////////////////////////////
///////////////   Display it in the peak fitting window ///////////////////////////////////////////////

function PlotPseudoVoigtSKDisplay(peakType,RawYWave, RawXWave,coefWave)
	string peakType
	string RawYWave
	string RawXWave
	string coefWave
	string TagName    // the Tag in the result window
	string PeakTag     // text in this tag
	string PkName, parentDataFolder //, cleanUpString=""		
	string BGName //background
	string PeakSumName
	NVAR FitMin = root:STFitAssVar:STFitMin
	NVAR FitMax = root:STFitAssVar:STFitMax
	
	wave cwave = $coefWave
	wave raw = $RawYWave
//	wave xraw = $RawXWave
	variable LenCoefWave = DimSize(cwave,0)
	
	//create some waves, to display the peak
	variable nPeaks = 0
	variable numCoef
	variable i,index,k
	variable xmin, xmax, step
	variable TagPosition   //the position of the tag in the result window
	variable totalPeakSumArea, partialPeakSumArea
	 
	strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
		case "":                                             //if empty
			duplicate /o /R=(FitMin,FitMax) raw WorkingDuplicate  		                     
			break
		case "_calculated_":
			 duplicate /o /R=(FitMin,FitMax) raw WorkingDuplicate  
			break
		default:                                                 // if not empty, x-axis wave neccessary
			//read in the start x-value and the step size from the x-axis wave
			wave xraw = $RawXWave
			xmax = max(xraw[0],xraw[numpnts(xraw)-1] )
			xmin = min(xraw[0],xraw[numpnts(xraw)-1] )
			step = (xmax - xmin ) / DimSize(xraw,0)
			// now change the scaling of the y-wave duplicate, so it gets equivalent to a data-wave imported from an igor-text file
			duplicate /o raw tempWaveForCutting  
			SetScale /I x, xmin, xmax, tempWaveForCutting  //OKAY, NOW THE SCALING IS ON THE ENTIRE RANGE
			duplicate /o /R=(FitMin,FitMax) tempWaveForCutting WorkingDuplicate  
			killwaves /z tempWaveForCutting
			//wave xraw = $RawXWave
			//xmax = rightx(xraw)
			//step = (xraw[0] - xraw[xmax] ) / DimSize(xraw,0)
			//// now change the scaling of the y-wave duplicate, so it gets equivalent to a data-wave imported from an igor-text file
			//SetScale /P x, xraw[xmax], step, WorkingDuplicate
			break
	endswitch
	
	parentDataFolder = GetDataFolder(1)
	
	
	//now make tabular rasa in the case of background functions
	string ListOfCurves = TraceNameList("CursorPanel#guiCursorDisplayFit",";",1)
	variable numberCurves = ItemsInList(ListOfCurves)
	//remove only the very last curve, if there are e.g. 3 curves on the graph it has the index #2
	
	// If a wave is given which needs an external x-axis (from an ASCII-file) create a duplicate which receives a proper x-scaling later on
	// the original wave will not be changed
	KillDataFolder /z :Peaks  //if it exists from a previous run, kill it
	//now recreate it, so everything is updated             
	NewDataFolder /O /S :Peaks

 	numCoef = 6   //Voigt with Shirley and Slope 
	nPeaks = (LenCoefWave-5)/numCoef
	
	PeakSumName = "pS_"+RawYWave
			
	duplicate /o WorkingDuplicate $PeakSumName
	wave tempSumDisplay = $PeakSumName
			
			
	//update the graph, remove everything	
	for (i =1; i<numberCurves; i +=1)
		RemoveFromGraph /W=CursorPanel#guiCursorDisplayFit /Z $"#"+num2istr(numberCurves-i)
	endfor
	
	tempSumDisplay = 0
	variable areaPeak=0
	for (i =0; i<nPeaks;i+=1)
		index = numCoef*i + 5
		PkName = "p" + num2istr(i+1) + "_" + RawYWave  //make a propper name
	 	//create a wave with this name and the correct  scaling and number of datapoints -> copy the fitted wave
		duplicate /o WorkingDuplicate $PkName	

		wave tempDisplay = $PkName                                            

		 //overwrite the original values in the wave with the values of a single peak
		tempDisplay = CalcSingleVoigtGLS(1,cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5],x)	
		areaPeak= IntegrateSingleVoigtGLS(1,cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5])
		tempDisplay /= areaPeak
		tempDisplay *= cwave[index]
		
		tempSumDisplay += tempDisplay
		RemoveFromGraph /W=CursorPanel#guiCursorDisplayFit /Z  $PkName#0
		AppendToGraph /w= CursorPanel#guiCursorDisplayFit tempDisplay                           //now plot it
		
		WaveStats /Q tempDisplay
		tagName = PkName+num2istr(i)
		PeakTag = num2istr(i+1)
		TagPosition = V_maxloc
		
		Tag /w= CursorPanel#guiCursorDisplayFit /C /N= $tagName  /F=0 /L=1  /Y =2.0  $PkName, TagPosition ,PeakTag
		ModifyGraph /w= CursorPanel#guiCursorDisplayFit rgb($PkName)=(0,0,0)       // and color-code it	
	endfor
	//get the sum of all peaks and (i) calculate the Shirley, (ii) calculate the line offset and (iii) display the background and the individual sums of peak + Background
	BGName ="bg_"+ RawYWave ///should name the background accordingly
	
	duplicate /o WorkingDuplicate $BGName
	wave tempBGDisplay = $BGName //this is the wave to keep the background
	
	duplicate /o WorkingDuplicate HGB
	wave hgb = HGB
	
	// now calculate the background with tempSumDisplay
	totalPeakSumArea = sum(tempSumDisplay)
	//print totalPeakSumArea
	partialPeakSumArea = 0
	if (pnt2x(WorkingDuplicate,0) < pnt2x(WorkingDuplicate,1))    //x decreases with index
		for ( i = 0; i < numpnts(tempSumDisplay); i+=1)
			partialPeakSumArea += tempSumDisplay[i]
			tempBGDisplay[i] =partialPeakSumArea/totalPeakSumArea 
		endfor
	else //x increases with index
		for ( i = 0; i < numpnts(tempSumDisplay); i+=1)
			partialPeakSumArea += tempSumDisplay[numpnts(tempSumDisplay) -1 - i]
			tempBGDisplay[numpnts(tempSumDisplay) -1 - i] = partialPeakSumArea/totalPeakSumArea 
		endfor
	endif
			
	//now add the Herrera-Gomez background
	partialPeakSumArea = 0
	totalPeakSumArea = sum(tempBGDisplay)
	if (pnt2x(WorkingDuplicate,0) < pnt2x(WorkingDuplicate,1))   //binding energy increases with point index
		for ( i = 0; i < numpnts(tempSumDisplay); i += 1)
			partialPeakSumArea += abs(tempBGDisplay[i])
			hgb[i] = partialPeakSumArea/totalPeakSumArea	
		endfor
	else                     //binding energy decreaeses with point index
		for ( i = 0; i < numpnts(tempSumDisplay); i += 1)
			partialPeakSumArea += abs(tempBGDisplay[numpnts(tempSumDisplay)-1-i])
			hgb[numpnts(tempSumDisplay)-1-i] = partialPeakSumArea/totalPeakSumArea	
		endfor
	endif
	hgb *= cwave[3]	
			
	tempBGDisplay *= cwave[4]  //shirley height
	tempBGDisplay += hgb
//	Killwaves /z temporaryShirleyWave
	
//	for (i =0; i<nPeaks;i+=1)
//		index = numCoef*i + 5
//		tempBGDisplay += 1e-3*cwave[3]*cwave[index]*( x - cwave[index+1] )^2 * ( x > cwave[index+1] ) 
//	endfor
			
	tempBGDisplay += cwave[0] + cwave[1]*x + cwave[2]*x^2
	
	AppendToGraph /w= CursorPanel#guiCursorDisplayFit tempBGDisplay 
		
	//now add the background to all peaks
	for (i =0; i<nPeaks;i+=1)
		index = numCoef*i
		PkName = "p" + num2istr(i+1) + "_"+RawYWave   //make a propper name
		wave tempDisplay = $PkName        //This needs some explanation, see commentary at the end of the file                                        
		
		 //overwrite the original values in the wave with the values of a single peak
		tempDisplay  += tempBGDisplay
	endfor
		
	tempSumDisplay += tempBGDisplay
	//for now, don't use tempSumDisplay, however, leave it in the code for possible future use
	killwaves /z tempSumDisplay   //remove this line, if the sum of the peaks is going to be used again
	killwaves /z HGB
	WaveStats /Q WorkingDuplicate
//	SetAxis /w = CursorPanel#guiCursorDisplayFit left -0.1*V_max, 1.1*V_max
	ModifyGraph /w= CursorPanel#guiCursorDisplayFit zero(left)=2 
	SetAxis/A/R /w = CursorPanel#guiCursorDisplayFit bottom
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit mirror(left)=2
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit mirror(bottom)=2
	Label  /w = CursorPanel#guiCursorDisplayFit Bottom "\\f01 binding energy (eV)"
	ModifyGraph  /w = CursorPanel#guiCursorDisplayFit minor(bottom)=1,sep(bottom)=2
	SetDataFolder parentDataFolder 
	killwaves /Z WorkingDuplicate
end




///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////

function DrawAndEvaluatePseudoVoigtSK(dataWave,fitWave,peakType,RawXWave,newFolder)
string dataWave
string fitWave
string peakType
string RawXWave
variable newFolder               //if this value is different from 1, no folder for the results will be created
SVAR projectName = root:STFitAssVar:ProjectName


wave cwave = W_coef
wave origWave = $dataWave
wave fitted = $fitWave
wave epsilon = epsilon
wave hold = hold
wave InitializeCoef = InitializeCoef
wave Min_Limit = Min_Limit
wave Max_Limit = Max_Limit
wave T_Constraints = T_Constraints
wave  CoefLegend = CoefLegend

if ( strlen(fitWave) >= 30)	
	doalert 0, "The name of the fit-wave is too long! Please shorten the names."
	return -1
endif


//define further local variables
variable LenCoefWave = DimSize(cwave,0)	
variable nPeaks
variable index
variable i =0                               //general counting variable
variable numCoef                       //variable to keep the number of coefficients of the selected peak type
							  // numCoef = 3   for Gauss Singlet     and numCoef =5 for VoigtGLS
variable pointLength, totalArea, partialArea
variable peakMax 
variable TagPosition
variable AnalyticalArea
variable EffectiveFWHM
variable GeneralAsymmetry        //  = 1 - (fwhm_right)/(fwhm_left)

string PkName                          //string to keep the name of a single peak wave
string foldername                       //string to keep the name of the datafolder, which is created later on for the single peak waves
string tempFoldername               //help-string to avoid naming conflicts
string parentDataFolder
string TagName
string PeakTag
string LastGraphName
string NotebookName = "Report"     //this is the initial notebook name, it is changed afterwards
string tempNotebookName
string tempString                          // for a formated output to the notebook
string BGName
//The following switch construct is necessary in order to plot waveform data (usually from igor-text files , *.itx) as well as
//raw spectra which need an extra x-axis (such data come usually from an x-y ASCII file)

strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
	case "":                                             //if empty
		display /K=1 origWave                 //the raw-data wave is in true waveform and has an intrinsic scale                     
		break
	case "_calculated_":
		display /K=1 origWave
		break
	default:
		wave xraw = $RawXWave                                                 // if not empty
		display /K=1 origWave vs xraw        // the raw-data wave has no intrinsic scale and needs a propper x-axis 
		break
endswitch
ModifyGraph mode($dataWave)=3,msize($dataWave)=1.3, marker($dataWave)=8
ModifyGraph mrkThick($dataWave)=0.7
ModifyGraph rgb($dataWave)=(0,0,0)           //color-code it
    
LastGraphName = WinList("*", "", "WIN:")    //get the name of the graph

//check if this Notebook already exists
V_Flag = 0
DoWindow $NotebookName   
// if yes, construct a new name
if (V_Flag)
	i = 1
	do 
		tempNoteBookName = NotebookName + num2istr(i)
		DoWindow $tempNotebookName
		i += 1
	while (V_Flag)
	NotebookName = tempNotebookName 
endif
//if not, just proceed

NewNotebook /F=1 /K=1 /N=$NotebookName      //make a new notebook to hold the fit report
Notebook $NoteBookName ,fsize=8
//Notebook $NoteBookName ,text="\r\r \t\t --- if necessary, insert plot by copy and paste ----    "
Notebook $NoteBookName ,text="\r\r\rPeak Shape:    "+ peakType

//prepare a new datafolder for the fitting results, in particular the single peaks
parentDataFolder = GetDataFolder(1)    //get the name of the current data folder

if (newFolder == 1)
	NewDataFolder /O /S subfolder
	//now, this folder is the actual data folder, all writing is done here and not in root
endif

duplicate /o fitted tempFitWave
wave locRefTempFit = tempFitWave
locRefTempFit = 0

//strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
//	case "":                                             //if empty
//		 sprintf  tempString, "\r\rIntegral area of the raw spectrum (wave area): %20.2f " ,  area(origWave)           
//		break
//	case "_calculated_":
//		sprintf tempString, "\r\rIntegral area of the raw spectrum (wave area): %20.2f " ,  area(origWave)
//		break
//	default:                                                 // if not empty
//		sprintf tempString, "\r\rIntegral area of the raw spectrum (wave area): %20.2f " ,  areaXY(xraw,origWave)
//		break
//endswitch
//Notebook $NoteBookName ,text=tempString

//take the fit-result and analyze the maximum, get the maximum signal, so a significance threshold can be calculated
//WaveStats /Q fitted
//peakMax = V_max

//now decompose the fit into single peaks --- if a further fit function is added, a further "case" has to be attached

		numCoef = 6
		nPeaks = (LenCoefWave-5)/numCoef         //get the number of  peaks from the output wave of the fit
		//check, if the peak type matches the length of the coefficient wave
		//if not so, clean up, inform and exit
		BGName = "PS_bg" +"_" + dataWave 
		duplicate /o fitted $BGName
		wave background = $BGName
		
		duplicate /o fitted HGB
		wave hgb = HGB
		AppendToGraph background
		
		if (mod(LenCoefWave-5,numCoef) != 0)
			DoAlert 0, "Mismatch, probably wrong peak type selected or wrong coefficient file, check your fit and peak type "
			SetDataFolder parentDataFolder 
			KillDataFolder  /Z subfolder
			print " ******* Peak type mismatch - check your fit and peak type ******"
			return 1
		endif 
		
		Notebook $NoteBookName ,text="\r\r\r"
		
		variable areaPeak = 0
		variable sumCoef = 0
		for (i =0; i<nPeaks;i+=1)
			index = numCoef*i + 5
			PkName = "p" + num2istr(i+1)+"_" + dataWave    //make a proper name
			 //create a wave with this name and the correct  scaling and number of datapoints -> copy the fitted wave
			duplicate /o fitted $PkName
							
			wave W = $PkName        //This needs some explanation, see commentary at the end of the file                                        
			 
			 //overwrite the original values in the wave with the values of a single peak
			W =  CalcSingleVoigtGLS(1,cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5],x)
			areaPeak= IntegrateSingleVoigtGLS(1,cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5])
			W /= areaPeak
			W *= cwave[index]
			sumCoef += cwave[index]
			locRefTempFit += W         
			
			AppendToGraph W                                                    //now plot it

			//append the peak-tags to the graph, let the arrow point to a maximum
			//append the peak-tags to the graph, let the arrow point to a maximum
			WaveStats /Q W                             // get the location of the maximum
			TagName = "tag"+num2istr(i)           //each tag has to have a name
			PeakTag = num2istr(i+1)                 // The tag displays the peak index
			TagPosition = V_maxloc                 // and is located at the maximum
			Tag  /C /N= $TagName  /F=0 /L=1  /Y =2.0  $PkName, TagPosition ,PeakTag    // Now put the tag there
			
			ModifyGraph rgb($PkName)=(10464,10464,10464)              // color code the peak
				
			//AnalyticalArea =  IntegrateSingleVoigtGLS(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5])
			
			EffectiveFWHM = CalcFWHMSingleVoigtGLS(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5])
			GeneralAsymmetry = CalcGeneralAsymmetry(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5])   
			
			//sprintf  tempString, "\r\r %1g	Area                                   |  Position |  FWHM   | GL-ratio  |   Asym.   | Asym. S. |\r",(i+1)
			sprintf  tempString, "\r Peak %1g	  Area\t|\tPosition\t|\tFWHM\t|\tGL-ratio\t|\tAsym.\t|\tAsym. Shift\t\r",(i+1)
			Notebook $NoteBookName ,text=tempString
			sprintf  tempString, "\t%s\t|\t%8.2f\t|\t%8.2f\t|\t%8.2f\t|\t%8.2f\t|\t%8.2f\t\r" , STnum2str(cwave[index]), cwave[index+1] ,cwave[index+2] ,cwave[index+3] , cwave[index+4],cwave[index+5]
			Notebook $NoteBookName ,text=tempString
			sprintf  tempString, "\rEffective maximum position\t\t\t\t%8.2f \r", V_maxloc  // "-> In case of asymmetry, this value does not represent an area any more"
			Notebook $NoteBookName ,text=tempString
			sprintf  tempString, "Effective FWHM\t\t\t\t\t%8.2f \r"	EffectiveFWHM	
			Notebook $NoteBookName ,text=tempString
			sprintf tempString, "Effective Asymmetry = 1 - (fwhm_right)/(fwhm_left)\t\t%8.2f \r\r\r\r" ,GeneralAsymmetry
			Notebook $NoteBookName ,text=tempString	
		endfor
		
		sprintf  tempString, "Total area of all peaks\r-------------------------------------\r  %s  \t\t(sum of fit coefficients - usually larger than visible area within measurement window) \r\r",  STnum2str(sumCoef)
		Notebook $NoteBookName ,text=tempString
		
		
		
		//and now, add the background
		pointLength = numpnts(locRefTempFit)
		totalArea = sum(locRefTempFit)
		partialArea = 0
		
		//distinguish between ascending and descending order of the points in the raw-data wave
		if (pnt2x(locRefTempFit,0) > pnt2x(locRefTempFit,1))   //with increasing index, x decreases
			for (i=pointLength-1; i ==0; i -=1)	
				partialArea += abs(locRefTempFit[i]) 
		
				background[i] = partialArea/totalArea

			endfor
			//now add the Herrera-Gomez background
			partialArea = 0
			totalArea = sum(background)
			for ( i = pointLength; i == 0; i -= 1)
				partialArea += abs(background[i])
				hgb[i] = partialArea/totalArea	
			endfor
			hgb *= cwave[3]
			background *= cwave[4]
			background += hgb
			//for (i =0; i<nPeaks;i+=1)
			//	index = numCoef*i + 5
		//		background += 1e-3*cwave[3]*cwave[index] * ( x - cwave[index + 1])^2 * ( x > cwave[index+1])
		//	endfor
			background += cwave[0] + cwave[1]*x + cwave[2]*x^2
		else
			for (i=0; i<pointLength; i += 1)
					partialArea += abs(locRefTempFit[i]) 
					background[i] =partialArea/totalArea 
			endfor
				//now add the Herrera-Gomez background
			partialArea = 0
			totalArea = sum(background)
			for ( i = 0; i < pointLength; i += 1)
				partialArea += abs(background[i])
				hgb[i] = partialArea/totalArea	
			endfor
			hgb *= cwave[3]
			
			
			background *= cwave[4]
			background += hgb
			background += cwave[0] + cwave[1]*x + cwave[2]*x^2
			
		endif
		//now, everything should be fine with the background .... carry on
		
		locRefTempFit += background
		
		
		tempString = "\r\r\rFurther Details\r"
		Notebook $NoteBookName ,text=tempString
		tempString = "==================================================================================================="
		Notebook $NoteBookName ,text=tempString
		strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
			case "":                                             //if empty
				 sprintf  tempString, "\r\rArea of the raw spectrum:\t\t\t\t%s \r" ,  STnum2str(area(origWave))           
			break
			case "_calculated_":
				sprintf tempString, "\r\rArea of the raw spectrum:\t\t\t\t%s \r" ,  STnum2str(area(origWave))
			break
			default:                                                 // if not empty
				sprintf tempString, "\r\rArea of the raw spectrum:\t\t\t\t%s \r" ,  STnum2str(areaXY(xraw,origWave))
				break
		endswitch
		Notebook $NoteBookName ,text=tempString
		
		sprintf  tempString, "Total area of all peaks in measurement window:\t\t%s \r" STnum2str(area(locRefTempFit) - area(background))
		Notebook $NoteBookName ,text=tempString
		sprintf  tempString, "Area of the background in measurement window:\t\t%s \r", STnum2str(area(background) ) 
		Notebook $NoteBookName ,text=tempString
		sprintf  tempString, "\rYou used a simultaneous fit of background  and signal: MAKE SURE the background shape makes sense.\r" 
		Notebook $NoteBookName ,text=tempString
		sprintf  tempString, "For details on the \r -- Peak shape:  Surface and Interface Analysis (2014), 46, 505 - 511  (If you use this program, please read and cite this paper) \r" 
		Notebook $NoteBookName ,text=tempString
		sprintf  tempString, " -- 'Pseudo-Tougaard' and Shirley contribution to the background:  J. Elec. Spectrosc. Rel. Phen (2013), 189, 76 - 80\r" 
		Notebook $NoteBookName ,text=tempString
		sprintf tempString,"\rPlease note that for asymmetric peaks the fit coefficients for position and FWHM are merely coefficients. \rIn this case, refer to the respective 'effective' values.\r"      
		Notebook $NoteBookName ,text=tempString
		
		sprintf  tempString, "\rBackground\r==================================\rThe background is calculated as follows:  Offset + a*x + b*x^2 + c * (pseudo-tougaard) + d * shirley\r" 
		Notebook $NoteBookName ,text=tempString
		sprintf  tempString, "\r(Here:  Offset=%8.2g;      a =%8.2g;      b =%8.2g;   c =%8.4g;   d =%8.2g; ) \r", cwave[0], cwave[1], cwave[2], cwave[3], cwave[4]
		Notebook $NoteBookName ,text=tempString
		
		sprintf  tempString, "\rThe parameters a and b  serve to cover general slopes or curvatures in the background, for example if the peak sits on a complex mixture of  broad Auger lines or plasmons.\r" 
		Notebook $NoteBookName ,text=tempString
		
		sprintf tempString, "\rPeak Areas\r==================================\rThe peak area is obtained by integrating the peak on an interval of +/- 90*FWHM around the peak position"
		Notebook $NoteBookName ,text=tempString
	//	sprintf  tempString,"\r'Fit wave area':     Peak area within the measured energy range"
	//	Notebook $NoteBookName ,text=tempString																			    
		sprintf tempString, "\rThis value is generally somewhat larger than the visible peak area within the limited measurement window."
		Notebook $NoteBookName ,text=tempString
		
		for (i =0; i<nPeaks;i+=1)
			PkName = "p" + num2istr(i+1)+"_" + dataWave    					
			wave W = $PkName       
			W += background			
		endfor
		

//killwaves /z  fitted     //this applies to the original fit-wave of Igor, since it is a reference to the wave root:Igor-FitWave, the original wave is possibly wrong
//duplicate /o locRefTempFit, $fitWave               //but we are still in the subfolder, 
//killwaves /z locRefTempFit
//wave fitted = $fitWave


	SetDataFolder parentDataFolder 

	//create a copy of the coefficient wave in the subfolder, so the waves 
	//and the complete fitting results are within that folder
	duplicate /o :$dataWave, :subfolder:$dataWave
	if (WaveExists($RawXWave))
		duplicate /o :$RawXWave, :subfolder:$RawXWave
	endif

	//duplicate  :$fitWave, :subfolder:$fitWave
//	killwaves /z :$fitWave            //probably fails, if the fit wave is displayed in the main panel as well
	duplicate /o $fitWave :subfolder:$fitWave
	AppendToGraph :subfolder:$fitWave                                //draw the complete fit
	ModifyGraph rgb($fitWave) = (0,0,0)       //color-code it
	//Remove the original wave, which is located in the parent directory and replace it by the copy in the subfolder
	RemoveFromGraph $"#0" 
	strswitch(RawXWave)                                //have a look at the string with the name of the x-axis wave
		case "":                                             //if empty
			AppendToGraph :subfolder:$dataWave                 //the raw-data wave is in true waveform and has an intrinsic scale                     
			break
		case "_calculated_":
			AppendToGraph  :subfolder:$dataWave
			break
		default:                                                 // if not empty
			AppendToGraph :subfolder:$dataWave vs :subfolder:$RawXWave        // the raw-data wave has no intrinsic scale and needs a propper x-axis 
		break
	endswitch

	foldername = "Report_" + dataWave
	
	//foldername ="Fit_"+ dataWave
	i=1   //index variable for the data folder	
	            //used also for the notebook
	V_Flag = 0
	DoWindow $folderName
	tempFoldername = foldername
	
	if (V_Flag)    //is there already a folder with this name
		do
			V_Flag = 0
			tempFoldername = foldername + "_" + num2istr(i)
			DoWindow $tempFolderName
			i+=1
		while(V_Flag)
		
		if ( strlen(tempFoldername) >= 30)	
			//doalert 0, "The output folder name is too long! Please shorten the names. The output folder of the current run is named 'subfolder'."
			string NewName = ""
			
			Prompt NewName, "The wave name is too long, please provide a shorter name "		// Set prompt for y param
			DoPrompt "Shorten the name", NewName
			if (V_Flag)
				return -1								// User canceled
			endif	
			tempFolderName = NewName	
		else
			RenameDataFolder subfolder, $tempFoldername                   //now rename the peak-folder accordingly
			Notebook  $NoteBookName, text="\r\r===================================================================================================\rXPST(2015)\t\t\t\t\tSurface and Interface Analysis (2014), 46, 505 - 511"
			//remove illegal characters from the string
			tempFoldername = stringClean(tempFoldername)
			DoWindow /C $tempFoldername
			DoWindow /F $LastGraphName
			DoWindow /C $tempFoldername + "_graph"
			//DoWindow /C /W=$LastGraphName $tempFoldername + "_graph"   //valid in Igor 6.x
		endif
		//TextBox/N=text0/A=LT tempFoldername        //prints into the graph
	else 
		//no datafolder of this name exist
		RenameDataFolder subfolder, $foldername 
		//TagWindow(foldername)
		Notebook  $NoteBookName, text="\r\r===================================================================================================\rXPST (2015)\t\t\t\t\tSurface and Interface Analysis (2014), 46, 505 - 511"
		//if ( strsearch(foldername,".",0) != -1 )      // strsearch returns -1 if  the string contains no "." 
		//	foldername = ReplaceString(".", foldername,"_")
		//endif
		foldername = stringClean(foldername)
		tempFoldername = stringClean(tempFoldername)
		DoWindow /C $foldername
		DoWindow /F $LastGraphName
		DoWindow /C $tempFoldername + "_graph"
	endif

//make the graph look good
ModifyGraph mode($dataWave)=3 ,msize($dataWave)=1.3 // ,marker($dataWave)=8, opaque($dataWave)=1
ModifyGraph opaque=1,marker($dataWave)=19
ModifyGraph rgb($dataWave)=(60928,60928,60928)
ModifyGraph useMrkStrokeRGB($dataWave)=1
ModifyGraph mrkStrokeRGB($dataWave)=(0,0,0)
//ModifyGraph mrkThick($dataWave)=0.7
//ModifyGraph rgb($dataWave)=(0,0,0)           //color-code it
ModifyGraph mirror=2,minor(bottom)=1
Label left "\\f01 intensity (counts)"
Label bottom "\\f01  binding energy (eV)"	
ModifyGraph width=255.118,height=157.465, standoff = 0, gfSize=11

//The following command works easily, but then the resulting graph is not displayed properly in the notebook
//SetAxis/A/R bottom
//instead do it like this:
variable left,right

strswitch(RawXWave)
	 	case "":
	 		 left = max(leftx($dataWave),rightx($dataWave))
	 		 right = min(leftx($dataWave),rightx($dataWave))
	 	break
	 	case "_calculated_":
	 		 left = max(leftx($dataWave),rightx($dataWave))
	 		 right = min(leftx($dataWave),rightx($dataWave))
	 	break
	 	default:
	 		waveStats /Q $RawXWave
	 		left = V_max
	 		right = V_min
	 	break
 endswitch
 SetAxis bottom left,right
//okay this is not perfectly elegant ... but

WaveStats /Q $dataWave
SetAxis left V_min-0.05*(V_max-V_min), 1.02*V_max
LastGraphName = WinList("*", "", "WIN:")

Notebook  $tempFoldername, selection = {startOfFile,startOfFile}

tempString = "\rReport for fitting project: \t\t " + projectName + "\r"
Notebook  $tempFoldername, text=tempString
Notebook  $tempFoldername, selection = {startOfPrevParagraph,endOfPrevParagraph}, fsize = 12, fstyle = 0

tempString = "\r==================================================================================================="
Notebook  $tempFoldername, selection = {startOfNextParagraph,endOfNextParagraph}, text=tempString
Notebook $tempFolderName ,selection = {startOfNextParagraph,startOfNextParagraph}, text="Saved in:\t\t\t\t\t" + IgorInfo(1) + ".pxp \r"
Notebook $tempFolderName ,text="Spectrum:\t\t\t\t" +dataWave
Notebook $tempFolderName ,text="\rApplied peak shape:\t\t\t"+ peakType +"\r\r"

Notebook  $tempFoldername, selection = {startOfNextParagraph,endOfNextParagraph}
Notebook  $tempFoldername, picture={$LastGraphName, 0, 1} , text="\r\r\r" 




//Notebook  $tempFoldername, selection = {startOfFile,startOfFile}
//tempString = "Fitting results for: " + tempFoldername + "\r"
//Notebook  $tempFoldername, text=tempString
//Notebook  $tempFoldername, selection = {startOfPrevParagraph,endOfPrevParagraph}, fsize = 12, fstyle = 0

//Notebook  $tempFoldername, selection = {startOfNextParagraph,startOfNextParagraph}
//Notebook  $tempFoldername, picture={$LastGraphName, 0, 1} , text="\r" 

killwaves /z hgb
KillWindow $LastGraphName
//Notebook  $tempFoldername, text="\r \r"  

KillDataFolder /z tempFoldername

//now clean up
killvariables  /Z V_chisq, V_numNaNs, V_numINFs, V_npnts, V_nterms,V_nheld,V_startRow, V_Rab, V_Pr
killvariables  /Z V_endRow, V_startCol, V_endCol, V_startLayer, V_endLayer, V_startChunk, V_endChunk, V_siga, V_sigb,V_q,VPr

end





function EvaluateSingletSK()

wave cwave = W_Coef /// hard-coded but what the hell....

wave /t output = Numerics    //here we write the results
variable AnalyticalArea = 0
variable EffectiveFWHM = 0
variable GeneralAsymmetry = 0

variable i,j,k,index, index2
variable numpeaks
variable numCoef = 6

variable lengthCoef = numpnts(cwave)
variable lengthNumerics = DimSize(output,0)
string parentFolder = ""
string item = ""
numpeaks = (lengthCoef-5)/6
variable EffectiveArea,EffectivePosition
string wavesInDataFolder = WaveList("*fit_*",";","DIMS:1")
wave fitted = $StringFromList(0,wavesInDataFolder)
variable areaW
string FormatString
variable areaPeak
for ( i = 0; i < numPeaks; i += 1 )
	index = 6*i + 5
	//this function also needs to analyze the waves to get the effective position etc.
	 //create a wave with this name and the correct  scaling and number of datapoints -> copy the fitted wave
	duplicate /o fitted tempForAnalysis
							
	wave W = tempForAnalysis
	 //overwrite the original values in the wave with the values of a single peak
	W =  CalcSingleVoigtGLS(1,cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5],x)
	areaPeak= IntegrateSingleVoigtGLS(1,cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5])
	W /= areaPeak
	W *= cwave[index]
	WaveStats/Q W
	areaW=area(W)
	//AnalyticalArea =  IntegrateSingleVoigtGLS(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5])
	//print AnalyticalArea
	EffectiveFWHM = CalcFWHMSingleVoigtGLS(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5])
	//print EffectiveFWHM
	GeneralAsymmetry = CalcGeneralAsymmetry(cwave[index],cwave[index+1],cwave[index+2],cwave[index+3],cwave[index+4],cwave[index+5])   
	//print GeneralAsymmetry
	//Now write the results to Numerics

	index2 = 4*i
	
	output[index2+1][2] = num2str(cwave[index])
	output[index2+2][2] = ""//num2str(areaW)
	output[index2+2][1] = ""
	output[index2+3][2] = ""//num2str(AnalyticalArea)
	output[index2+3][1] = ""
	
	output[index2+1][4] = num2str(cwave[index+1])
	output[index2+2][4] = num2str(V_maxloc)
	
	output[index2+1][6] = num2str(cwave[index+2])
	output[index2+2][6] = num2str(EffectiveFWHM)
	
	output[index2+1][8] = num2str(cwave[index+3])
	
	output[index2+1][10]=num2str(cwave[index+4])
	sprintf  FormatString, "%10.3 f " GeneralAsymmetry 
	output[index2+2][10] =FormatString

	output[index2+1][12] = num2str(cwave[index+5])
	
	killwaves /Z W
endfor

end