#pragma rtGlobals=1		// Use modern global access method.
#pragma hide=1

//////////////////////////////////////////////////////////////////////////////////
//Licence
//////////////////////////////////////////////////////////////////////////////////
//
//	If you use the 'Fit Assistant' to analyze data, please read and cite the following paper:
//	Surace and Interface Analysis, (2014), 46, 505-511
//	This publication contains more details on the fit function used by XPST.
//
//	XPST is a front-end to the IgorPro fitting engine. Although every possible effort was made 
//	to prevent serious bugs and wrong results, the author can not give any warranty for XPST as 
//	large parts actually depend on the underlying IgorPro code. 
//	Author: Dr. Martin Schmid
//
//////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////////////////////////////
//		Peak Integration
////////////////////////////////////////////////////////////////////////////////////////////////////////


Function GetAreaXY()
//place the cursos on the graph, make sure both are there!
print "Use this function if you have a plot of wave 'A' vs. wave 'B'!"

strswitch (CsrWave(A))
	case "":       //A not on grah
		strswitch (CsrWave(B) )
			case "":  // B missing as well
				DoAlert 0, "Please set cursor A and B on the curve to mark the range wich you want to cut. Use Ctrl+I to activate cursors..."
				return 1
			default:
				DoAlert 0, "You also have to use cursor A"
				return 1 // A is not there but B
		endswitch
	break
	default:       // A is on graph
		strswitch (CsrWave(B) )
			case "":  // B is missing
				DoAlert 0, "You have also to use cursor B"
				return 1
			default:
			break // A and B are there, proceed
		endswitch
	endswitch

string LastGraphName = WinList("*", "", "WIN:")
wave xvals = XWaveRefFromTrace(LastGraphName,CsrWave(A))
wave yVals = CsrWaveRef(A)



variable offset
// compensate for values below zero
if (yvals[pcsr(A)]<0 || yvals[pcsr(B)] < 0)
	offset = abs(min(yVals[pcsr(A)], yVals[pcsr(B)]))
	yvals += offset
endif

//get the area between the cursors
print "The area between", xvals[pcsr(A)], "and", xvals[pcsr(B)] , "with background is:"
variable areaRaw =abs(areaXY(xvals,yvals, xvals[pcsr(A)], xvals[pcsr(B)] ) )
print areaRaw


//subtract the base-area
print "The area without background is:"
variable baseline = abs(xvals[pcsr(A)] - xvals[pcsr(B)])

variable rectHeight = abs(min(yVals[pcsr(A)], yVals[pcsr(B)]))
variable areaBackground = baseline * rectHeight   // rectangular region
areaBackground += 0.5 * baseline * ( abs(max(yVals[pcsr(A)], yVals[pcsr(B)])) - abs(min(yVals[pcsr(A)], yVals[pcsr(B)])) )  // + triangle
variable areaCorrect = areaRaw - areaBackground

print areaCorrect

end

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
////////////////////////////

Function GetArea()
//place the cursos on the graph, make sure both are there!
strswitch (CsrWave(A))
	case "":       //A not on grah
		strswitch (CsrWave(B) )
			case "":  // B missing as well
				DoAlert 0, "Please set cursor A and B on the curve to mark the range wich you want to cut. Use Ctrl+I to activate cursors..."
				return 1
			default:
				DoAlert 0, "You also have to use cursor A"
				return 1 // A is not there but B
		endswitch
	break
	default:       // A is on graph
		strswitch (CsrWave(B) )
			case "":  // B is missing
				DoAlert 0, "You have also to use cursor B"
				return 1
			default:
			break // A and B are there, proceed
		endswitch
	endswitch

string LastGraphName = WinList("*", "", "WIN:")
//wave xvals = XWaveRefFromTrace(LastGraphName,CsrWave(A))
wave yVals = CsrWaveRef(A)
variable offset
// compensate for values below zero
if (yvals[xcsr(A)]<0 || yvals[xcsr(B)] < 0)
	offset = abs(min(yVals[xcsr(A)], yVals[xcsr(B)]))
	yvals += offset
endif

//get the area between the cursors
print "The area between", xcsr(A), "and", xcsr(B) , "with background is:"
variable areaRaw = abs(area(yvals, xcsr(A), xcsr(B) )) 
print areaRaw


//subtract the base-area
print "The area without background is:"
variable baseline = abs(xcsr(A) - xcsr(B))

variable rectHeight = abs(min(yVals[pcsr(A)], yVals[pcsr(B)]))
variable areaBackground = baseline * rectHeight   // rectangular region
areaBackground += 0.5 * baseline * ( abs(max(yVals[pcsr(A)], yVals[pcsr(B)])) - abs(min(yVals[pcsr(A)], yVals[pcsr(B)])) )  // + triangle
variable areaCorrect = areaRaw - areaBackground

print areaCorrect

end

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
////////////////////////////